=== modified file 'CMakeLists.txt'
--- CMakeLists.txt	2011-09-09 09:54:02 +0000
+++ CMakeLists.txt	2012-01-12 06:48:58 +0000
@@ -1,19 +1,44 @@
+#
+# Main CMakeLists.txt for compiz
+#
 project (compiz)
 
 set (CMAKE_MODULE_PATH_ORIG ${CMAKE_MODULE_PATH})
 set (COMPIZ_CMAKE_MODULE_PATH ${CMAKE_SOURCE_DIR}/cmake)
 set (CMAKE_MODULE_PATH ${CMAKE_MODULE_PATH} ${COMPIZ_CMAKE_MODULE_PATH})
+
+# Common compiz CMake macros & functions.
 include (CompizCommon)
+# Packaging-specific CMake macros & functions.
 include (CompizPackage)
 include (cmake/base.cmake)
+# Check if specific function is present in library.
 include (CheckFunctionExists)
-include (CTest)
-
-
-set (CMAKE_CONFIGURATION_TYPES "Debug;Release;RelWithDebInfo;MinSizeRe" CACHE INTERNAL "" FORCE)
-if (NOT CMAKE_BUILD_TYPE)
-    set (CMAKE_BUILD_TYPE "Debug" CACHE STRING "Build type (Debug/Release/RelWithDebInfo/MinSizeRe)" FORCE)
-endif (NOT CMAKE_BUILD_TYPE)
+
+enable_testing()
+
+# Import macro for doing coverage reporting
+include(EnableCoverageReport)
+
+#####################################################################
+# Enable code coverage calculation with gcov/gcovr/lcov
+# Usage:
+#  * Switch build type to coverage (use ccmake or cmake-gui)
+#  * Invoke make, make test, make coverage
+#  * Find html report in subdir coveragereport
+#  * Find xml report feasible for jenkins in coverage.xml
+#####################################################################
+IF(CMAKE_BUILD_TYPE MATCHES [cC][oO][vV][eE][rR][aA][gG][eE])
+  SET(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} -ftest-coverage -fprofile-arcs" )
+  SET(CMAKE_C_FLAGS "${CMAKE_C_FLAGS} -ftest-coverage -fprofile-arcs" )
+  SET(CMAKE_MODULE_LINKER_FLAGS "${CMAKE_MODULE_LINKER_FLAGS} -ftest-coverage -fprofile-arcs" )
+  SET(CMAKE_SHARED_LINKER_FLAGS "${CMAKE_SHARED_LINKER_FLAGS} -ftest-coverage -fprofile-arcs" )
+ENDIF(CMAKE_BUILD_TYPE MATCHES [cC][oO][vV][eE][rR][aA][gG][eE])
+
+#set (CMAKE_CONFIGURATION_TYPES "Debug;Release;RelWithDebInfo;MinSizeRe" CACHE INTERNAL "" FORCE)
+#if (NOT CMAKE_BUILD_TYPE)
+#    set (CMAKE_BUILD_TYPE "Debug" CACHE STRING "Build type (Debug/Release/RelWithDebInfo/MinSizeRe)" FORCE)
+#endif (NOT CMAKE_BUILD_TYPE)
 
 file (READ ${CMAKE_SOURCE_DIR}/VERSION COMPIZ_RELEASE_VERSION LIMIT 12 OFFSET 0)
 string (STRIP ${COMPIZ_RELEASE_VERSION} COMPIZ_RELEASE_VERSION)
@@ -41,6 +66,7 @@
 set (compiz_metadatadir ${datadir}/compiz)
 set (COMPIZ_I18N_DIR ${CMAKE_SOURCE_DIR}/po)
 
+# Translation
 set (ALL_LINGUAS af ar bg bn bn_IN bs ca cs cy da de el en_GB en_US es eu et fi fr gl gu he hi hr hu id it ja ka km ko lo lt mk mr nb nl or pa pl pt pt_BR ro ru sk sl sr sv ta tr uk vi xh zh_CN zh_TW zu)
 set (GETTEXT_PACKAGE compiz)
 
@@ -82,16 +108,6 @@
 compiz_configure_file (${compiz_SOURCE_DIR}/config.h.core.in ${compiz_BINARY_DIR}/generated/config.h)
 
 compiz_configure_file (
-    ${CMAKE_SOURCE_DIR}/include/compiz-common.h.in
-    ${CMAKE_BINARY_DIR}/generated/compiz-common.h
-)
-
-install (
-    FILES ${CMAKE_BINARY_DIR}/generated/compiz-common.h
-    DESTINATION ${COMPIZ_DESTDIR}${includedir}/compiz
-)
-
-compiz_configure_file (
     ${CMAKE_SOURCE_DIR}/compiz.pc.in
     ${CMAKE_BINARY_DIR}/compiz.pc
     COMPIZ_REQUIRES
@@ -103,6 +119,20 @@
     DESTINATION ${COMPIZ_DESTDIR}${libdir}/pkgconfig
 )
 
+# Build Google Test and make its headers known
+find_package (GTest REQUIRED)
+find_library (GMOCK_LIBRARY gmock)
+find_library (GMOCK_MAIN_LIBRARY gmock_main)
+
+if (NOT GMOCK_LIBRARY OR NOT GMOCK_MAIN_LIBRARY)
+  message (FATAL_ERROR "Google Mock not found!")
+endif (NOT GMOCK_LIBRARY OR NOT GMOCK_MAIN_LIBRARY)
+
+include_directories (${GTEST_INCLUDE_DIRS})
+
+find_package (Threads REQUIRED)
+
+# Add the rest of compiz
 add_subdirectory (cmake)
 add_subdirectory (include)
 add_subdirectory (images)
@@ -115,10 +145,6 @@
 add_subdirectory (xslt)
 add_subdirectory (plugins)
 
-if (BUILD_TESTING)
-	add_subdirectory (tests)
-endif (BUILD_TESTING)
-
 compiz_ensure_linkage ()
 compiz_package_generation ("Compiz")
 compiz_add_uninstall ()
@@ -131,4 +157,7 @@
 
 _check_compiz_cmake_macro (${CMAKE_MODULE_PATH_ORIG})
 
+# Enable coverage reporting for compiz
+enable_coverage_report()
+
 

=== modified file 'cmake/CompizCommon.cmake'
--- cmake/CompizCommon.cmake	2011-09-19 13:00:51 +0000
+++ cmake/CompizCommon.cmake	2012-01-12 06:48:58 +0000
@@ -1,12 +1,12 @@
 cmake_minimum_required (VERSION 2.6)
 
-if ("${CMAKE_BINARY_DIR}" STREQUAL "${CMAKE_SOURCE_DIR}")
+if ("${CMAKE_CURRENT_SOURCE_DIR}" STREQUAL "${CMAKE_CURRENT_BINARY_DIR}")
     message (SEND_ERROR "Building in the source directory is not supported.")
     message (FATAL_ERROR "Please remove the created \"CMakeCache.txt\" file, the \"CMakeFiles\" directory and create a build directory and call \"${CMAKE_COMMAND} <path to the sources>\".")
-endif ("${CMAKE_BINARY_DIR}" STREQUAL "${CMAKE_SOURCE_DIR}")
+endif ("${CMAKE_CURRENT_SOURCE_DIR}" STREQUAL "${CMAKE_CURRENT_BINARY_DIR}")
 
 #### CTest
-include (CTest)
+enable_testing()
 
 #### policies
 
@@ -404,7 +404,7 @@
     endforeach (_val ${ARGN})
 endfunction ()
 
-function (compiz_add_plugins_in_folder folder)
+macro (compiz_add_plugins_in_folder folder)
     set (COMPIZ_PLUGIN_PACK_BUILD 1)
     file (
         GLOB _plugins_in
@@ -416,7 +416,7 @@
         get_filename_component (_plugin_dir ${_plugin} PATH)
         add_subdirectory (${folder}/${_plugin_dir})
     endforeach ()
-endfunction ()
+endmacro ()
 
 #### pkg-config handling
 
@@ -489,6 +489,380 @@
     endif ()
 endfunction ()
 
+#### modules / tests
+macro (_get_parameters _prefix)
+    set (_current_var _foo)
+    set (_supported_var PKGDEPS PLUGINDEPS MODULES LDFLAGSADD CFLAGSADD LIBRARIES LIBDIRS INCDIRS DEFSADD)
+    foreach (_val ${_supported_var})
+	set (${_prefix}_${_val})
+    endforeach (_val)
+    foreach (_val ${ARGN})
+	set (_found FALSE)
+	foreach (_find ${_supported_var})
+	    if ("${_find}" STREQUAL "${_val}")
+		set (_found TRUE)
+	    endif ("${_find}" STREQUAL "${_val}")
+	endforeach (_find)
+
+	if (_found)
+	    set (_current_var ${_prefix}_${_val})
+	else (_found)
+	    list (APPEND ${_current_var} ${_val})
+	endif (_found)
+    endforeach (_val)
+endmacro (_get_parameters)
+
+macro (_check_pkg_deps _prefix)
+    set (${_prefix}_HAS_PKG_DEPS TRUE)
+    foreach (_val ${ARGN})
+        string (REGEX REPLACE "[<>=\\.]" "_" _name ${_val})
+	string (TOUPPER ${_name} _name)
+
+	compiz_pkg_check_modules (_${_name} ${_val})
+
+	if (_${_name}_FOUND)
+	    list (APPEND ${_prefix}_PKG_LIBDIRS "${_${_name}_LIBRARY_DIRS}")
+	    list (APPEND ${_prefix}_PKG_LIBRARIES "${_${_name}_LIBRARIES}")
+	    list (APPEND ${_prefix}_PKG_INCDIRS "${_${_name}_INCLUDE_DIRS}")
+	else (_${_name}_FOUND)
+	    set (${_prefix}_HAS_PKG_DEPS FALSE)
+	    compiz_set (${_prefix}_MISSING_DEPS "${${_prefix}_MISSING_DEPS} ${_val}")
+	    set(__pkg_config_checked__${_name} 0 CACHE INTERNAL "" FORCE)
+	endif (_${_name}_FOUND)
+    endforeach ()
+endmacro (_check_pkg_deps)
+
+macro (_build_include_flags _prefix)
+    foreach (_include ${ARGN})
+	if (NOT ${_prefix}_INCLUDE_CFLAGS)
+	    compiz_set (${_prefix}_INCLUDE_CFLAGS "" PARENT_SCOPE)
+	endif (NOT ${_prefix}_INCLUDE_CFLAGS)
+	list (APPEND ${_prefix}_INCLUDE_CFLAGS -I${_include})
+    endforeach (_include)
+endmacro (_build_include_flags)
+
+macro (_build_definitions_flags _prefix)
+    foreach (_def ${ARGN})
+	if (NOT ${_prefix}_DEFINITIONS_CFLAGS)
+	    compiz_set (${_prefix}_DEFINITIONS_CFLAGS "")
+	endif (NOT ${_prefix}_DEFINITIONS_CFLAGS)
+	list (APPEND ${_prefix}_DEFINITIONS_CFLAGS -D${_def})
+    endforeach (_def)
+endmacro (_build_definitions_flags)
+
+macro (_build_link_dir_flags _prefix)
+    foreach (_link_dir ${ARGN})
+	if (NOT ${_prefix}_LINK_DIR_LDFLAGS)
+	    compiz_set (${_prefix}_LINK_DIR_LDFLAGS "")
+	endif (NOT ${_prefix}_LINK_DIR_LDFLAGS)
+	list (APPEND ${_prefix}_LINK_DIR_LDFLAGS -L${_link_dir})
+    endforeach (_link_dir)
+endmacro (_build_link_dir_flags)
+
+macro (_build_library_flags _prefix)
+    foreach (_library ${ARGN})
+	if (NOT ${_prefix}_LIBRARY_LDFLAGS)
+	    compiz_set (${_prefix}_LIBRARY_LDFLAGS "")
+	endif (NOT ${_prefix}_LIBRARY_LDFLAGS)
+	list (APPEND ${_prefix}_LIBRARY_LDFLAGS -l${_library})
+    endforeach (_library)
+endmacro (_build_library_flags)
+
+function (_build_compiz_module _prefix _name _full_prefix)
+
+    if (${_full_prefix}_INCLUDE_DIRS)
+	_build_include_flags (${_full_prefix} ${${_full_prefix}_INCLUDE_DIRS})
+    endif (${_full_prefix}_INCLUDE_DIRS)
+    _build_include_flags (${_full_prefix} ${${_full_prefix}_SOURCE_DIR})
+    _build_include_flags (${_full_prefix} ${${_full_prefix}_INCLUDE_DIR})
+
+    if (${_full_prefix}_DEFSADD)
+	_build_definitions_flags (${_full_prefix} ${${_full_prefix}_DEFSADD})
+    endif (${_full_prefix}_DEFSADD)
+
+    if (${_full_prefix}_LIBRARY_DIRS)
+	_build_link_dir_flags (${_full_prefix} ${${_full_prefix}_LIBRARY_DIRS})
+    endif (${_full_prefix}_LIBRARY_DIRS)
+
+    if (${_full_prefix}_LIBRARIES)
+	_build_library_flags (${_full_prefix} ${${_full_prefix}_LIBRARIES})
+    endif (${_full_prefix}_LIBRARIES)			      
+
+    file (GLOB _cpp_files "${${_full_prefix}_SOURCE_DIR}/*.cpp")
+
+    add_library (${_prefix}_${_name}_internal STATIC ${_cpp_files})
+
+    target_link_libraries (${_prefix}_${_name}_internal
+			   ${${_full_prefix}_LIBRARIES} m pthread dl)
+
+    list (APPEND ${_full_prefix}_COMPILE_FLAGS ${${_full_prefix}_INCLUDE_CFLAGS})
+    list (APPEND ${_full_prefix}_COMPILE_FLAGS ${${_full_prefix}_DEFINITIONS_CFLAGS})
+    list (APPEND ${_full_prefix}_COMPILE_FLAGS ${${_full_prefix}_CFLAGSADD})
+
+    set (${_full_prefix}_COMPILE_FLAGS_STR " ")
+    foreach (_flag ${${_full_prefix}_COMPILE_FLAGS})
+	set (${_full_prefix}_COMPILE_FLAGS_STR "${_flag} ${${_full_prefix}_COMPILE_FLAGS_STR}")
+    endforeach (_flag)
+
+    list (APPEND ${_full_prefix}_LINK_FLAGS ${${_full_prefix}_LINK_LDFLAGS})
+    list (APPEND ${_full_prefix}_LINK_FLAGS ${${_full_prefix}_LDFLAGSADD})
+    list (APPEND ${_full_prefix}_LINK_FLAGS ${${_full_prefix}_LIBARY_FLAGS})
+
+    set (${_full_prefix}_LINK_FLAGS_STR " ")
+    foreach (_flag ${${_full_prefix}_LINK_FLAGS})
+	set (${_full_prefix}_LINK_FLAGS_STR "${_flag} ${${_full_prefix}_LINK_FLAGS_STR}")
+    endforeach (_flag)
+
+    set_target_properties (${_prefix}_${_name}_internal PROPERTIES
+			   COMPILE_FLAGS ${${_full_prefix}_COMPILE_FLAGS_STR}
+			   LINK_FLAGS ${${_full_prefix}_LINK_FLAGS_STR})
+
+    file (GLOB _h_files "${_full_prefix}_INCLUDE_DIR/*.h")
+
+    foreach (_file ${_h_files})
+
+	install (
+	    FILES ${_file}
+	    DESTINATION ${COMPIZ_DESTDIR}${includedir}/compiz/${_prefix}
+	)
+
+    endforeach (_file)
+
+endfunction (_build_compiz_module)
+
+macro (compiz_module _prefix _name)
+
+    string (TOUPPER ${_prefix} _PREFIX)
+    string (TOUPPER ${_name} _NAME)
+    set (_FULL_PREFIX ${_PREFIX}_${_NAME})
+
+    _get_parameters (${_FULL_PREFIX} ${ARGN})
+    _check_pkg_deps (${_FULL_PREFIX} ${${_FULL_PREFIX}_PKGDEPS})
+
+    if (${_FULL_PREFIX}_HAS_PKG_DEPS)
+
+	list (APPEND ${_FULL_PREFIX}_LIBRARIES ${${_FULL_PREFIX}_PKG_LIBRARIES})
+	list (APPEND ${_FULL_PREFIX}_INCLUDE_DIRS ${${_FULL_PREFIX}_INCDIRS})
+	list (APPEND ${_FULL_PREFIX}_INCLUDE_DIRS ${${_FULL_PREFIX}_PKG_INCDIRS})
+	list (APPEND ${_FULL_PREFIX}_LIBRARY_DIRS ${${_FULL_PREFIX}_LIBDIRS})
+	list (APPEND ${_FULL_PREFIX}_LIBRARY_DIRS ${${_FULL_PREFIX}_PKG_LIBDIRS})
+
+	# also add modules
+	foreach (_module ${${_FULL_PREFIX}_MODULES})
+	    string (TOUPPER ${_module} _MODULE)
+	    list (APPEND ${_FULL_PREFIX}_INCLUDE_DIRS ${${_MODULE}_INCLUDE_DIR})
+	    list (APPEND ${_FULL_PREFIX}_LIBRARY_DIRS ${${_MODULE}_BINARY_DIR})
+	    list (APPEND ${_FULL_PREFIX}_LIBRARIES ${_module}_internal)
+	endforeach (_module)
+
+	compiz_set (${_FULL_PREFIX}_BINARY_DIR ${CMAKE_CURRENT_BINARY_DIR}/${_name})
+	compiz_set (${_FULL_PREFIX}_SOURCE_DIR ${CMAKE_CURRENT_SOURCE_DIR}/${_name}/src)
+	compiz_set (${_FULL_PREFIX}_INCLUDE_DIR ${CMAKE_CURRENT_SOURCE_DIR}/${_name}/include)
+	compiz_set (${_FULL_PREFIX}_TESTS_DIR ${CMAKE_CURRENT_SOURCE_DIR}/${_name}tests)
+
+	# Need to abuse set_property here since set () with CACHE INTERNAL will save the 
+	# value to the cache which we will just read right back (but we need to regenerate that)
+	set_property (GLOBAL APPEND PROPERTY ${_PREFIX}_MOD_LIBRARY_DIRS ${${_FULL_PREFIX}_BINARY_DIR})
+	set_property (GLOBAL APPEND PROPERTY ${_PREFIX}_MOD_INCLUDE_DIRS ${${_FULL_PREFIX}_INCLUDE_DIR})
+	set_property (GLOBAL APPEND PROPERTY ${_PREFIX}_MOD_INCLUDE_DIRS ${${_FULL_PREFIX}_SOURCE_DIR})
+	set_property (GLOBAL APPEND PROPERTY ${_PREFIX}_MOD_LIBRARIES ${_prefix}_${_name}_internal)
+
+	_build_compiz_module (${_prefix} ${_name} ${_FULL_PREFIX})
+
+	add_subdirectory (${CMAKE_CURRENT_SOURCE_DIR}/${_name}/tests)
+
+    else (${_FULL_PREFIX}_HAS_PKG_DEPS)
+	message (STATUS "[WARNING] One or more dependencies for module ${_name} for ${_prefix} not found. Skipping module.")
+	message (STATUS "Missing dependencies :${${_FULL_PREFIX}_MISSING_DEPS}")
+	compiz_set (${_FULL_PREFIX}_BUILD FALSE)
+    endif (${_FULL_PREFIX}_HAS_PKG_DEPS)
+
+    
+endmacro (compiz_module)
+
+function (_build_compiz_test_base _prefix _module _full_prefix)
+
+    file (GLOB _cpp_files "${${_FULL_TEST_BASE_PREFIX}_SOURCE_DIR}/*.cpp")
+
+    if (${_full_prefix}_INCLUDE_DIRS)
+	_build_include_flags (${_full_prefix} ${${_full_prefix}_INCLUDE_DIRS})
+    endif (${_full_prefix}_INCLUDE_DIRS)
+    _build_include_flags (${_full_prefix} ${${_full_prefix}_SOURCE_DIR})
+    _build_include_flags (${_full_prefix} ${${_full_prefix}_INCLUDE_DIR})
+
+    if (${_full_prefix}_DEFSADD)
+	_build_definitions_flags (${_full_prefix} ${${_full_prefix}_DEFSADD})
+    endif (${_full_prefix}_DEFSADD)
+
+    if (${_full_prefix}_LIBRARY_DIRS)
+	_build_link_dir_flags (${_full_prefix} ${${_full_prefix}_LIBRARY_DIRS})
+    endif (${_full_prefix}_LIBRARY_DIRS)
+
+    if (${_full_prefix}_LIBRARIES)
+	_build_library_flags (${_full_prefix} ${${_full_prefix}_LIBRARIES})
+    endif (${_full_prefix}_LIBRARIES)
+
+    add_library (${_prefix}_${_module}_test_internal STATIC
+		 ${_cpp_files})
+
+    target_link_libraries (${_prefix}_${_module}_test_internal
+			   ${${_full_prefix}_LIBRARIES}
+			   ${_prefix}_${_module}_internal)
+
+
+    list (APPEND ${_full_prefix}_COMPILE_FLAGS ${${_full_prefix}_INCLUDE_CFLAGS})
+    list (APPEND ${_full_prefix}_COMPILE_FLAGS ${${_full_prefix}_DEFINITIONS_CFLAGS})
+    list (APPEND ${_full_prefix}_COMPILE_FLAGS ${${_full_prefix}_CFLAGSADD})
+
+    set (${_full_prefix}_COMPILE_FLAGS_STR "  ")
+    foreach (_flag ${${_full_prefix}_COMPILE_FLAGS})
+	set (${_full_prefix}_COMPILE_FLAGS_STR "${_flag} ${${_full_prefix}_COMPILE_FLAGS_STR}")
+    endforeach (_flag)
+
+    list (APPEND ${_full_prefix}_LINK_FLAGS ${${_full_prefix}_LINK_LDFLAGS})
+    list (APPEND ${_full_prefix}_LINK_FLAGS ${${_full_prefix}_LDFLAGSADD})
+    list (APPEND ${_full_prefix}_LINK_FLAGS ${${_full_prefix}_LIBARY_FLAGS})
+
+    set (${_full_prefix}_LINK_FLAGS_STR " ")
+    foreach (_flag ${${_full_prefix}_LINK_FLAGS})
+	set (${_full_prefix}_LINK_FLAGS_STR "${_flag} ${${_full_prefix}_LINK_FLAGS_STR}")
+    endforeach (_flag)
+
+    set_target_properties (${_prefix}_${_module}_test_internal PROPERTIES
+			   COMPILE_FLAGS "${${_full_prefix}_COMPILE_FLAGS_STR}"
+			   LINK_FLAGS "${${_full_prefix}_LINK_FLAGS_STR}")
+endfunction (_build_compiz_test_base)
+
+macro (compiz_test_base _prefix _module)
+
+    string (TOUPPER ${_prefix} _PREFIX)
+    string (TOUPPER ${_module} _MODULE)
+
+    set (_FULL_MODULE_PREFIX ${_PREFIX}_${_NAME})
+    set (_FULL_TEST_BASE_PREFIX ${_FULL_MODULE_PREFIX}_TEST_BASE)
+
+    _get_parameters (${_FULL_TEST_BASE_PREFIX} ${ARGN})
+    _check_pkg_deps (${_FULL_TEST_BASE_PREFIX} ${${_FULL_TEST_BASE_PREFIX}_PKGDEPS})
+
+    if (${_FULL_TEST_BASE_PREFIX}_HAS_PKG_DEPS)
+
+	list (APPEND ${_FULL_TEST_BASE_PREFIX}_LIBRARIES ${${_FULL_TEST_BASE_PREFIX}_PKG_LIBDIRS})
+	list (APPEND ${_FULL_TEST_BASE_PREFIX}_INCLUDE_DIRS ${${_FULL_TEST_BASE_PREFIX}_INCDIRS})
+	list (APPEND ${_FULL_TEST_BASE_PREFIX}_INCLUDE_DIRS ${${_FULL_TEST_BASE_PREFIX}_PKG_INCDIRS})
+	list (APPEND ${_FULL_TEST_BASE_PREFIX}_LIBRARY_DIRS ${${_FULL_TEST_BASE_PREFIX}_LIBDIRS})
+	list (APPEND ${_FULL_TEST_BASE_PREFIX}_LIBRARY_DIRS ${${_FULL_TEST_BASE_PREFIX}_PKG_LIBDIRS})
+
+	compiz_set (${_FULL_TEST_BASE_PREFIX}_BINARY_DIR ${CMAKE_CURRENT_BINARY_DIR})
+	compiz_set (${_FULL_TEST_BASE_PREFIX}_SOURCE_DIR ${CMAKE_CURRENT_SOURCE_DIR})
+	compiz_set (${_FULL_TEST_BASE_PREFIX}_INCLUDE_DIR ${CMAKE_CURRENT_SOURCE_DIR})
+
+	list (APPEND ${_FULL_TEST_BASE_PREFIX}_INCLUDE_DIRS ${${_FULL_MODULE_PREFIX}_INCLUDE_DIRS})
+	list (APPEND ${_FULL_TEST_BASE_PREFIX}_INCLUDE_DIRS ${${_FULL_MODULE_PREFIX}_INCLUDE_DIR})
+	list (APPEND ${_FULL_TEST_BASE_PREFIX}_INCLUDE_DIRS ${${_FULL_MODULE_PREFIX}_SOURCE_DIR})
+	list (APPEND ${_FULL_TEST_BASE_PREFIX}_LIBRARY_DIRS ${${_FULL_MODULE_PREFIX}_LIBRARY_DIRS})
+	list (APPEND ${_FULL_TEST_BASE_PREFIX}_LIBRARY_DIRS ${${_FULL_MODULE_PREFIX}_BINARY_DIR})
+	list (APPEND ${_FULL_TEST_BASE_PREFIX}_LIBRARIES ${${_FULL_MODULE_PREFIX}_LIBRARIES})
+
+	_build_compiz_test_base (${_prefix} ${_module} ${_FULL_TEST_BASE_PREFIX})
+    else (${_FULL_TEST_BASE_PREFIX}_HAS_PKG_DEPS)
+	message (STATUS "[WARNING] One or more dependencies for test base on module ${_module} for ${_prefix} not found. Skipping test base.")
+	message (STATUS "Missing dependencies :${${_FULL_TEST_BASE_PREFIX}_MISSING_DEPS}")
+	compiz_set (${_FULL_TEST_BASE_PREFIX}_BUILD FALSE)
+    endif (${_FULL_TEST_BASE_PREFIX}_HAS_PKG_DEPS)
+endmacro (compiz_test_base)
+
+function (_build_compiz_test _prefix _module _test _full_prefix)
+    file (GLOB _cpp_files "${${_FULL_TEST_PREFIX}_SOURCE_DIR}/*.cpp")
+
+    if (${_full_prefix}_INCLUDE_DIRS)
+	_build_include_flags (${_full_prefix} ${${_full_prefix}_INCLUDE_DIRS})
+    endif (${_full_prefix}_INCLUDE_DIRS)
+    _build_include_flags (${_full_prefix} ${${_full_prefix}_SOURCE_DIR})
+    _build_include_flags (${_full_prefix} ${${_full_prefix}_INCLUDE_DIR})
+
+    if (${_full_prefix}_DEFSADD)
+	_build_definitions_flags (${_full_prefix} ${${_full_prefix}_DEFSADD})
+    endif (${_full_prefix}_DEFSADD)
+
+    if (${_full_prefix}_LIBRARY_DIRS)
+	_build_link_dir_flags (${_full_prefix} ${${_full_prefix}_LIBRARY_DIRS})
+    endif (${_full_prefix}_LIBRARY_DIRS)
+
+    if (${_full_prefix}_LIBRARIES)
+	_build_library_flags (${_full_prefix} ${${_full_prefix}_LIBRARIES})
+    endif (${_full_prefix}_LIBRARIES)
+
+    add_executable (${_prefix}_${_module}_${_test}_test
+		    ${_cpp_files})
+
+    target_link_libraries (${_prefix}_${_module}_${_test}_test
+			   ${${_full_prefix}_LIBRARIES}
+			   ${_prefix}_${_module}_internal
+			   ${_prefix}_${_module}_test_internal)
+
+    list (APPEND ${_full_prefix}_COMPILE_FLAGS ${${_full_prefix}_INCLUDE_CFLAGS})
+    list (APPEND ${_full_prefix}_COMPILE_FLAGS ${${_full_prefix}_DEFINITIONS_CFLAGS})
+    list (APPEND ${_full_prefix}_COMPILE_FLAGS ${${_full_prefix}_CFLAGSADD})
+
+    set (${_full_prefix}_COMPILE_FLAGS_STR " ")
+    foreach (_flag ${${_full_prefix}_COMPILE_FLAGS})
+	set (${_full_prefix}_COMPILE_FLAGS_STR "${_flag} ${${_full_prefix}_COMPILE_FLAGS_STR}")
+    endforeach (_flag)
+
+    list (APPEND ${_full_prefix}_LINK_FLAGS ${${_full_prefix}_LINK_LDFLAGS})
+    list (APPEND ${_full_prefix}_LINK_FLAGS ${${_full_prefix}_LDFLAGSADD})
+    list (APPEND ${_full_prefix}_LINK_FLAGS ${${_full_prefix}_LIBARY_FLAGS})
+
+    set (${_full_prefix}_LINK_FLAGS_STR " ")
+    foreach (_flag ${${_full_prefix}_LINK_FLAGS})
+	set (${_full_prefix}_LINK_FLAGS_STR "${_flag} ${${_full_prefix}_LINK_FLAGS_STR}")
+    endforeach (_flag)
+
+    set_target_properties (${_prefix}_${_module}_${_test}_test PROPERTIES
+			   COMPILE_FLAGS "${${_full_prefix}_COMPILE_FLAGS_STR}"
+			   LINK_FLAGS "${${_full_prefix}_LINK_FLAGS_STR}")
+
+    add_test (test-${_prefix}-${_module}-${_test}
+	      ${CMAKE_CURRENT_BINARY_DIR}/${_prefix}_${_module}_${_test}_test)
+endfunction (_build_compiz_test)
+
+macro (compiz_test _prefix _module _test)
+
+    set (_supported_var PKGDEPS LDFLAGSADD CFLAGSADD LIBRARIES LIBDIRS INCDIRS DEFSADD)
+
+    set (_FULL_TEST_PREFIX ${_FULL_MODULE_PREFIX}_TEST)
+
+    _get_parameters (${_FULL_TEST_PREFIX} ${ARGN})
+    _check_pkg_deps (${_FULL_TEST_PREFIX} ${${_FULL_TEST_PREFIX}_PKGDEPS})
+
+    if (${_FULL_TEST_PREFIX}_HAS_PKG_DEPS)
+	list (APPEND ${_FULL_TEST_PREFIX}_LIBRARIES ${${_FULL_TEST_PREFIX}_PKG_LIBDIRS})
+	list (APPEND ${_FULL_TEST_PREFIX}_INCLUDE_DIRS ${${_FULL_TEST_PREFIX}_INCDIRS})
+	list (APPEND ${_FULL_TEST_PREFIX}_INCLUDE_DIRS ${${_FULL_TEST_PREFIX}_PKG_INCDIRS})
+	list (APPEND ${_FULL_TEST_PREFIX}_LIBRARY_DIRS ${${_FULL_TEST_PREFIX}_LIBDIRS})
+	list (APPEND ${_FULL_TEST_PREFIX}_LIBRARY_DIRS ${${_FULL_TEST_PREFIX}_PKG_LIBDIRS})
+
+	compiz_set (${_FULL_TEST_PREFIX}_BINARY_DIR ${CMAKE_CURRENT_BINARY_DIR})
+	compiz_set (${_FULL_TEST_PREFIX}_SOURCE_DIR ${CMAKE_CURRENT_SOURCE_DIR}/src)
+	compiz_set (${_FULL_TEST_PREFIX}_INCLUDE_DIR ${CMAKE_CURRENT_SOURCE_DIR})
+
+	list (APPEND ${_FULL_TEST_PREFIX}_INCLUDE_DIRS ${${_FULL_TEST_BASE_PREFIX}_INCLUDE_DIRS})
+	list (APPEND ${_FULL_TEST_PREFIX}_INCLUDE_DIRS ${${_FULL_TEST_BASE_PREFIX}_INCLUDE_DIR})
+	list (APPEND ${_FULL_TEST_PREFIX}_INCLUDE_DIRS ${${_FULL_TEST_BASE_PREFIX}_SOURCE_DIR})
+	list (APPEND ${_FULL_TEST_PREFIX}_LIBRARY_DIRS ${${_FULL_TEST_BASE_PREFIX}_LIBRARY_DIRS})
+	list (APPEND ${_FULL_TEST_PREFIX}_LIBRARY_DIRS ${${_FULL_TEST_BASE_PREFIX}_BINARY_DIR})
+	list (APPEND ${_FULL_TEST_PREFIX}_LIBRARIES ${${_FULL_TEST_BASE_PREFIX}_LIBRARIES})
+
+	_build_compiz_test (${_prefix} ${_module} ${_test} ${_FULL_TEST_PREFIX})
+
+    else (${_FULL_TEST_PREFIX}_HAS_PKG_DEPS)
+	message (STATUS "[WARNING] One or more dependencies for test ${_test} on module ${_name} for ${_prefix} not found. Skipping test.")
+	message (STATUS "Missing dependencies :${${_FULL_TEST_PREFIX}_MISSING_DEPS}")
+	compiz_set (${_FULL_TEST_PREFIX}_BUILD FALSE)
+    endif (${_FULL_TEST_PREFIX}_HAS_PKG_DEPS)
+
+endmacro (compiz_test)
+
 #### optional file install
 
 function (compiz_opt_install_file _src _dst)

=== modified file 'cmake/CompizPlugin.cmake'
--- cmake/CompizPlugin.cmake	2011-09-01 19:52:58 +0000
+++ cmake/CompizPlugin.cmake	2012-01-12 06:48:58 +0000
@@ -136,50 +136,6 @@
     endif ("${COMPIZ_PLUGIN_INSTALL_TYPE}" STREQUAL "package")
 endmacro (_prepare_directories)
 
-# parse plugin macro parameter
-macro (_get_plugin_parameters _prefix)
-    set (_current_var _foo)
-    set (_supported_var PKGDEPS PLUGINDEPS LDFLAGSADD CFLAGSADD LIBRARIES LIBDIRS INCDIRS)
-    foreach (_val ${_supported_var})
-	set (${_prefix}_${_val})
-    endforeach (_val)
-    foreach (_val ${ARGN})
-	set (_found FALSE)
-	foreach (_find ${_supported_var})
-	    if ("${_find}" STREQUAL "${_val}")
-		set (_found TRUE)
-	    endif ("${_find}" STREQUAL "${_val}")
-	endforeach (_find)
-
-	if (_found)
-	    set (_current_var ${_prefix}_${_val})
-	else (_found)
-	    list (APPEND ${_current_var} ${_val})
-	endif (_found)
-    endforeach (_val)
-endmacro (_get_plugin_parameters)
-
-# check pkgconfig dependencies
-macro (_check_plugin_pkg_deps _prefix)
-    set (${_prefix}_HAS_PKG_DEPS TRUE)
-    foreach (_val ${ARGN})
-        string (REGEX REPLACE "[<>=\\.]" "_" _name ${_val})
-	string (TOUPPER ${_name} _name)
-
-	compiz_pkg_check_modules (_${_name} ${_val})
-
-	if (_${_name}_FOUND)
-	    list (APPEND ${_prefix}_PKG_LIBDIRS "${_${_name}_LIBRARY_DIRS}")
-	    list (APPEND ${_prefix}_PKG_LIBRARIES "${_${_name}_LIBRARIES}")
-	    list (APPEND ${_prefix}_PKG_INCDIRS "${_${_name}_INCLUDE_DIRS}")
-	else ()
-	    set (${_prefix}_HAS_PKG_DEPS FALSE)
-	    compiz_set (COMPIZ_${_prefix}_MISSING_DEPS "${COMPIZ_${_prefix}_MISSING_DEPS} ${_val}")
-	    set(__pkg_config_checked__${_name} 0 CACHE INTERNAL "" FORCE)
-	endif ()
-    endforeach ()
-endmacro ()
-
 # check plugin dependencies
 macro (_check_plugin_plugin_deps _prefix)
     set (${_prefix}_HAS_PLUGIN_DEPS TRUE)
@@ -216,11 +172,8 @@
     endforeach ()
 endmacro ()
 
-
-
-
 # main function
-function (_build_compiz_plugin plugin)
+macro (_build_compiz_plugin plugin)
     string (TOUPPER ${plugin} _PLUGIN)
 
     if (COMPIZ_PLUGIN_INSTALL_TYPE)
@@ -235,7 +188,7 @@
 	)
     endif (COMPIZ_PLUGIN_INSTALL_TYPE)
 
-    _get_plugin_parameters (${_PLUGIN} ${ARGN})
+    _get_parameters (${_PLUGIN} ${ARGN})
     _prepare_directories ()
 
     find_file (
@@ -259,7 +212,7 @@
     # check dependencies
     compiz_unset (COMPIZ_${_PLUGIN}_MISSING_DEPS)
     _check_plugin_plugin_deps (${_PLUGIN} ${${_PLUGIN}_PLUGINDEPS})
-    _check_plugin_pkg_deps (${_PLUGIN} ${${_PLUGIN}_PKGDEPS})
+    _check_pkg_deps (${_PLUGIN} ${${_PLUGIN}_PKGDEPS})
 
     if (${_PLUGIN}_HAS_PKG_DEPS AND ${_PLUGIN}_HAS_PLUGIN_DEPS)
 
@@ -357,6 +310,30 @@
 	add_definitions (-DPREFIX='\"${PLUGIN_PREFIX}\"'
 			 ${COMPIZ_DEFINITIONS_ADD})
 
+	foreach (_def ${_PLUGIN}_DEFSADD)
+	    add_definitions (-D${_def})
+	endforeach (_def)
+
+	# Need to know the include-dirs for the internal
+	# modules to this plugin, core (if built with core)
+	# and any other plugins that we depend on
+
+	get_property (${_PLUGIN}_MOD_INCLUDE_DIRS
+		      GLOBAL
+		      PROPERTY ${_PLUGIN}_MOD_INCLUDE_DIRS)
+
+	get_property (CORE_MOD_INCLUDE_DIRS
+		      GLOBAL
+		      PROPERTY CORE_MOD_INCLUDE_DIRS)
+
+	foreach (_plugindep ${${_PLUGIN}_PLUGINDEPS})
+	    string (TOUPPER ${_plugindep} _PLUGINDEP)
+	    get_property (${_PLUGINDEP}_MOD_INCLUDE_DIRS
+			  GLOBAL
+			  PROPERTY ${_PLUGINDEP}_MOD_INCLUDE_DIRS)
+	    list (APPEND ${_PLUGIN}_PLUGINDEP_MOD_INCLUDE_DIRS ${${_PLUGINDEP}_MOD_INCLUDE_DIRS})
+	endforeach (_plugindep)
+
 	include_directories (
             ${CMAKE_CURRENT_SOURCE_DIR}/src
             ${CMAKE_CURRENT_SOURCE_DIR}/include
@@ -367,8 +344,15 @@
             ${COMPIZ_INCLUDE_DIRS}
             ${CMAKE_PREFIX_PATH}/include
             ${CMAKE_INCLUDE_PATH}
+	    ${${_PLUGIN}_MOD_INCLUDE_DIRS}
+	    ${CORE_MOD_INCLUDE_DIRS}
+	    ${${_PLUGIN}_PLUGINDEP_MOD_INCLUDE_DIRS}
 	)
 
+	get_property (${_PLUGIN}_MOD_LIBRARY_DIRS
+		      GLOBAL
+		      PROPERTY ${_PLUGIN}_MOD_LIBRARY_DIRS)
+
 	link_directories (
             ${COMPIZ_LINK_DIRS}
             ${${_PLUGIN}_PKG_LIBDIRS}
@@ -378,6 +362,7 @@
             ${CMAKE_PREFIX_PATH}/lib
             ${CMAKE_PREFIX_PATH}/lib32
             ${CMAKE_PREFIX_PATH}/lib64
+	    ${${_PLUGIN}_MOD_LIBRARY_DIRS}
 	)
 
 	add_library (
@@ -409,11 +394,16 @@
 	    )
 	endif (COMPIZ_BUILD_WITH_RPATH)
 
+	get_property (${_PLUGIN}_MOD_LIBRARIES
+		      GLOBAL
+		      PROPERTY ${_PLUGIN}_MOD_LIBRARIES)
+
 	target_link_libraries (
 	    ${plugin} ${COMPIZ_LIBRARIES}
 		      ${${_PLUGIN}_LOCAL_LIBRARIES}
 		      ${${_PLUGIN}_PKG_LIBRARIES}
 		      ${${_PLUGIN}_LIBRARIES}
+		      ${${_PLUGIN}_MOD_LIBRARIES}
 	)
 
 	install (
@@ -449,7 +439,7 @@
 	message (STATUS "Missing dependencies :${COMPIZ_${_PLUGIN}_MISSING_DEPS}")
 	compiz_set (COMPIZ_${_PLUGIN}_BUILD FALSE)
     endif ()
-endfunction ()
+endmacro ()
 
 macro (compiz_plugin plugin)
     string (TOUPPER ${plugin} _PLUGIN)

=== added file 'cmake/EnableCoverageReport.cmake'
--- cmake/EnableCoverageReport.cmake	1970-01-01 00:00:00 +0000
+++ cmake/EnableCoverageReport.cmake	2011-12-19 07:06:22 +0000
@@ -0,0 +1,153 @@
+# - Creates a special coverage build type and target on GCC.
+#
+# Defines a function ENABLE_COVERAGE_REPORT which generates the coverage target
+# for selected targets. Optional arguments to this function are used to filter
+# unwanted results using globbing expressions. Moreover targets with tests for
+# the source code can be specified to trigger regenerating the report if the
+# test has changed
+#
+# ENABLE_COVERAGE_REPORT(TARGETS target... [FILTER filter...] [TESTS test targets...])
+#
+# To generate a coverage report first build the project with
+# CMAKE_BUILD_TYPE=coverage, then call make test and afterwards make coverage.
+#
+# The coverage report is based on gcov. Depending on the availability of lcov
+# a HTML report will be generated and/or an XML report of gcovr is found.
+# The generated coverage target executes all found solutions. Special targets
+# exist to create e.g. only the xml report: coverage-xml. 
+#
+# Copyright (C) 2010 by Johannes Wienke <jwienke at techfak dot uni-bielefeld dot de>
+#
+# This program is free software; you can redistribute it
+# and/or modify it under the terms of the GNU General
+# Public License as published by the Free Software Foundation;
+# either version 2, or (at your option)
+# any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+
+INCLUDE(ParseArguments)
+
+FIND_PACKAGE(Lcov)
+FIND_PACKAGE(gcovr)
+
+FUNCTION(ENABLE_COVERAGE_REPORT)
+    
+    # argument parsing
+    PARSE_ARGUMENTS(ARG "FILTER;TARGETS;TESTS" "" ${ARGN})
+    
+    SET(COVERAGE_RAW_FILE "${CMAKE_BINARY_DIR}/coverage.raw.info")
+    SET(COVERAGE_FILTERED_FILE "${CMAKE_BINARY_DIR}/coverage.info")
+    SET(COVERAGE_REPORT_DIR "${CMAKE_BINARY_DIR}/coveragereport")
+    SET(COVERAGE_XML_FILE "${CMAKE_BINARY_DIR}/coverage.xml")
+    SET(COVERAGE_XML_COMMAND_FILE "${CMAKE_BINARY_DIR}/coverage-xml.cmake")
+    
+    # decide if there is any tool to create coverage data
+    SET(TOOL_FOUND FALSE)
+    IF(LCOV_FOUND OR GCOVR_FOUND)
+        SET(TOOL_FOUND TRUE)
+    ENDIF()
+    IF(NOT TOOL_FOUND)
+        MESSAGE(STATUS "Cannot enable coverage targets because neither lcov nor gcovr are found.")
+    ENDIF()
+    
+    STRING(TOLOWER "${CMAKE_BUILD_TYPE}" COVERAGE_BUILD_TYPE)
+    IF(CMAKE_COMPILER_IS_GNUCXX AND TOOL_FOUND AND "${COVERAGE_BUILD_TYPE}" MATCHES "coverage")
+    
+        MESSAGE(STATUS "Coverage support enabled for targets: ${ARG_TARGETS}")
+    
+        # create coverage build type
+        SET(CMAKE_CXX_FLAGS_COVERAGE ${CMAKE_CXX_FLAGS_DEBUG} PARENT_SCOPE)
+        SET(CMAKE_C_FLAGS_COVERAGE ${CMAKE_C_FLAGS_DEBUG} PARENT_SCOPE)
+        SET(CMAKE_CONFIGURATION_TYPES ${CMAKE_CONFIGURATION_TYPES} coverage PARENT_SCOPE)
+    
+        # instrument targets
+        SET_TARGET_PROPERTIES(${ARG_TARGETS} PROPERTIES COMPILE_FLAGS --coverage
+                                                        LINK_FLAGS --coverage)
+    
+        # html report
+        IF (LCOV_FOUND)
+        
+            MESSAGE(STATUS "Enabling HTML coverage report")
+    
+            # set up coverage target
+            
+            ADD_CUSTOM_COMMAND(OUTPUT ${COVERAGE_RAW_FILE}
+                               COMMAND ${LCOV_EXECUTABLE} -c -d ${CMAKE_BINARY_DIR} -o ${COVERAGE_RAW_FILE}
+                               WORKING_DIRECTORY ${CMAKE_BINARY_DIR}
+                               COMMENT "Collecting coverage data"
+                               DEPENDS ${ARG_TARGETS} ${ARG_TESTS}
+                               VERBATIM)
+            
+            # filter unwanted stuff
+            LIST(LENGTH ARG_FILTER FILTER_LENGTH)
+            IF(${FILTER_LENGTH} GREATER 0)
+                SET(FILTER COMMAND ${LCOV_EXECUTABLE})
+                FOREACH(F ${ARG_FILTER})
+                    SET(FILTER ${FILTER} -r ${COVERAGE_FILTERED_FILE} ${F})
+                ENDFOREACH()
+                SET(FILTER ${FILTER} -o ${COVERAGE_FILTERED_FILE})
+            ELSE()
+                SET(FILTER "")
+            ENDIF()
+            
+            ADD_CUSTOM_COMMAND(OUTPUT ${COVERAGE_FILTERED_FILE}
+                               COMMAND ${LCOV_EXECUTABLE} -e ${COVERAGE_RAW_FILE} "${CMAKE_SOURCE_DIR}*"  -o ${COVERAGE_FILTERED_FILE}
+                               ${FILTER}
+                               DEPENDS ${COVERAGE_RAW_FILE}
+                               COMMENT "Filtering recorded coverage data for project-relevant entries"
+                               VERBATIM)
+            ADD_CUSTOM_COMMAND(OUTPUT ${COVERAGE_REPORT_DIR}
+                               COMMAND ${CMAKE_COMMAND} -E make_directory ${COVERAGE_REPORT_DIR}
+                               COMMAND ${GENHTML_EXECUTABLE} --legend --show-details -t "${PROJECT_NAME} test coverage" -o ${COVERAGE_REPORT_DIR} ${COVERAGE_FILTERED_FILE}
+                               DEPENDS ${COVERAGE_FILTERED_FILE}
+                               COMMENT "Generating HTML coverage report in ${COVERAGE_REPORT_DIR}"
+                               VERBATIM)
+                               
+            ADD_CUSTOM_TARGET(coverage-html
+                              DEPENDS ${COVERAGE_REPORT_DIR})
+                              
+        ENDIF()
+        
+        # xml coverage report
+        IF(GCOVR_FOUND)
+        
+            MESSAGE(STATUS "Enabling XML coverage report")
+        
+            # gcovr cannot write directly to a file so the execution needs to
+            # be wrapped in a cmake file that generates the file output
+            FILE(WRITE ${COVERAGE_XML_COMMAND_FILE}
+                 "SET(ENV{LANG} en)\n")
+            FILE(APPEND ${COVERAGE_XML_COMMAND_FILE}
+                 "EXECUTE_PROCESS(COMMAND \"${GCOVR_EXECUTABLE}\" -x -r \"${CMAKE_SOURCE_DIR}\" OUTPUT_FILE \"${COVERAGE_XML_FILE}\" WORKING_DIRECTORY \"${CMAKE_BINARY_DIR}\")\n")
+        
+            ADD_CUSTOM_COMMAND(OUTPUT ${COVERAGE_XML_FILE}
+                               COMMAND ${CMAKE_COMMAND} ARGS -P ${COVERAGE_XML_COMMAND_FILE}
+                               COMMENT "Generating coverage XML report"
+                               VERBATIM)
+                               
+            ADD_CUSTOM_TARGET(coverage-xml
+                              DEPENDS ${COVERAGE_XML_FILE})
+        
+        ENDIF()
+        
+        # provide a global coverage target executing both steps if available
+        SET(GLOBAL_DEPENDS "")
+        IF(LCOV_FOUND)
+            LIST(APPEND GLOBAL_DEPENDS ${COVERAGE_REPORT_DIR})
+        ENDIF()
+        IF(GCOVR_FOUND)
+            LIST(APPEND GLOBAL_DEPENDS ${COVERAGE_XML_FILE})
+        ENDIF()
+        IF(LCOV_FOUND OR GCOVR_FOUND)
+            ADD_CUSTOM_TARGET(coverage
+                              DEPENDS ${GLOBAL_DEPENDS})
+        ENDIF()
+                          
+    ENDIF()
+
+ENDFUNCTION()   

=== added file 'cmake/FindLcov.cmake'
--- cmake/FindLcov.cmake	1970-01-01 00:00:00 +0000
+++ cmake/FindLcov.cmake	2011-12-19 07:06:22 +0000
@@ -0,0 +1,29 @@
+# - Find lcov 
+# Will define:
+#
+# LCOV_EXECUTABLE - the lcov binary
+# GENHTML_EXECUTABLE - the genhtml executable
+#
+# Copyright (C) 2010 by Johannes Wienke <jwienke at techfak dot uni-bielefeld dot de>
+#
+# This program is free software; you can redistribute it
+# and/or modify it under the terms of the GNU General
+# Public License as published by the Free Software Foundation;
+# either version 2, or (at your option)
+# any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+
+INCLUDE(FindPackageHandleStandardArgs)
+
+FIND_PROGRAM(LCOV_EXECUTABLE lcov)
+FIND_PROGRAM(GENHTML_EXECUTABLE genhtml)
+
+FIND_PACKAGE_HANDLE_STANDARD_ARGS(Lcov DEFAULT_MSG LCOV_EXECUTABLE GENHTML_EXECUTABLE)
+
+# only visible in advanced view
+MARK_AS_ADVANCED(LCOV_EXECUTABLE GENHTML_EXECUTABLE)

=== added file 'cmake/Findgcovr.cmake'
--- cmake/Findgcovr.cmake	1970-01-01 00:00:00 +0000
+++ cmake/Findgcovr.cmake	2011-12-19 07:06:22 +0000
@@ -0,0 +1,31 @@
+# - Find gcovr scrip 
+# Will define:
+#
+# GCOVR_EXECUTABLE - the gcovr script
+#
+# Uses:
+#
+# GCOVR_ROOT - root to search for the script
+#
+# Copyright (C) 2011 by Johannes Wienke <jwienke at techfak dot uni-bielefeld dot de>
+#
+# This program is free software; you can redistribute it
+# and/or modify it under the terms of the GNU General
+# Public License as published by the Free Software Foundation;
+# either version 2, or (at your option)
+# any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+
+INCLUDE(FindPackageHandleStandardArgs)
+
+FIND_PROGRAM(GCOVR_EXECUTABLE gcovr HINTS ${GCOVR_ROOT} "${GCOVR_ROOT}/bin")
+
+FIND_PACKAGE_HANDLE_STANDARD_ARGS(gcovr DEFAULT_MSG GCOVR_EXECUTABLE)
+
+# only visible in advanced view
+MARK_AS_ADVANCED(GCOVR_EXECUTABLE)

=== added file 'cmake/ParseArguments.cmake'
--- cmake/ParseArguments.cmake	1970-01-01 00:00:00 +0000
+++ cmake/ParseArguments.cmake	2011-12-19 07:06:22 +0000
@@ -0,0 +1,52 @@
+# Parse arguments passed to a function into several lists separated by
+# upper-case identifiers and options that do not have an associated list e.g.:
+#
+# SET(arguments
+#   hello OPTION3 world
+#   LIST3 foo bar
+#   OPTION2
+#   LIST1 fuz baz
+#   )
+# PARSE_ARGUMENTS(ARG "LIST1;LIST2;LIST3" "OPTION1;OPTION2;OPTION3" ${arguments})
+#
+# results in 7 distinct variables:
+#  * ARG_DEFAULT_ARGS: hello;world
+#  * ARG_LIST1: fuz;baz
+#  * ARG_LIST2:
+#  * ARG_LIST3: foo;bar
+#  * ARG_OPTION1: FALSE
+#  * ARG_OPTION2: TRUE
+#  * ARG_OPTION3: TRUE
+#
+# taken from http://www.cmake.org/Wiki/CMakeMacroParseArguments 
+
+MACRO(PARSE_ARGUMENTS prefix arg_names option_names)
+    SET(DEFAULT_ARGS)
+    FOREACH(arg_name ${arg_names})    
+        SET(${prefix}_${arg_name})
+    ENDFOREACH(arg_name)
+    FOREACH(option ${option_names})
+        SET(${prefix}_${option} FALSE)
+    ENDFOREACH(option)
+    
+    SET(current_arg_name DEFAULT_ARGS)
+    SET(current_arg_list)
+    FOREACH(arg ${ARGN})            
+        SET(larg_names ${arg_names})    
+        LIST(FIND larg_names "${arg}" is_arg_name)                   
+        IF (is_arg_name GREATER -1)
+            SET(${prefix}_${current_arg_name} ${current_arg_list})
+            SET(current_arg_name ${arg})
+            SET(current_arg_list)
+        ELSE (is_arg_name GREATER -1)
+            SET(loption_names ${option_names})    
+            LIST(FIND loption_names "${arg}" is_option)            
+            IF (is_option GREATER -1)
+                SET(${prefix}_${arg} TRUE)
+            ELSE (is_option GREATER -1)
+                SET(current_arg_list ${current_arg_list} ${arg})
+            ENDIF (is_option GREATER -1)
+        ENDIF (is_arg_name GREATER -1)
+    ENDFOREACH(arg)
+    SET(${prefix}_${current_arg_name} ${current_arg_list})
+ENDMACRO(PARSE_ARGUMENTS)

=== modified file 'gtk/window-decorator/metacity.c'
--- gtk/window-decorator/metacity.c	2011-10-13 12:06:12 +0000
+++ gtk/window-decorator/metacity.c	2012-01-12 06:48:58 +0000
@@ -52,7 +52,6 @@
     gint	    bottom_stretch_offset;
     gint	    left_stretch_offset;
     gint	    right_stretch_offset;
-
     win_extents = frame_win_extents = d->frame->win_extents;
     max_win_extents = frame_max_win_extents = d->frame->max_win_extents;
 
@@ -60,9 +59,19 @@
      * pixmap type decorations */
     if (!d->frame_window)
     {
-       frame_win_extents.left += settings->mutter_draggable_border_width;
-       frame_win_extents.right += settings->mutter_draggable_border_width;
-       frame_win_extents.bottom += settings->mutter_draggable_border_width;
+	if (flags & META_FRAME_ALLOWS_HORIZONTAL_RESIZE)
+	{
+	    frame_win_extents.left += settings->mutter_draggable_border_width;
+	    frame_win_extents.right += settings->mutter_draggable_border_width;
+	    frame_max_win_extents.left += settings->mutter_draggable_border_width;
+	    frame_max_win_extents.right += settings->mutter_draggable_border_width;
+	}
+
+	if (flags & META_FRAME_ALLOWS_VERTICAL_RESIZE)
+	{
+	    frame_win_extents.bottom += settings->mutter_draggable_border_width;
+	    frame_max_win_extents.bottom += settings->mutter_draggable_border_width;
+	}
     }
 
     w = d->border_layout.top.x2 - d->border_layout.top.x1 -
@@ -1096,7 +1105,7 @@
 	*x += d->frame->win_extents.left + 4;
 	*y += d->frame->win_extents.top + 2;
     }
-    else
+    else if (flags & META_FRAME_ALLOWS_HORIZONTAL_RESIZE)
     {
 	*x += settings->mutter_draggable_border_width;
     }

=== modified file 'include/CMakeLists.txt'
--- include/CMakeLists.txt	2010-05-18 11:30:17 +0000
+++ include/CMakeLists.txt	2012-01-12 06:48:58 +0000
@@ -1,5 +1,4 @@
 set (_headers
-    compiz.h
     decoration.h
 )
 

=== removed file 'include/compiz-common.h.in'
--- include/compiz-common.h.in	2011-07-06 22:55:10 +0000
+++ include/compiz-common.h.in	1970-01-01 00:00:00 +0000
@@ -1,5 +0,0 @@
-#ifndef COMPIZ_COMMON_H
-#define COMPIZ_COMMON_H
-
-#define COMPIZ_VERSION_STRING "@VERSION@"
-#endif

=== removed file 'include/compiz.h'
--- include/compiz.h	2011-06-25 05:06:53 +0000
+++ include/compiz.h	1970-01-01 00:00:00 +0000
@@ -1,103 +0,0 @@
-/*
- * Copyright © 2007 Novell, Inc.
- *
- * Permission to use, copy, modify, distribute, and sell this software
- * and its documentation for any purpose is hereby granted without
- * fee, provided that the above copyright notice appear in all copies
- * and that both that copyright notice and this permission notice
- * appear in supporting documentation, and that the name of
- * Novell, Inc. not be used in advertising or publicity pertaining to
- * distribution of the software without specific, written prior permission.
- * Novell, Inc. makes no representations about the suitability of this
- * software for any purpose. It is provided "as is" without express or
- * implied warranty.
- *
- * NOVELL, INC. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
- * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
- * NO EVENT SHALL NOVELL, INC. BE LIABLE FOR ANY SPECIAL, INDIRECT OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
- * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
- * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
- * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * Author: David Reveman <davidr@novell.com>
- */
-
-#ifndef _COMPIZ_H
-#define _COMPIZ_H
-
-#include <compiz-common.h>
-
-#include <string>
-#include <list>
-#include <cstdarg>
-
-#define STRINGIFY(x) #x
-#define TOSTRING(x) STRINGIFY (x)
-
-#define RESTRICT_VALUE(value, min, max)				     \
-    (((value) < (min)) ? (min): ((value) > (max)) ? (max) : (value))
-
-#define MOD(a,b) ((a) < 0 ? ((b) - ((-(a) - 1) % (b))) - 1 : (a) % (b))
-
-#define TIMEVALDIFF(tv1, tv2)						   \
-    ((tv1)->tv_sec == (tv2)->tv_sec || (tv1)->tv_usec >= (tv2)->tv_usec) ? \
-    ((((tv1)->tv_sec - (tv2)->tv_sec) * 1000000) +			   \
-     ((tv1)->tv_usec - (tv2)->tv_usec)) / 1000 :			   \
-    ((((tv1)->tv_sec - 1 - (tv2)->tv_sec) * 1000000) +			   \
-     (1000000 + (tv1)->tv_usec - (tv2)->tv_usec)) / 1000
-
-#define TIMESPECDIFF(ts1, ts2)						   \
-    ((ts1)->tv_sec == (ts2)->tv_sec || (ts1)->tv_nsec >= (ts2)->tv_nsec) ? \
-    ((((ts1)->tv_sec - (ts2)->tv_sec) * 1000000) +			   \
-     ((ts1)->tv_nsec - (ts2)->tv_nsec)) / 1000000 :			   \
-    ((((ts1)->tv_sec - 1 - (ts2)->tv_sec) * 1000000) +			   \
-     (1000000 + (ts1)->tv_nsec - (ts2)->tv_nsec)) / 1000000
-
-#define MULTIPLY_USHORT(us1, us2)		 \
-    (((GLuint) (us1) * (GLuint) (us2)) / 0xffff)
-
-#define DEG2RAD (M_PI / 180.0f)
-
-#if defined(HAVE_SCANDIR_POSIX)
-  // POSIX (2008) defines the comparison function like this:
-  #define scandir(a,b,c,d) scandir((a), (b), (c), (int(*)(const dirent **, const dirent **))(d));
-#else
-  #define scandir(a,b,c,d) scandir((a), (b), (c), (int(*)(const void*,const void*))(d));
-#endif
-
-typedef std::string CompString;
-typedef std::list<CompString> CompStringList;
-
-CompString compPrintf (const char *format, ...);
-CompString compPrintf (const char *format, va_list ap);
-
-extern bool debugOutput;
-
-typedef enum {
-    CompLogLevelFatal = 0,
-    CompLogLevelError,
-    CompLogLevelWarn,
-    CompLogLevelInfo,
-    CompLogLevelDebug
-} CompLogLevel;
-
-void
-logMessage (const char   *componentName,
-	    CompLogLevel level,
-	    const char   *message);
-
-void
-compLogMessage (const char   *componentName,
-		CompLogLevel level,
-		const char   *format,
-		...);
-
-const char *
-logLevelToString (CompLogLevel level);
-
-extern char       *programName;
-extern char       **programArgv;
-extern int        programArgc;
-
-#endif

=== modified file 'include/core/CMakeLists.txt'
--- include/core/CMakeLists.txt	2011-07-07 16:59:41 +0000
+++ include/core/CMakeLists.txt	2012-01-12 06:48:58 +0000
@@ -10,8 +10,6 @@
     output.h
     plugin.h
     point.h
-    pluginclasshandler.h
-    pluginclasses.h
     propertywriter.h
     privateunion.h
     rect.h
@@ -20,9 +18,6 @@
     serialization.h
     session.h
     size.h
-    timeouthandler.h
-    timer.h
-    valueholder.h
     window.h
     wrapsystem.h
 )

=== modified file 'include/core/core.h'
--- include/core/core.h	2011-10-13 09:53:38 +0000
+++ include/core/core.h	2012-01-12 06:48:58 +0000
@@ -32,6 +32,10 @@
 #include <stdio.h>
 #include <assert.h>
 
+#include <string>
+#include <list>
+#include <cstdarg>
+
 #include <X11/Xlib-xcb.h>
 #include <X11/Xutil.h>
 #include <X11/extensions/Xdamage.h>
@@ -69,6 +73,19 @@
 #  define BITMAP_BIT_ORDER LSBFirst
 #endif
 
+#define STRINGIFY(x) #x
+#define TOSTRING(x) STRINGIFY (x)
+#define DEG2RAD (M_PI / 180.0f)
+
+#if defined(HAVE_SCANDIR_POSIX)
+  // POSIX (2008) defines the comparison function like this:
+  #define scandir(a,b,c,d) scandir((a), (b), (c), (int(*)(const dirent **, const dirent **))(d));
+#else
+  #define scandir(a,b,c,d) scandir((a), (b), (c), (int(*)(const void*,const void*))(d));
+#endif
+
+#include <core/global.h>
+
 #include <core/pluginclasses.h>
 #include <core/screen.h>
 #include <core/window.h>
@@ -85,5 +102,7 @@
 #include <core/region.h>
 #include <core/countedlist.h>
 #include <core/timeouthandler.h>
+#include <core/logmessage.h>
+#include <core/string.h>
 
 #endif

=== added file 'include/core/global.h'
--- include/core/global.h	1970-01-01 00:00:00 +0000
+++ include/core/global.h	2011-12-19 07:06:22 +0000
@@ -0,0 +1,37 @@
+/*
+ * Copyright © 2007 Novell, Inc.
+ *
+ * Permission to use, copy, modify, distribute, and sell this software
+ * and its documentation for any purpose is hereby granted without
+ * fee, provided that the above copyright notice appear in all copies
+ * and that both that copyright notice and this permission notice
+ * appear in supporting documentation, and that the name of
+ * Novell, Inc. not be used in advertising or publicity pertaining to
+ * distribution of the software without specific, written prior permission.
+ * Novell, Inc. makes no representations about the suitability of this
+ * software for any purpose. It is provided "as is" without express or
+ * implied warranty.
+ *
+ * NOVELL, INC. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
+ * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
+ * NO EVENT SHALL NOVELL, INC. BE LIABLE FOR ANY SPECIAL, INDIRECT OR
+ * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
+ * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
+ * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
+ * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+ * Author: David Reveman <davidr@novell.com>
+ */
+#ifndef _COMPIZ_GLOBAL_H
+#define _COMPIZ_GLOBAL_H
+
+/**
+* Flag indicating whether to produce debug output
+*/
+extern bool debugOutput;
+
+extern char       *programName;
+extern char       **programArgv;
+extern int        programArgc;
+
+#endif // _COMPIZ_GLOBAL_H

=== added file 'include/core/logmessage.h'
--- include/core/logmessage.h	1970-01-01 00:00:00 +0000
+++ include/core/logmessage.h	2011-12-19 07:06:22 +0000
@@ -0,0 +1,51 @@
+/*
+ * Copyright © 2007 Novell, Inc.
+ *
+ * Permission to use, copy, modify, distribute, and sell this software
+ * and its documentation for any purpose is hereby granted without
+ * fee, provided that the above copyright notice appear in all copies
+ * and that both that copyright notice and this permission notice
+ * appear in supporting documentation, and that the name of
+ * Novell, Inc. not be used in advertising or publicity pertaining to
+ * distribution of the software without specific, written prior permission.
+ * Novell, Inc. makes no representations about the suitability of this
+ * software for any purpose. It is provided "as is" without express or
+ * implied warranty.
+ *
+ * NOVELL, INC. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
+ * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
+ * NO EVENT SHALL NOVELL, INC. BE LIABLE FOR ANY SPECIAL, INDIRECT OR
+ * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
+ * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
+ * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
+ * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+ * Author: David Reveman <davidr@novell.com>
+ */
+
+#ifndef _COMPIZ_LOGMESSAGE_H
+#define _COMPIZ_LOGMESSAGE_H
+
+typedef enum {
+    CompLogLevelFatal = 0,
+    CompLogLevelError,
+    CompLogLevelWarn,
+    CompLogLevelInfo,
+    CompLogLevelDebug
+} CompLogLevel;
+
+void
+logMessage (const char   *componentName,
+	    CompLogLevel level,
+	    const char   *message);
+
+void
+compLogMessage (const char   *componentName,
+		CompLogLevel level,
+		const char   *format,
+		...);
+
+const char *
+logLevelToString (CompLogLevel level);
+
+#endif

=== modified file 'include/core/match.h'
--- include/core/match.h	2010-03-24 01:31:41 +0000
+++ include/core/match.h	2012-01-12 06:48:58 +0000
@@ -28,6 +28,7 @@
 #ifndef _COMPMATCH_H
 #define _COMPMATCH_H
 
+#include <core/string.h>
 #include <core/core.h>
 
 class PrivateMatch;

=== modified file 'include/core/option.h'
--- include/core/option.h	2011-03-16 19:39:25 +0000
+++ include/core/option.h	2012-01-12 06:48:58 +0000
@@ -28,8 +28,7 @@
 #ifndef _COMPOPTION_H
 #define _COMPOPTION_H
 
-#include <compiz.h>
-
+#include <core/string.h>
 #include <vector>
 
 class PrivateOption;

=== modified file 'include/core/plugin.h'
--- include/core/plugin.h	2011-06-25 02:31:02 +0000
+++ include/core/plugin.h	2012-01-12 06:48:58 +0000
@@ -26,7 +26,7 @@
 #ifndef _COMPIZ_PLUGIN_H
 #define _COMPIZ_PLUGIN_H
 
-#include <compiz.h>
+#include <core/string.h>
 #include <core/option.h>
 #include <core/privateunion.h>
 

=== modified file 'include/core/region.h'
--- include/core/region.h	2010-07-25 03:46:58 +0000
+++ include/core/region.h	2012-01-12 06:48:58 +0000
@@ -74,7 +74,7 @@
 	/**
 	 * Returns the internal XRegion handle
 	 */
-	const Region handle () const;
+	Region handle () const;
 
 	/**
 	 * Returns true if the specified CompPoint falls within the

=== modified file 'include/core/session.h'
--- include/core/session.h	2009-03-05 19:42:59 +0000
+++ include/core/session.h	2012-01-12 06:48:58 +0000
@@ -28,6 +28,7 @@
 
 #define SN_API_NOT_YET_FROZEN
 #include <libsn/sn.h>
+#include <core/string.h>
 
 namespace CompSession {
 

=== added file 'include/core/string.h'
--- include/core/string.h	1970-01-01 00:00:00 +0000
+++ include/core/string.h	2011-10-31 13:51:00 +0000
@@ -0,0 +1,39 @@
+/*
+ * Copyright © 2007 Novell, Inc.
+ *
+ * Permission to use, copy, modify, distribute, and sell this software
+ * and its documentation for any purpose is hereby granted without
+ * fee, provided that the above copyright notice appear in all copies
+ * and that both that copyright notice and this permission notice
+ * appear in supporting documentation, and that the name of
+ * Novell, Inc. not be used in advertising or publicity pertaining to
+ * distribution of the software without specific, written prior permission.
+ * Novell, Inc. makes no representations about the suitability of this
+ * software for any purpose. It is provided "as is" without express or
+ * implied warranty.
+ *
+ * NOVELL, INC. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
+ * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
+ * NO EVENT SHALL NOVELL, INC. BE LIABLE FOR ANY SPECIAL, INDIRECT OR
+ * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
+ * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
+ * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
+ * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+ * Author: David Reveman <davidr@novell.com>
+ */
+
+#ifndef _COMPIZ_STRING_H
+#define _COMPIZ_STRING_H
+
+#include <string>
+#include <list>
+#include <cstdarg>
+
+typedef std::string CompString;
+typedef std::list<CompString> CompStringList;
+
+CompString compPrintf (const char *format, ...);
+CompString compPrintf (const char *format, va_list ap);
+
+#endif

=== modified file 'kde/window-decorator-kde4/window.cpp'
--- kde/window-decorator-kde4/window.cpp	2011-09-03 07:56:45 +0000
+++ kde/window-decorator-kde4/window.cpp	2012-01-12 06:48:58 +0000
@@ -1043,10 +1043,10 @@
 {
     Atom    atom = Atoms::compizWindowBlurDecor;
     QRegion topQRegion, bottomQRegion, leftQRegion, rightQRegion;
-    Region  topRegion = NULL;
-    Region  bottomRegion = NULL;
-    Region  leftRegion = NULL;
-    Region  rightRegion = NULL;
+    ::Region  topRegion = NULL;
+    ::Region  bottomRegion = NULL;
+    ::Region  leftRegion = NULL;
+    ::Region  rightRegion = NULL;
     int     size = 0;
     int     w, h;
 

=== modified file 'metadata/CMakeLists.txt'
--- metadata/CMakeLists.txt	2011-08-18 18:15:11 +0000
+++ metadata/CMakeLists.txt	2012-01-12 06:48:58 +0000
@@ -9,9 +9,9 @@
 if (XSLTPROC_EXECUTABLE)
 
     add_custom_command (
-	OUTPUT ${CMAKE_BINARY_DIR}/generated/core.xml.in
+	OUTPUT ${compiz_BINARY_DIR}/generated/core.xml.in
 	COMMAND ${XSLTPROC_EXECUTABLE}
-		-o ${CMAKE_BINARY_DIR}/generated/core.xml.in
+		-o ${compiz_BINARY_DIR}/generated/core.xml.in
 		--param default_plugins "\"'${COMPIZ_DEFAULT_PLUGINS}'\""
 		${CMAKE_SOURCE_DIR}/xslt/default_plugins.xslt
 		${CMAKE_CURRENT_SOURCE_DIR}/core.xml.in

=== modified file 'plugins/CMakeLists.txt'
--- plugins/CMakeLists.txt	2010-11-12 04:35:58 +0000
+++ plugins/CMakeLists.txt	2012-01-12 06:48:58 +0000
@@ -7,4 +7,14 @@
 
 add_definitions ( -DHAVE_CONFIG_H)
 
+include_directories( 
+  ${CMAKE_CURRENT_SOURCE_DIR}/../include
+  
+  ${CMAKE_CURRENT_SOURCE_DIR}/../src
+  ${CMAKE_CURRENT_SOURCE_DIR}/../src/timer/include
+  ${CMAKE_CURRENT_SOURCE_DIR}/../src/string/include
+  ${CMAKE_CURRENT_SOURCE_DIR}/../src/pluginclasshandler/include
+  ${CMAKE_CURRENT_SOURCE_DIR}/../logmessage/include
+)
+
 compiz_add_plugins_in_folder (${CMAKE_CURRENT_SOURCE_DIR})

=== modified file 'plugins/composite/src/screen.cpp'
--- plugins/composite/src/screen.cpp	2011-12-19 08:54:57 +0000
+++ plugins/composite/src/screen.cpp	2012-01-12 06:48:58 +0000
@@ -43,6 +43,8 @@
 #include <X11/extensions/shape.h>
 #include <X11/extensions/Xrandr.h>
 
+#include <core/timer.h>
+
 CompWindow *lastDamagedWindow = 0;
 
 void
@@ -703,7 +705,7 @@
     {
 	struct timeval now;
 	gettimeofday (&now, 0);
-	int elapsed = TIMEVALDIFF (&now, &lastRedraw);
+	int elapsed = compiz::core::timer::timeval_diff (&now, &lastRedraw);
 	if (elapsed < 0)
 	    elapsed = 0;
  	delay = elapsed < optimalRedrawTime ? optimalRedrawTime - elapsed : 1;
@@ -750,7 +752,7 @@
 	if (priv->pHnd)
 	    priv->pHnd->prepareDrawing ();
 
-	timeDiff = TIMEVALDIFF (&tv, &priv->lastRedraw);
+	timeDiff = compiz::core::timer::timeval_diff (&tv, &priv->lastRedraw);
 
 	/* handle clock rollback */
 	if (timeDiff < 0)

=== modified file 'plugins/decor/src/decor.cpp'
--- plugins/decor/src/decor.cpp	2011-10-26 23:18:47 +0000
+++ plugins/decor/src/decor.cpp	2012-01-12 06:48:58 +0000
@@ -1384,6 +1384,7 @@
 {
     Decoration	     *old, *decoration = NULL;
     bool	     decorate = false;
+    bool	     shadowOnly = true;
     int		     moveDx, moveDy;
     int		     oldShiftX = 0;
     int		     oldShiftY  = 0;
@@ -1409,28 +1410,31 @@
 	    case CompWindowTypeMenuMask:
 	    case CompWindowTypeNormalMask:
 		if (window->mwmDecor () & (MwmDecorAll | MwmDecorTitle))
-		    decorate = (window->frame () ||
-				window->hasUnmapReference ()) ? true : false;
+		    shadowOnly = false;
 	    default:
 		break;
 	}
 
 	if (window->overrideRedirect ())
-	    decorate = false;
+	    shadowOnly = true;
 
 	if (window->wmType () & (CompWindowTypeDockMask | CompWindowTypeDesktopMask))
-	    decorate = false;
+	    shadowOnly = true;
 
-	if (decorate)
+	if (!shadowOnly)
 	{
 	    if (!dScreen->optionGetDecorationMatch ().evaluate (window))
-		decorate = false;
+		shadowOnly = true;
 	}
     }
     else
-	decorate = true;
-
-    if (decorate)
+	shadowOnly = false;
+
+    decorate = ((window->frame () ||
+		 window->hasUnmapReference ()) && !shadowOnly) ||
+		 isSwitcher;
+
+    if (decorate || frameExtentsRequested)
     {
         /* Attempt to find a matching decoration */
         decoration = decor.findMatchingDecoration (window, true);
@@ -1451,6 +1455,11 @@
 	    else if (dScreen->dmSupports & WINDOW_DECORATION_TYPE_WINDOW)
 		decoration = &dScreen->windowDefault;
 	}
+
+	/* Do not allow windows which are later undecorated
+	 * to have a set _NET_FRAME_EXTENTS */
+	if (decorate)
+	    frameExtentsRequested = false;
     }
     else
     {
@@ -1531,7 +1540,8 @@
 	moveDy = shiftY () - oldShiftY;
 
 	/* Update the input and output frame */
-	updateFrame ();
+	if (decorate)
+	    updateFrame ();
 	window->updateWindowOutputExtents ();
 	if (dScreen->cmActive)
 	    cWindow->damageOutputExtents ();
@@ -1545,7 +1555,8 @@
 	/* Undecorated windows need to have the
 	 * input and output frame removed and the
 	 * frame window geometry reset */
-	updateFrame ();
+	if (decorate)
+	    updateFrame ();
 
 	memset (&emptyExtents, 0, sizeof (CompWindowExtents));
 
@@ -2056,6 +2067,8 @@
 	this->dmSupports = dmSupports;
 
 	/* Create new default decorations */
+	screen->updateSupportedWmHints ();
+
 	if (dmWin)
 	{
 	    for (i = 0; i < DECOR_NUM; i++)
@@ -2849,7 +2862,10 @@
 {
     screen->addSupportedAtoms (atoms);
 
-    atoms.push_back (requestFrameExtentsAtom);
+    /* Don't support _NET_REQUEST_FRAME_EXTENTS
+     * where there is no decorator running yet */
+    if (dmWin)
+	atoms.push_back (requestFrameExtentsAtom);
 }
 
 void
@@ -3024,7 +3040,8 @@
     updateReg (true),
     unshading (false),
     shading (false),
-    isSwitcher (false)
+    isSwitcher (false),
+    frameExtentsRequested (false)
 {
     WindowInterface::setHandler (window);
 

=== modified file 'plugins/decor/src/decor.h'
--- plugins/decor/src/decor.h	2011-10-15 11:00:51 +0000
+++ plugins/decor/src/decor.h	2012-01-12 06:48:58 +0000
@@ -274,6 +274,8 @@
 	bool	  unshading;
 	bool	  shading;
 	bool	  isSwitcher;
+
+	bool      frameExtentsRequested;
 };
 
 class DecorPluginVTable :

=== modified file 'plugins/kde/src/dispatcher.cpp'
--- plugins/kde/src/dispatcher.cpp	2010-08-20 12:52:54 +0000
+++ plugins/kde/src/dispatcher.cpp	2012-01-12 06:48:58 +0000
@@ -31,7 +31,7 @@
 
 #include "dispatcher.moc"
 
-EventDispatcherCompiz::EventDispatcherCompiz (QObject *parent)
+EventDispatcherCompiz::EventDispatcherCompiz (QObject *)
 {
     mWakeUpPipe[0] = 0;
     mWakeUpPipe[1] = 0;

=== modified file 'plugins/move/src/move.cpp'
--- plugins/move/src/move.cpp	2011-10-15 11:00:51 +0000
+++ plugins/move/src/move.cpp	2012-01-12 06:48:58 +0000
@@ -708,6 +708,7 @@
 {
     hasCompositing = true;
     cScreen->registerPaintHandler (pHnd);
+    return true;
 }
 
 void

=== modified file 'plugins/opengl/include/opengl/opengl.h'
--- plugins/opengl/include/opengl/opengl.h	2011-10-15 11:02:37 +0000
+++ plugins/opengl/include/opengl/opengl.h	2012-01-12 06:48:58 +0000
@@ -239,8 +239,6 @@
 {
     public:
 
-	virtual bool glInitContext (XVisualInfo *);
-
 	/**
 	 * Hookable function used for plugins to use openGL to draw on an output
 	 *
@@ -385,20 +383,21 @@
 	 */
 	const float * projectionMatrix ();
 
-	WRAPABLE_HND (0, GLScreenInterface, bool, glInitContext, XVisualInfo *);
-	WRAPABLE_HND (1, GLScreenInterface, bool, glPaintOutput,
+	bool glInitContext (XVisualInfo *);
+
+	WRAPABLE_HND (0, GLScreenInterface, bool, glPaintOutput,
 		      const GLScreenPaintAttrib &, const GLMatrix &,
 		      const CompRegion &, CompOutput *, unsigned int);
-	WRAPABLE_HND (2, GLScreenInterface, void, glPaintTransformedOutput,
+	WRAPABLE_HND (1, GLScreenInterface, void, glPaintTransformedOutput,
 		      const GLScreenPaintAttrib &,
 		      const GLMatrix &, const CompRegion &, CompOutput *,
 		      unsigned int);
-	WRAPABLE_HND (3, GLScreenInterface, void, glApplyTransform,
+	WRAPABLE_HND (2, GLScreenInterface, void, glApplyTransform,
 		      const GLScreenPaintAttrib &, CompOutput *, GLMatrix *);
 
-	WRAPABLE_HND (4, GLScreenInterface, void, glEnableOutputClipping,
+	WRAPABLE_HND (3, GLScreenInterface, void, glEnableOutputClipping,
 		      const GLMatrix &, const CompRegion &, CompOutput *);
-	WRAPABLE_HND (5, GLScreenInterface, void, glDisableOutputClipping);
+	WRAPABLE_HND (4, GLScreenInterface, void, glDisableOutputClipping);
 
 	friend class GLTexture;
 

=== modified file 'plugins/opengl/src/matrix.cpp'
--- plugins/opengl/src/matrix.cpp	2010-10-24 14:42:07 +0000
+++ plugins/opengl/src/matrix.cpp	2012-01-12 06:48:58 +0000
@@ -46,7 +46,6 @@
  * From Mesa 3-D graphics library.
  */
 
-#include <compiz.h>
 #include <string.h>
 #include <math.h>
 #include <core/core.h>

=== modified file 'plugins/opengl/src/privates.h'
--- plugins/opengl/src/privates.h	2011-10-26 23:15:02 +0000
+++ plugins/opengl/src/privates.h	2012-01-12 06:48:58 +0000
@@ -124,7 +124,6 @@
 	bool hasCompositing;
 
 	GLIcon defaultIcon;
-	CompTimer initContext;
 
 	Pixmap rootPixmapCopy;
 	CompSize rootPixmapSize;

=== modified file 'plugins/opengl/src/screen.cpp'
--- plugins/opengl/src/screen.cpp	2011-12-07 09:48:24 +0000
+++ plugins/opengl/src/screen.cpp	2012-01-12 06:48:58 +0000
@@ -96,7 +96,6 @@
     GLfloat		 light0Position[] = { -0.5f, 0.5f, -9.0f, 1.0f };
     const char           *glRenderer;
     CompOption::Vector o (0);
-    WRAPABLE_HND_FUNC_RETURN (0, bool, glInitContext, visinfo);
 
     priv->ctx = glXCreateContext (dpy, visinfo, NULL, !indirectRendering);
     if (!priv->ctx)
@@ -285,7 +284,7 @@
     if (GL::textureFromPixmap)
 	registerBindPixmap (TfpTexture::bindPixmapToTexture);
 
-    return false;
+    return true;
 }
 
 
@@ -549,10 +548,10 @@
 			"this isn't going to work.");
 	screen->handleCompizEvent ("opengl", "fatal_fallback", o);
 	setFailed ();
-	return;
     }
 
-    priv->initContext.start (boost::bind (&GLScreen::glInitContext, this, visinfo), 0, 0);
+    if (!glInitContext (visinfo))
+	setFailed ();
 }
 
 GLScreen::~GLScreen ()
@@ -958,10 +957,6 @@
 }
 
 bool
-GLScreenInterface::glInitContext (XVisualInfo *visinfo)
-    WRAPABLE_DEF (glInitContext, visinfo);
-
-bool
 GLScreenInterface::glPaintOutput (const GLScreenPaintAttrib &sAttrib,
 			          const GLMatrix            &transform,
 			          const CompRegion          &region,

=== modified file 'plugins/opengl/src/texture.cpp'
--- plugins/opengl/src/texture.cpp	2011-02-24 07:52:09 +0000
+++ plugins/opengl/src/texture.cpp	2012-01-12 06:48:58 +0000
@@ -27,8 +27,6 @@
 #  include <config.h>
 #endif
 
-#include <compiz.h>
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>

=== modified file 'po/POTFILES.in'
--- po/POTFILES.in	2009-10-18 20:23:24 +0000
+++ po/POTFILES.in	2012-01-12 06:48:58 +0000
@@ -1,7 +1,4 @@
-gtk/gnome/50-compiz-desktop-key.xml.in
-gtk/gnome/50-compiz-key.xml.in
 gtk/gnome/compiz.desktop.in
-gtk/gnome/compiz-wm.desktop.in
 gtk/window-decorator/gwd.schemas.in
 gtk/window-decorator/gtk-window-decorator.c
 metadata/core.xml.in

=== modified file 'src/CMakeLists.txt'
--- src/CMakeLists.txt	2011-09-19 12:54:22 +0000
+++ src/CMakeLists.txt	2012-01-12 06:48:58 +0000
@@ -1,19 +1,45 @@
 include (CompizBcop)
 
+
+add_subdirectory( string )
+add_subdirectory( logmessage )
+add_subdirectory( timer )
+add_subdirectory( pluginclasshandler )
+
 compiz_add_bcop_targets (
     core
     ${compiz_BINARY_DIR}/generated/core.xml.in
     _bcop_sources
 )
 
+get_property (CORE_MOD_INCLUDE_DIRS
+	      GLOBAL
+	      PROPERTY CORE_MOD_INCLUDE_DIRS)
+
 include_directories (
     ${compiz_SOURCE_DIR}/include
     ${compiz_BINARY_DIR}
     ${compiz_BINARY_DIR}/generated
-    ${CMAKE_CURRENT_SOURCE_DIR}
     ${COMPIZ_INCLUDE_DIRS}
     ${CMAKE_PREFIX_PATH}/include
     ${CMAKE_INCLUDE_PATH}
+    ${CORE_MOD_INCLUDE_DIRS}
+    
+    ${CMAKE_CURRENT_SOURCE_DIR}
+    
+    # Module specific include dirs
+    ${CMAKE_CURRENT_SOURCE_DIR}/string/include
+    ${CMAKE_CURRENT_SOURCE_DIR}/string/src
+    
+    ${CMAKE_CURRENT_SOURCE_DIR}/logmessage/include
+    ${CMAKE_CURRENT_SOURCE_DIR}/logmessage/src
+    
+    ${CMAKE_CURRENT_SOURCE_DIR}/timer/include
+    ${CMAKE_CURRENT_SOURCE_DIR}/timer/src
+    
+    ${CMAKE_CURRENT_SOURCE_DIR}/pluginclasshandler/include
+    ${CMAKE_CURRENT_SOURCE_DIR}/pluginclasshandler/src
+    
 )
 
 add_definitions (
@@ -23,47 +49,66 @@
     -DMETADATADIR=\\\"${compiz_metadatadir}\\\"
 )
 
+get_property (CORE_MOD_LIBRARY_DIRS
+	      GLOBAL
+	      PROPERTY CORE_MOD_LIBRARY_DIRS)
+
 link_directories (
     ${COMPIZ_LINK_DIRS}
+    ${CORE_MOD_LIBRARY_DIRS}
 )
 
 add_executable (compiz
-    region.cpp
-    atoms.cpp
-    timer.cpp
-    main.cpp
-    actions.cpp
-    screen.cpp
-    window.cpp
-    action.cpp
-    option.cpp
-    string.cpp
-    match.cpp
-    pluginclasses.cpp
-    event.cpp
-    plugin.cpp
-    session.cpp
-    output.cpp
-    rect.cpp
-    size.cpp
-    point.cpp
-    windowgeometry.cpp
-    icon.cpp
-    modifierhandler.cpp
-    propertywriter.cpp
-    eventsource.cpp
-    timeouthandler.cpp
-    valueholder.cpp
-    logmessage.cpp
-    stackdebugger.cpp
+    ${CMAKE_CURRENT_SOURCE_DIR}/region.cpp
+    ${CMAKE_CURRENT_SOURCE_DIR}/atoms.cpp
+    ${CMAKE_CURRENT_SOURCE_DIR}/main.cpp
+    ${CMAKE_CURRENT_SOURCE_DIR}/actions.cpp
+    ${CMAKE_CURRENT_SOURCE_DIR}/screen.cpp
+    ${CMAKE_CURRENT_SOURCE_DIR}/window.cpp
+    ${CMAKE_CURRENT_SOURCE_DIR}/action.cpp
+    ${CMAKE_CURRENT_SOURCE_DIR}/option.cpp
+    ${CMAKE_CURRENT_SOURCE_DIR}/match.cpp
+    ${CMAKE_CURRENT_SOURCE_DIR}/event.cpp
+    ${CMAKE_CURRENT_SOURCE_DIR}/plugin.cpp
+    ${CMAKE_CURRENT_SOURCE_DIR}/session.cpp
+    ${CMAKE_CURRENT_SOURCE_DIR}/output.cpp
+    ${CMAKE_CURRENT_SOURCE_DIR}/rect.cpp
+    ${CMAKE_CURRENT_SOURCE_DIR}/size.cpp
+    ${CMAKE_CURRENT_SOURCE_DIR}/point.cpp
+    ${CMAKE_CURRENT_SOURCE_DIR}/windowgeometry.cpp
+    ${CMAKE_CURRENT_SOURCE_DIR}/icon.cpp
+    ${CMAKE_CURRENT_SOURCE_DIR}/modifierhandler.cpp
+    ${CMAKE_CURRENT_SOURCE_DIR}/propertywriter.cpp
+    ${CMAKE_CURRENT_SOURCE_DIR}/eventsource.cpp
+    ${CMAKE_CURRENT_SOURCE_DIR}/stackdebugger.cpp
+    
     ${_bcop_sources}
 )
 
+# workaround for build race
+add_dependencies (compiz core-xml-file)
+
+get_property (CORE_MOD_LIBRARIES
+	      GLOBAL
+	      PROPERTY CORE_MOD_LIBRARIES)
+
 target_link_libraries (
-    compiz ${COMPIZ_LIBRARIES} m pthread dl
+    compiz ${COMPIZ_LIBRARIES} 
+    
+    m 
+    pthread 
+    dl
+    
+    compiz_string
+    compiz_timer
+    compiz_logmessage
+    compiz_pluginclasshandler
+#    ${CORE_MOD_LIBRARIES}
 )
 
 install (
     TARGETS compiz
     DESTINATION ${COMPIZ_DESTDIR}${exec_prefix}
 )
+
+enable_coverage_report( TARGETS compiz )

=== modified file 'src/action.cpp'
--- src/action.cpp	2010-11-09 14:13:19 +0000
+++ src/action.cpp	2012-01-12 06:48:58 +0000
@@ -27,8 +27,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <compiz.h>
-
 #include <core/option.h>
 #include <core/action.h>
 #include <core/screen.h>

=== added directory 'src/logmessage'
=== added file 'src/logmessage/CMakeLists.txt'
--- src/logmessage/CMakeLists.txt	1970-01-01 00:00:00 +0000
+++ src/logmessage/CMakeLists.txt	2011-12-19 07:06:22 +0000
@@ -0,0 +1,37 @@
+
+INCLUDE_DIRECTORIES(  
+  ${CMAKE_CURRENT_SOURCE_DIR}/include
+  
+  ${compiz_SOURCE_DIR}/include
+    
+  ${Boost_INCLUDE_DIRS}
+)
+
+SET( PUBLIC_HEADERS ${CMAKE_CURRENT_SOURCE_DIR}/include/core/logmessage.h )
+SET( PRIVATE_HEADERS )
+
+SET( SRCS ${CMAKE_CURRENT_SOURCE_DIR}/src/logmessage.cpp )
+
+ADD_LIBRARY( 
+  compiz_logmessage STATIC
+  
+  ${SRCS}
+  
+  ${PUBLIC_HEADERS}
+  ${PRIVATE_HEADERS}
+)
+
+ADD_SUBDIRECTORY( ${CMAKE_CURRENT_SOURCE_DIR}/tests )
+
+SET_TARGET_PROPERTIES(
+  compiz_logmessage PROPERTIES
+  PUBLIC_HEADER "${PUBLIC_HEADERS}"
+)
+
+INSTALL(
+  TARGETS                                 compiz_logmessage
+  RUNTIME             DESTINATION         bin
+  LIBRARY             DESTINATION         lib
+  ARCHIVE             DESTINATION         lib
+  PUBLIC_HEADER       DESTINATION         include/compiz
+)

=== added directory 'src/logmessage/include'
=== added directory 'src/logmessage/include/core'
=== added file 'src/logmessage/include/core/logmessage.h'
--- src/logmessage/include/core/logmessage.h	1970-01-01 00:00:00 +0000
+++ src/logmessage/include/core/logmessage.h	2011-10-31 13:51:00 +0000
@@ -0,0 +1,51 @@
+/*
+ * Copyright © 2007 Novell, Inc.
+ *
+ * Permission to use, copy, modify, distribute, and sell this software
+ * and its documentation for any purpose is hereby granted without
+ * fee, provided that the above copyright notice appear in all copies
+ * and that both that copyright notice and this permission notice
+ * appear in supporting documentation, and that the name of
+ * Novell, Inc. not be used in advertising or publicity pertaining to
+ * distribution of the software without specific, written prior permission.
+ * Novell, Inc. makes no representations about the suitability of this
+ * software for any purpose. It is provided "as is" without express or
+ * implied warranty.
+ *
+ * NOVELL, INC. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
+ * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
+ * NO EVENT SHALL NOVELL, INC. BE LIABLE FOR ANY SPECIAL, INDIRECT OR
+ * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
+ * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
+ * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
+ * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+ * Author: David Reveman <davidr@novell.com>
+ */
+
+#ifndef _COMPIZ_LOGMESSAGE_H
+#define _COMPIZ_LOGMESSAGE_H
+
+typedef enum {
+    CompLogLevelFatal = 0,
+    CompLogLevelError,
+    CompLogLevelWarn,
+    CompLogLevelInfo,
+    CompLogLevelDebug
+} CompLogLevel;
+
+void
+logMessage (const char   *componentName,
+	    CompLogLevel level,
+	    const char   *message);
+
+void
+compLogMessage (const char   *componentName,
+		CompLogLevel level,
+		const char   *format,
+		...);
+
+const char *
+logLevelToString (CompLogLevel level);
+
+#endif

=== added directory 'src/logmessage/src'
=== renamed file 'src/logmessage.cpp' => 'src/logmessage/src/logmessage.cpp'
--- src/logmessage.cpp	2011-07-05 19:49:12 +0000
+++ src/logmessage/src/logmessage.cpp	2012-01-12 06:48:58 +0000
@@ -23,9 +23,13 @@
  * Author: David Reveman <davidr@novell.com>
  */
 
-#include <compiz.h>
+#include <core/global.h>
+#include <core/logmessage.h>
+
 #include <cstdio>
 
+#include <stdarg.h>
+
 const char *
 logLevelToString (CompLogLevel level)
 {

=== added directory 'src/logmessage/tests'
=== added file 'src/logmessage/tests/CMakeLists.txt'
=== added file 'src/logmessage/tests/test-logmessage.cpp'
=== modified file 'src/main.cpp'
--- src/main.cpp	2011-10-13 14:46:33 +0000
+++ src/main.cpp	2012-01-12 06:48:58 +0000
@@ -27,8 +27,6 @@
 #  include <config.h>
 #endif
 
-#include <compiz.h>
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>

=== renamed directory 'tests/pluginclasshandler' => 'src/pluginclasshandler'
=== added file 'src/pluginclasshandler/CMakeLists.txt'
--- src/pluginclasshandler/CMakeLists.txt	1970-01-01 00:00:00 +0000
+++ src/pluginclasshandler/CMakeLists.txt	2011-12-19 07:06:22 +0000
@@ -0,0 +1,50 @@
+
+INCLUDE_DIRECTORIES(  
+  ${CMAKE_CURRENT_SOURCE_DIR}/include
+  ${CMAKE_CURRENT_SOURCE_DIR}../logmessage/include
+  
+  ${compiz_SOURCE_DIR}/include
+    
+  ${Boost_INCLUDE_DIRS}
+)
+
+SET( 
+  PUBLIC_HEADERS 
+  ${CMAKE_CURRENT_SOURCE_DIR}/include/core/pluginclasses.h 
+  ${CMAKE_CURRENT_SOURCE_DIR}/include/core/pluginclasshandler.h
+)
+
+SET( 
+  PRIVATE_HEADERS 
+  ${CMAKE_CURRENT_SOURCE_DIR}/src/valueholder.h
+)
+
+SET( 
+  SRCS 
+  ${CMAKE_CURRENT_SOURCE_DIR}/src/valueholder.cpp 
+  ${CMAKE_CURRENT_SOURCE_DIR}/src/pluginclasses.cpp
+)
+
+ADD_LIBRARY( 
+  compiz_pluginclasshandler STATIC
+  
+  ${SRCS}
+  
+  ${PUBLIC_HEADERS}
+  ${PRIVATE_HEADERS}
+)
+
+ADD_SUBDIRECTORY( ${CMAKE_CURRENT_SOURCE_DIR}/tests )
+
+SET_TARGET_PROPERTIES(
+  compiz_pluginclasshandler PROPERTIES
+  PUBLIC_HEADER "${PUBLIC_HEADERS}"
+)
+
+INSTALL(
+  TARGETS                                 compiz_pluginclasshandler
+  RUNTIME             DESTINATION         bin
+  LIBRARY             DESTINATION         lib
+  ARCHIVE             DESTINATION         lib
+  PUBLIC_HEADER       DESTINATION         include/compiz
+)

=== added directory 'src/pluginclasshandler/include'
=== added directory 'src/pluginclasshandler/include/core'
=== renamed file 'include/core/pluginclasses.h' => 'src/pluginclasshandler/include/core/pluginclasses.h'
=== renamed file 'include/core/pluginclasshandler.h' => 'src/pluginclasshandler/include/core/pluginclasshandler.h'
--- include/core/pluginclasshandler.h	2011-09-16 00:51:33 +0000
+++ src/pluginclasshandler/include/core/pluginclasshandler.h	2012-01-12 06:48:58 +0000
@@ -29,7 +29,7 @@
 #include <typeinfo>
 #include <boost/preprocessor/cat.hpp>
 
-#include <compiz.h>
+#include <core/string.h>
 #include <core/valueholder.h>
 #include <core/pluginclasses.h>
 

=== renamed file 'include/core/valueholder.h' => 'src/pluginclasshandler/include/core/valueholder.h'
--- include/core/valueholder.h	2011-06-25 02:31:02 +0000
+++ src/pluginclasshandler/include/core/valueholder.h	2012-01-12 06:48:58 +0000
@@ -26,7 +26,8 @@
 #ifndef _COMPIZ_VALUEHOLDER_H
 #define _COMPIZ_VALUEHOLDER_H
 
-#include <compiz.h>
+#include <core/string.h>
+#include <core/logmessage.h>
 #include <core/privateunion.h>
 
 class PrivateValueHolder;
@@ -50,4 +51,4 @@
 	PrivateValueHolder *priv;
 };
 
-#endif
\ No newline at end of file
+#endif

=== added directory 'src/pluginclasshandler/src'
=== renamed file 'src/pluginclasses.cpp' => 'src/pluginclasshandler/src/pluginclasses.cpp'
=== renamed file 'src/valueholder.cpp' => 'src/pluginclasshandler/src/valueholder.cpp'
=== renamed file 'src/valueholder.h' => 'src/pluginclasshandler/src/valueholder.h'
=== added directory 'src/pluginclasshandler/tests'
=== renamed file 'tests/pluginclasshandler/CMakeLists.txt' => 'src/pluginclasshandler/tests/CMakeLists.txt'
--- tests/pluginclasshandler/CMakeLists.txt	2011-07-06 18:53:03 +0000
+++ src/pluginclasshandler/tests/CMakeLists.txt	2012-01-12 06:48:58 +0000
@@ -1,6 +1,81 @@
-set (COMPIZ_TEST_PCH_BASE_DIR ${CMAKE_CURRENT_SOURCE_DIR})
-
-add_subdirectory (construct)
-add_subdirectory (get)
-add_subdirectory (typenames)
-add_subdirectory (indexes)
+include_directories( ${CMAKE_CURRENT_SOURCE_DIR} )
+
+add_library( compiz_pch_test STATIC test-pluginclasshandler.cpp )
+
+add_executable( 
+  compiz_pch_construct
+  
+  ${CMAKE_CURRENT_SOURCE_DIR}/construct/src/test-pch-construct.cpp
+)
+
+add_executable( 
+  compiz_pch_get
+
+  ${CMAKE_CURRENT_SOURCE_DIR}/get/src/test-pch-get.cpp
+)
+
+#add_executable( 
+#  compiz_pch_indexes
+#  
+#  ${CMAKE_CURRENT_SOURCE_DIR}/indexes/src/test-pch-indexes.cpp
+#)
+
+add_executable( 
+  compiz_pch_typenames
+  
+  ${CMAKE_CURRENT_SOURCE_DIR}/typenames/src/test-pch-typenames.cpp
+)
+
+target_link_libraries( 
+  compiz_pch_construct 
+  compiz_pch_test
+  
+  compiz_logmessage
+  compiz_pluginclasshandler 
+  compiz_string 
+  
+  ${GTEST_BOTH_LIBRARIES}
+  ${CMAKE_THREAD_LIBS_INIT} # Link in pthread.
+)
+
+target_link_libraries( 
+  compiz_pch_get 
+  compiz_pch_test
+  
+  compiz_logmessage
+  compiz_pluginclasshandler 
+  compiz_string 
+  
+  ${GTEST_BOTH_LIBRARIES}
+  ${CMAKE_THREAD_LIBS_INIT} # Link in pthread.
+)
+
+target_link_libraries( 
+  compiz_pch_typenames
+  compiz_pch_test
+  
+  compiz_logmessage
+  compiz_pluginclasshandler 
+  compiz_string 
+  
+  ${GTEST_BOTH_LIBRARIES}
+  ${CMAKE_THREAD_LIBS_INIT} # Link in pthread.
+)
+
+# Not compilable
+#target_link_libraries( 
+#  compiz_pch_indexes
+#  compiz_pch_test
+#  
+#  compiz_pluginclasshandler 
+#  compiz_string 
+#  
+#  gtest
+#  gtest_main
+#)
+
+add_test( compiz_pch_construct compiz_pch_construct )
+add_test( compiz_pch_get compiz_pch_get )
+#add_test( compiz_pch_indexes compiz_pch_indexes )
+add_test( compiz_pch_typenames compiz_pch_typenames )
+

=== renamed directory 'tests/pluginclasshandler/construct' => 'src/pluginclasshandler/tests/construct'
=== added directory 'src/pluginclasshandler/tests/construct/src'
=== renamed file 'tests/pluginclasshandler/construct/test-pch-construct.cpp' => 'src/pluginclasshandler/tests/construct/src/test-pch-construct.cpp'
--- tests/pluginclasshandler/construct/test-pch-construct.cpp	2011-07-06 18:53:03 +0000
+++ src/pluginclasshandler/tests/construct/src/test-pch-construct.cpp	2012-01-12 06:48:58 +0000
@@ -1,64 +1,49 @@
 #include <test-pluginclasshandler.h>
 
-class ConstructPlugin :
-    public Plugin,
-    public PluginClassHandler <ConstructPlugin, Base>
+class ConstructPlugin: public Plugin, public PluginClassHandler<ConstructPlugin,
+	Base>
 {
-    public:
-	ConstructPlugin (Base *);
+public:
+    ConstructPlugin (Base * base) :
+	    Plugin(base), PluginClassHandler<ConstructPlugin, Base>(base)
+    {
+    }
 };
 
-ConstructPlugin::ConstructPlugin (Base *base):
-    Plugin (base),
-    PluginClassHandler <ConstructPlugin, Base> (base)
-{
-}
-
-CompizPCHTestConstruct::CompizPCHTestConstruct (Global *g) :
-    CompizPCHTest (g)
-{
-}
-
-void
-CompizPCHTestConstruct::run ()
+TEST_F( CompizPCHTest, TestConstruct )
 {
     Plugin *p;
 
-    std::cout << "-= TEST: Basic construction" << std::endl;
-
-    bases.push_back (new Base ());
-    plugins.push_back (static_cast <Plugin *> (new ConstructPlugin (bases.back ())));
-    bases.push_back (new Base ());
-    plugins.push_back (static_cast <Plugin *> (new ConstructPlugin (bases.back ())));
-
-    if (bases.front ()->pluginClasses.size () != globalPluginClassIndices.size ())
-    {
-	std::cout << "FAIL: allocated number of plugin classes is not the same as the"\
-		     " global number of allocated plugin classes" << std::endl;
-	exit (1);
-    }
-
-    if (!ValueHolder::Default ()->hasValue (compPrintf ("%s_index_%lu", typeid (ConstructPlugin).name (), 0)))
-    {
-	std::cout << "FAIL: ValueHolder does not have value " << compPrintf ("%s_index_%lu", typeid (ConstructPlugin).name (), 0) << std::endl;
-	exit (1);
-    }
-
-    p = ConstructPlugin::get (bases.front ());
-
-    if (p != plugins.front ())
-    {
-	std::cout << "FAIL: Returned Plugin * is not plugins.front ()" << std::endl;
-	exit (1);
-    }
-
-    p = ConstructPlugin::get (bases.back ());
-
-    if (p != plugins.back ())
-    {
-	std::cout << "FAIL: Returned Plugin * is not plugins.back ()" << std::endl;
-	exit (1);
-    }
-
-    std::cout << "PASS: Basic construction" << std::endl;
+    bases.push_back(new Base());
+    plugins.push_back(static_cast<Plugin *>(new ConstructPlugin(bases.back())));
+    bases.push_back(new Base());
+    plugins.push_back(static_cast<Plugin *>(new ConstructPlugin(bases.back())));
+
+    if (bases.front()->pluginClasses.size() != globalPluginClassIndices.size())
+    {
+	FAIL() << "allocated number of plugin classes is not the same as the"
+		" global number of allocated plugin classes";
+    }
+
+    if (!ValueHolder::Default()->hasValue(
+	    compPrintf("%s_index_%lu", typeid(ConstructPlugin).name(), 0)))
+    {
+	FAIL() << "ValueHolder does not have value "
+		<< compPrintf("%s_index_%lu", typeid(ConstructPlugin).name(), 0);
+    }
+
+    p = ConstructPlugin::get(bases.front());
+
+    if (p != plugins.front())
+    {
+	FAIL() << "Returned Plugin * is not plugins.front ()";
+    }
+
+    p = ConstructPlugin::get(bases.back());
+
+    if (p != plugins.back())
+    {
+	FAIL() << "Returned Plugin * is not plugins.back ()";
+    }
 }
+

=== renamed directory 'tests/pluginclasshandler/get' => 'src/pluginclasshandler/tests/get'
=== added directory 'src/pluginclasshandler/tests/get/src'
=== renamed file 'tests/pluginclasshandler/get/test-pch-get.cpp' => 'src/pluginclasshandler/tests/get/src/test-pch-get.cpp'
--- tests/pluginclasshandler/get/test-pch-get.cpp	2011-07-06 18:53:03 +0000
+++ src/pluginclasshandler/tests/get/src/test-pch-get.cpp	2012-01-12 06:48:58 +0000
@@ -14,18 +14,10 @@
 {
 }
 
-CompizPCHTestGet::CompizPCHTestGet (Global *g) :
-    CompizPCHTest (g)
-{
-}
-
-void
-CompizPCHTestGet::run ()
+TEST_F( CompizPCHTest, TestGet)
 {
     Plugin *p;
 
-    std::cout << "-= TEST: Object Retreival" << std::endl;
-
     bases.push_back (new Base ());
     plugins.push_back (new GetPlugin (bases.back ()));
     bases.push_back (new Base ());
@@ -35,16 +27,14 @@
 
     if (p != plugins.front ())
     {
-	std::cout << "FAIL: Returned Plugin * is not plugins.front ()" << std::endl;
-	exit (1);
+	FAIL() << "Returned Plugin * is not plugins.front ()";
     }
 
     p = GetPlugin::get (bases.back ());
 
     if (p != plugins.back ())
     {
-	std::cout << "FAIL: Returned Plugin * is not plugins.back ()" << std::endl;
-	exit (1);
+	FAIL() << "Returned Plugin * is not plugins.back ()";
     }
 
     /* Now create a third base and check if plugin is implicitly created */
@@ -56,9 +46,6 @@
 
     if (p->b != bases.back ())
     {
-	std::cout << "FAIL: Returned Plugin * is not the plugin for bases.back ()" << std::endl;
-	exit (1);
+	FAIL() << "Returned Plugin * is not the plugin for bases.back ()";
     }
-
-    std::cout << "PASS: Object Retreival" << std::endl;
 }

=== renamed directory 'tests/pluginclasshandler/indexes' => 'src/pluginclasshandler/tests/indexes'
=== added directory 'src/pluginclasshandler/tests/indexes/src'
=== renamed file 'tests/pluginclasshandler/indexes/test-pch-indexes.cpp' => 'src/pluginclasshandler/tests/indexes/src/test-pch-indexes.cpp'
--- tests/pluginclasshandler/indexes/test-pch-indexes.cpp	2011-07-06 18:53:03 +0000
+++ src/pluginclasshandler/tests/indexes/src/test-pch-indexes.cpp	2012-01-12 06:48:58 +0000
@@ -103,10 +103,26 @@
 {
 }
 
-CompizPCHTestIndexes::CompizPCHTestIndexes (Global *g) :
-    CompizPCHTest (g)
+class CompizPCHTestIndexes :
+    public CompizPCHTest
 {
-}
+    public:
+
+	void run ();
+
+	template <typename I>
+	void printFailure (I *);
+
+    public:
+	unsigned int ePluginClassHandlerIndex;
+	unsigned int eIndex;
+	int 	 eRefCount;
+	bool	 eInitiated;
+	bool	 eFailed;
+	bool	 ePcFailed;
+	unsigned int ePcIndex;
+	bool	 eLoadFailed;
+};
 
 template <typename I>
 void
@@ -115,7 +131,6 @@
     std::cout << "FAIL: index value does not match!" << std::endl;
     std::cout << "INFO: Values of " << std::hex << (void *) i << " are " << std::endl;
     std::cout << "INFO: pluginClassHandlerIndex is " << pluginClassHandlerIndex << std::endl;
-    std::cout << "INFO: mIndex.index " << i->mIndex.index << std::endl;
     std::cout << "INFO: mIndex.refCount " << i->mIndex.refCount << std::endl;
     std::cout << "INFO: mIndex.initiated " << i->mIndex.initiated << std::endl;
     std::cout << "INFO: mIndex.failed " << i->mIndex.failed << std::endl;
@@ -142,8 +157,7 @@
 i->mIndex.pcFailed != ePcFailed || \
 i->mIndex.pcIndex != ePcIndex
 
-void
-CompizPCHTestIndexes::run ()
+TEST_F( CompizPCHTestIndexes, TestIndexes )
 {
     Plugin      *p;
     Base 	*b;
@@ -154,16 +168,12 @@
     IPB3Handler *ipbhandler_p3;
     FIPBHandler *fipbhandler_p;
 
-    std::cout << "-= TEST: Index Sanity" << std::endl;
-
     bases.push_back (new Base ());
     plugins.push_back (new IndexesPlugin (bases.back ()));
 
     p = plugins.back ();
     ipbhandler_p = (IPBHandler *) p;
 
-    std::cout << "STEP: to base #1 plugin #1(1) " << plugins.size () << std::endl;
-
     ePluginClassHandlerIndex = 1;
     eIndex = 0;
     eRefCount = 1;
@@ -176,7 +186,7 @@
     if (INDEX_INCORRECT (ipbhandler_p))
     {
 	printFailure<IPBHandler> (ipbhandler_p);
-	exit (1);
+	FAIL();
     }
 
     bases.push_back (new Base ());
@@ -185,7 +195,6 @@
     p = plugins.back ();
     ipbhandler_p = (IPBHandler *) p;
 
-    std::cout << "STEP: to base #3 plugin #1(2) " << plugins.size () << std::endl;
     ePluginClassHandlerIndex = 1;
     eIndex = 0;
     eRefCount = 2;
@@ -198,7 +207,7 @@
     if (INDEX_INCORRECT (ipbhandler_p))
     {
 	printFailure<IPBHandler> (ipbhandler_p);
-	exit (1);
+	FAIL();
     }
 
     bases.push_back (new Base ());
@@ -207,7 +216,6 @@
     p = plugins.back ();
     ipbhandler_p2 = (IPB2Handler *) p;
 
-    std::cout << "STEP: to base #3 plugin #2(1) " << plugins.size () << std::endl;
     ePluginClassHandlerIndex = 2;
     eIndex = 1;
     eRefCount = 1;
@@ -233,7 +241,6 @@
     p = plugins.back ();
     ipbhandler_p = (IPBHandler *) p;
 
-    std::cout << "STEP: back to base #2 plugin #1(2) " << plugins.size () << std::endl;
     ePluginClassHandlerIndex = 3;
     eIndex = 0;
     eRefCount = 2;
@@ -246,7 +253,7 @@
     if (INDEX_INCORRECT (ipbhandler_p))
     {
 	printFailure<IPBHandler> (ipbhandler_p);
-	exit (1);
+	FAIL();
     }
 
     plugins.pop_back ();
@@ -259,7 +266,6 @@
     p = plugins.back ();
     ipbhandler_p = (IPBHandler *) p;
 
-    std::cout << "STEP: back to base #1 plugin #1(1) " << plugins.size () << std::endl;
     ePluginClassHandlerIndex = 3;
     eIndex = 0;
     eRefCount = 1;
@@ -272,13 +278,12 @@
     if (INDEX_INCORRECT (ipbhandler_p))
     {
 	printFailure<IPBHandler> (ipbhandler_p);
-	exit (1);
+	FAIL();
     }
 
     p = IndexesPlugin::get (bases.back ());
     ipbhandler_p = (IPBHandler *) p;
 
-    std::cout << "STEP: back to base #1 plugin #1(1) ::get " << plugins.size () << std::endl;
     ePluginClassHandlerIndex = 3;
     eIndex = 0;
     eRefCount = 1;
@@ -291,7 +296,7 @@
     if (INDEX_INCORRECT (ipbhandler_p))
     {
 	printFailure<IPBHandler> (ipbhandler_p);
-	exit (1);
+	FAIL();
     }
 
     bases.push_back (new Base ());
@@ -300,7 +305,6 @@
     p = plugins.back ();
     ipbhandler_p3 = (IPB3Handler *) p;
 
-    std::cout << "STEP: to base #2 plugin #3(1) (load failing plugin) " << plugins.size () << std::endl;
     ePluginClassHandlerIndex = 4;
     eIndex = 1;
     eRefCount = 1;
@@ -326,7 +330,6 @@
     p = plugins.back ();
     ipbhandler_p = (IPBHandler *) p;
 
-    std::cout << "STEP: to base #2 plugin #1(2) (failing key) " << plugins.size () << std::endl;
     ePluginClassHandlerIndex = 4;
     eIndex = 0;
     eRefCount = 2;
@@ -339,7 +342,7 @@
     if (INDEX_INCORRECT (ipbhandler_p))
     {
 	printFailure<IPBHandler> (ipbhandler_p);
-	exit (1);
+	FAIL();
     }
 
     plugins.push_back (new IndexesPlugin2 (bases.back ()));
@@ -347,7 +350,6 @@
     p = plugins.back ();
     ipbhandler_p2 = (IPB2Handler *) p;
 
-    std::cout << "STEP: to base #1 plugin #2(1) (failing key) " << plugins.size () << std::endl;
     ePluginClassHandlerIndex = 5;
     eIndex = 2;
     eRefCount = 1;
@@ -360,14 +362,13 @@
     if (INDEX_INCORRECT (ipbhandler_p2))
     {
 	printFailure<IPB2Handler> (ipbhandler_p2);
-	exit (1);
+	FAIL();
     }
 
     /* Now call ::get on the first one to reset the pluginClassHandlerIndex */
     p = IndexesPlugin::get (bases.back ());
     ipbhandler_p = (IPBHandler *) p;
 
-    std::cout << "STEP: to base #1 plugin #1(2) (failing key - after get) " << plugins.size () << std::endl;
     ePluginClassHandlerIndex = 5;
     eIndex = 0;
     eRefCount = 2;
@@ -380,7 +381,7 @@
     if (INDEX_INCORRECT (ipbhandler_p))
     {
 	printFailure<IPBHandler> (ipbhandler_p);
-	exit (1);
+	FAIL();
     }
 
     /* Now erase the key that was used by the second plugin so subsequent attempts to ::get
@@ -391,7 +392,6 @@
     ValueHolder::Default ()->eraseValue (ipbhandler_p2->keyName ());
     p = IndexesPlugin2::get (bases.back ());
 
-    std::cout << "STEP: to base #1 plugin #2(1) (failing key - after erasure) " << plugins.size () << std::endl;
     ePluginClassHandlerIndex = 5;
     eIndex = 2;
     eRefCount = 1;
@@ -404,14 +404,13 @@
     if (INDEX_INCORRECT (ipbhandler_p2))
     {
 	printFailure<IPB2Handler> (ipbhandler_p2);
-	exit (1);
+	FAIL();
     }
 
     fb = new FailingBase ();
     fp = new FailingIndexesPlugin (fb);
     fipbhandler_p = (FIPBHandler *) fp;
 
-    std::cout << "STEP: to base #3 (failing base) plugin #1(2) " << plugins.size () << std::endl;
     ePluginClassHandlerIndex = 5;
     eIndex = 0;
     eRefCount = 0;
@@ -424,11 +423,9 @@
     if (INDEX_INCORRECT (fipbhandler_p))
     {
 	printFailure<FIPBHandler> (fipbhandler_p);
-	exit (1);
+	FAIL();
     }
 
     delete fp;
     delete fb;
-
-    std::cout << "PASS: Index Sanity" << std::endl;
 }

=== renamed file 'tests/pluginclasshandler/test-pluginclasshandler.cpp' => 'src/pluginclasshandler/tests/test-pluginclasshandler.cpp'
--- tests/pluginclasshandler/test-pluginclasshandler.cpp	2011-07-06 18:53:03 +0000
+++ src/pluginclasshandler/tests/test-pluginclasshandler.cpp	2012-01-12 06:48:58 +0000
@@ -60,42 +60,14 @@
 {
 }
 
-CompizPCHTest::CompizPCHTest (Global *g) :
-    global (g)
+CompizPCHTest::CompizPCHTest () :
+    global (new Global())
 {
     ValueHolder::SetDefault (static_cast<ValueHolder *> (global));
+    gTest = this;
 }
 
 CompizPCHTest::~CompizPCHTest ()
 {
     delete global;
 }
-
-int
-main (int argc, char **argv)
-{
-    programName = argv[0];
-
-    gTest = static_cast <CompizPCHTest *> (new OBJECT (new Global ()));
-    gTest->run ();
-
-    while (gTest->plugins.size ())
-    {
-	Plugin *p = gTest->plugins.front ();
-	gTest->plugins.pop_front ();
-
-	delete p;
-    }
-
-    while (gTest->bases.size ())
-    {
-	Base *b = gTest->bases.front ();
-	gTest->bases.pop_front ();
-
-	delete b;
-    }
-
-    delete gTest;
-    
-    return 0;
-}

=== renamed file 'tests/pluginclasshandler/test-pluginclasshandler.h' => 'src/pluginclasshandler/tests/test-pluginclasshandler.h'
--- tests/pluginclasshandler/test-pluginclasshandler.h	2011-07-06 18:53:03 +0000
+++ src/pluginclasshandler/tests/test-pluginclasshandler.h	2012-01-12 06:48:58 +0000
@@ -1,5 +1,8 @@
 #include <core/pluginclasshandler.h>
 #include <core/pluginclasses.h>
+
+#include <gtest/gtest.h>
+
 #include <list>
 #include <boost/foreach.hpp>
 
@@ -46,69 +49,14 @@
 	Base *b;
 };
 
-class CompizPCHTest
+class CompizPCHTest : public ::testing::Test
 {
 public:
 
-     CompizPCHTest (Global *g);
+     CompizPCHTest ();
      virtual ~CompizPCHTest ();
 
      Global *global;
      std::list <Base *> bases;
      std::list <Plugin *> plugins;
-
-     virtual void run () = 0;
-};
-
-class CompizPCHTestConstruct :
-    public CompizPCHTest
-{
-    public:
-
-	CompizPCHTestConstruct (Global *g);
-
-	void run ();
-};
-
-class CompizPCHTestGet :
-    public CompizPCHTest
-{
-    public:
-
-	CompizPCHTestGet (Global *g);
-
-	void run ();
-};
-
-class CompizPCHTestTypenames :
-    public CompizPCHTest
-{
-    public:
-
-	CompizPCHTestTypenames (Global *g);
-
-	void run ();
-};
-
-class CompizPCHTestIndexes :
-    public CompizPCHTest
-{
-    public:
-
-	CompizPCHTestIndexes (Global *g);
-
-	void run ();
-
-	template <typename I>
-	void printFailure (I *);
-
-    public:
-	unsigned int ePluginClassHandlerIndex;
-	unsigned int eIndex;
-	int 	 eRefCount;
-	bool	 eInitiated;
-	bool	 eFailed;
-	bool	 ePcFailed;
-	unsigned int ePcIndex;
-	bool	 eLoadFailed;
 };

=== renamed directory 'tests/pluginclasshandler/typenames' => 'src/pluginclasshandler/tests/typenames'
=== added directory 'src/pluginclasshandler/tests/typenames/src'
=== renamed file 'tests/pluginclasshandler/typenames/test-pch-typenames.cpp' => 'src/pluginclasshandler/tests/typenames/src/test-pch-typenames.cpp'
--- tests/pluginclasshandler/typenames/test-pch-typenames.cpp	2011-07-06 18:53:03 +0000
+++ src/pluginclasshandler/tests/typenames/src/test-pch-typenames.cpp	2012-01-12 06:48:58 +0000
@@ -1,83 +1,75 @@
 #include <test-pluginclasshandler.h>
 
-class TypenamesPlugin :
-    public Plugin,
-    public PluginClassHandler <TypenamesPlugin, Base>
-{
-    public:
-	TypenamesPlugin (Base *);
-};
-
-class TypenamesPluginABI1 :
-    public Plugin,
-    public PluginClassHandler <TypenamesPluginABI1, Base, 1>
-{
-    public:
-	TypenamesPluginABI1 (Base *);
-};
-
-class TypenamesPluginABI2 :
-    public Plugin,
-    public PluginClassHandler <TypenamesPluginABI2, Base, 2>
-{
-    public:
-	TypenamesPluginABI2 (Base *);
-};
-
-TypenamesPlugin::TypenamesPlugin (Base *base):
-    Plugin (base),
-    PluginClassHandler <TypenamesPlugin, Base> (base)
-{
-}
-
-TypenamesPluginABI1::TypenamesPluginABI1 (Base *base):
-    Plugin (base),
-    PluginClassHandler <TypenamesPluginABI1, Base, 1> (base)
-{
-}
-
-TypenamesPluginABI2::TypenamesPluginABI2 (Base *base):
-    Plugin (base),
-    PluginClassHandler <TypenamesPluginABI2, Base, 2> (base)
-{
-}
-
-CompizPCHTestTypenames::CompizPCHTestTypenames (Global *g) :
-    CompizPCHTest (g)
-{
-}
-
-void
-CompizPCHTestTypenames::run ()
-{
-    std::list <Plugin *>::iterator it;
-
-    std::cout << "-= TEST: Typenames" << std::endl;
-
-    bases.push_back (new Base ());
-    plugins.push_back (new TypenamesPlugin (bases.back ()));
-    bases.push_back (new Base ());
-    plugins.push_back (new TypenamesPluginABI1 (bases.back ()));
-    bases.push_back (new Base ());
-    plugins.push_back (new TypenamesPluginABI2 (bases.back ()));
-
-    if (!ValueHolder::Default ()->hasValue (compPrintf ("%s_index_%lu", typeid (TypenamesPlugin).name (), 0)))
-    {
-	std::cout << "FAIL: ValueHolder does not have value " << compPrintf ("%s_index_%lu", typeid (TypenamesPlugin).name (), 0) << std::endl;
-	exit (1);
-    }
-
-    if (!ValueHolder::Default ()->hasValue (compPrintf ("%s_index_%lu", typeid (TypenamesPluginABI1).name (), 1)))
-    {
-	std::cout << "FAIL: ValueHolder does not have value " << compPrintf ("%s_index_%lu", typeid (TypenamesPluginABI1).name (), 0) << std::endl;
-	exit (1);
-    }
-
-    if (!ValueHolder::Default ()->hasValue (compPrintf ("%s_index_%lu", typeid (TypenamesPluginABI2).name (), 2)))
-    {
-	std::cout << "FAIL: ValueHolder does not have value " << compPrintf ("%s_index_%lu", typeid (TypenamesPluginABI2).name (), 2) << std::endl;
-	exit (1);
-    }
-
-    std::cout << "PASS: Typenames" << std::endl;
+class TypenamesPlugin: public Plugin, public PluginClassHandler<TypenamesPlugin,
+	Base>
+{
+public:
+    TypenamesPlugin (Base *);
+};
+
+class TypenamesPluginABI1: public Plugin, public PluginClassHandler<
+	TypenamesPluginABI1, Base, 1>
+{
+public:
+    TypenamesPluginABI1 (Base *);
+};
+
+class TypenamesPluginABI2: public Plugin, public PluginClassHandler<
+	TypenamesPluginABI2, Base, 2>
+{
+public:
+    TypenamesPluginABI2 (Base *);
+};
+
+TypenamesPlugin::TypenamesPlugin (Base *base) :
+	Plugin(base), PluginClassHandler<TypenamesPlugin, Base>(base)
+{
+}
+
+TypenamesPluginABI1::TypenamesPluginABI1 (Base *base) :
+	Plugin(base), PluginClassHandler<TypenamesPluginABI1, Base, 1>(base)
+{
+}
+
+TypenamesPluginABI2::TypenamesPluginABI2 (Base *base) :
+	Plugin(base), PluginClassHandler<TypenamesPluginABI2, Base, 2>(base)
+{
+}
+
+TEST_F( CompizPCHTest, TestTypenames)
+{
+    std::list<Plugin *>::iterator it;
+
+    bases.push_back(new Base());
+    plugins.push_back(new TypenamesPlugin(bases.back()));
+    bases.push_back(new Base());
+    plugins.push_back(new TypenamesPluginABI1(bases.back()));
+    bases.push_back(new Base());
+    plugins.push_back(new TypenamesPluginABI2(bases.back()));
+
+    if (!ValueHolder::Default()->hasValue(
+	    compPrintf("%s_index_%lu", typeid(TypenamesPlugin).name(), 0)))
+    {
+	FAIL() << "ValueHolder does not have value "
+		<< compPrintf("%s_index_%lu", typeid(TypenamesPlugin).name(), 0);
+    }
+
+    if (!ValueHolder::Default()->hasValue(
+	    compPrintf("%s_index_%lu", typeid(TypenamesPluginABI1).name(), 1)))
+    {
+	FAIL()
+		<< "ValueHolder does not have value "
+		<< compPrintf("%s_index_%lu",
+			typeid(TypenamesPluginABI1).name(), 0);
+    }
+
+    if (!ValueHolder::Default()->hasValue(
+	    compPrintf("%s_index_%lu", typeid(TypenamesPluginABI2).name(), 2)))
+    {
+	FAIL()
+		<< "FAIL: ValueHolder does not have value "
+		<< compPrintf("%s_index_%lu",
+			typeid(TypenamesPluginABI2).name(), 2);
+    }
+
 }

=== modified file 'src/privatescreen.h'
--- src/privatescreen.h	2011-10-10 14:56:06 +0000
+++ src/privatescreen.h	2012-01-12 06:48:58 +0000
@@ -103,6 +103,20 @@
 
 namespace compiz
 {
+namespace core
+{
+namespace screen
+{
+    inline int wraparound_mod (int a, int b)
+    {
+	if (a < 0)
+	    return (b - ((-a - 1) % (b))) - 1;
+	else
+	    return a % b;
+    };
+}
+}
+
 namespace X11
 {
 class PendingEvent {

=== modified file 'src/region.cpp'
--- src/region.cpp	2010-12-04 21:28:09 +0000
+++ src/region.cpp	2012-01-12 06:48:58 +0000
@@ -104,7 +104,7 @@
     delete priv;
 }
 
-const Region
+Region
 CompRegion::handle () const
 {
     return priv->region;

=== modified file 'src/screen.cpp'
--- src/screen.cpp	2011-10-13 14:28:30 +0000
+++ src/screen.cpp	2012-01-12 06:48:58 +0000
@@ -3502,11 +3502,11 @@
     CompPoint pnt;
 
     tx = priv->vp.x () - tx;
-    tx = MOD (tx, priv->vpSize.width ());
+    tx = compiz::core::screen::wraparound_mod (tx, priv->vpSize.width ());
     tx -= priv->vp.x ();
 
     ty = priv->vp.y () - ty;
-    ty = MOD (ty, priv->vpSize.height ());
+    ty = compiz::core::screen::wraparound_mod (ty, priv->vpSize.height ());
     ty -= priv->vp.y ();
 
     if (!tx && !ty)

=== modified file 'src/session.cpp'
--- src/session.cpp	2011-03-16 20:27:14 +0000
+++ src/session.cpp	2012-01-12 06:48:58 +0000
@@ -27,8 +27,6 @@
 #  include <config.h>
 #endif
 
-#include <compiz.h>
-
 #include <stdlib.h>
 #include <stdio.h>
 #include <poll.h>

=== added directory 'src/string'
=== added file 'src/string/CMakeLists.txt'
--- src/string/CMakeLists.txt	1970-01-01 00:00:00 +0000
+++ src/string/CMakeLists.txt	2011-12-19 07:06:22 +0000
@@ -0,0 +1,35 @@
+INCLUDE_DIRECTORIES(
+  ${compiz_SOURCE_DIR}/include
+  ${CMAKE_CURRENT_SOURCE_DIR}/include
+  
+  ${Boost_INCLUDE_DIRS}
+)
+
+SET( PUBLIC_HEADERS )
+SET( PRIVATE_HEADERS )
+
+SET( SRCS ${CMAKE_CURRENT_SOURCE_DIR}/src/string.cpp )
+
+ADD_LIBRARY( 
+  compiz_string STATIC
+  
+  ${SRCS}
+  
+  ${PUBLIC_HEADERS}
+  ${PRIVATE_HEADERS}
+)
+
+ADD_SUBDIRECTORY( ${CMAKE_CURRENT_SOURCE_DIR}/tests )
+
+SET_TARGET_PROPERTIES(
+  compiz_string PROPERTIES
+  PUBLIC_HEADER "${PUBLIC_HEADERS}"
+)
+
+INSTALL(
+  TARGETS                                 compiz_string
+  RUNTIME             DESTINATION         bin
+  LIBRARY             DESTINATION         lib
+  ARCHIVE             DESTINATION         lib
+  PUBLIC_HEADER       DESTINATION         include/compiz
+)

=== added directory 'src/string/include'
=== added directory 'src/string/src'
=== renamed file 'src/string.cpp' => 'src/string/src/string.cpp'
--- src/string.cpp	2009-08-08 05:52:48 +0000
+++ src/string/src/string.cpp	2012-01-12 06:48:58 +0000
@@ -23,12 +23,12 @@
  * Authors: Dennis Kasprzyk <onestone@compiz-fusion.org>
  */
 
-#include <compiz.h>
-
 #include <cstring>
 #include <cstdarg>
 #include <cstdio>
 
+#include <core/string.h>
+
 CompString compPrintf (const char *format, ...)
 {
     va_list    ap;

=== added directory 'src/string/tests'
=== added file 'src/string/tests/CMakeLists.txt'
--- src/string/tests/CMakeLists.txt	1970-01-01 00:00:00 +0000
+++ src/string/tests/CMakeLists.txt	2011-12-19 07:06:22 +0000
@@ -0,0 +1,20 @@
+include_directories( 
+  ${CMAKE_CURRENT_SOURCE_DIR} 
+)
+
+add_executable( 
+  compiz_string_test
+  
+  ${CMAKE_CURRENT_SOURCE_DIR}/printf/src/test-string-printf.cpp
+)
+
+target_link_libraries( 
+  compiz_string_test 
+  
+  compiz_string
+  
+  ${GTEST_BOTH_LIBRARIES}
+  ${CMAKE_THREAD_LIBS_INIT} # Link in pthread.
+)
+
+add_test( compiz_string_test compiz_string_test )

=== added directory 'src/string/tests/printf'
=== added directory 'src/string/tests/printf/src'
=== added file 'src/string/tests/printf/src/test-string-printf.cpp'
--- src/string/tests/printf/src/test-string-printf.cpp	1970-01-01 00:00:00 +0000
+++ src/string/tests/printf/src/test-string-printf.cpp	2011-12-19 07:06:22 +0000
@@ -0,0 +1,161 @@
+/*
+ * Copyright © 2011 Canonical Ltd.
+ *
+ * Permission to use, copy, modify, distribute, and sell this software
+ * and its documentation for any purpose is hereby granted without
+ * fee, provided that the above copyright notice appear in all copies
+ * and that both that copyright notice and this permission notice
+ * appear in supporting documentation, and that the name of
+ * Canonical Ltd. not be used in advertising or publicity pertaining to
+ * distribution of the software without specific, written prior permission.
+ * Canonical Ltd. makes no representations about the suitability of this
+ * software for any purpose. It is provided "as is" without express or
+ * implied warranty.
+ *
+ * CANONICAL, LTD. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
+ * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
+ * NO EVENT SHALL CANONICAL, LTD. BE LIABLE FOR ANY SPECIAL, INDIRECT OR
+ * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
+ * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
+ * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
+ * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+ * Authored by: Sam Spilsbury <sam.spilsbury@canonical.com>
+ */
+
+#include <core/string.h>
+
+#include <gtest/gtest.h>
+
+#include <string>
+#include <iostream>
+#include <map>
+#include <boost/shared_ptr.hpp>
+#include <boost/pointer_cast.hpp>
+#include <typeinfo>
+
+namespace compiz
+{
+namespace string
+{
+namespace printf_test
+{
+
+class Value
+{
+public:
+    virtual ~Value ();
+
+    typedef boost::shared_ptr<Value> Ptr;
+
+protected:
+
+    Value ()
+    {
+    }
+    ;
+
+    void *v;
+};
+
+template<typename T>
+class TValue: public Value
+{
+public:
+    TValue (const T &);
+    ~TValue ();
+
+    const T & value ();
+};
+
+template<typename T>
+TValue<T>::TValue (const T &av)
+{
+    v = new T(av);
+}
+
+template<typename T>
+TValue<T>::~TValue ()
+{
+    delete (reinterpret_cast<T *>(v));
+}
+
+template<typename T>
+const T &
+TValue<T>::value ()
+{
+    return *(reinterpret_cast<T *>(v));
+}
+
+Value::~Value ()
+{
+}
+
+CompString get_format (const CompString &fmt, Value::Ptr v)
+{
+    if (fmt == "%i" || fmt == "%d")
+	return compPrintf(fmt.c_str(),
+		(boost::shared_static_cast<TValue<int> >(v))->value());
+    if (fmt == "%f")
+	return compPrintf(fmt.c_str(),
+		(boost::shared_static_cast<TValue<float> >(v))->value());
+    if (fmt == "%s")
+	return compPrintf(
+		fmt.c_str(),
+		(boost::shared_static_cast<TValue<std::string> >(v))->value().c_str());
+    if (fmt == "%x")
+	return compPrintf(fmt.c_str(),
+		(boost::shared_static_cast<TValue<int> >(v))->value());
+
+    return "not_reached";
+}
+
+}
+}
+}
+
+TEST(CompizString,PrintfTest)
+{
+    CompString s1;
+    CompString s2;
+    std::map<CompString, compiz::string::printf_test::Value::Ptr> formatValues;
+    std::map<CompString, CompString> formatStrings;
+
+    s1 = "foo";
+
+    const char *other_foo = "foo";
+    s2 = compPrintf ("%s", other_foo);
+
+    ASSERT_EQ(s1, s2);
+
+    s1 = "3";
+    s2 = compPrintf ("%i", 3, NULL);
+
+    ASSERT_EQ(s1, s2);
+
+    s1 = "3.012600";
+    s2 = compPrintf ("%f", 3.0126, NULL);
+
+    ASSERT_EQ(s1, s2);
+
+    s1 = "0x4f567";
+    s2 = compPrintf ("0x%x", 0x4f567, NULL);
+
+    ASSERT_EQ(s1, s2);
+
+    formatValues["%i"] = boost::shared_static_cast <compiz::string::printf_test::Value> (compiz::string::printf_test::Value::Ptr (new compiz::string::printf_test::TValue<int> (6)));
+    formatStrings["%i"] = CompString ("6");
+    formatValues["%f"] = boost::shared_static_cast <compiz::string::printf_test::Value> (compiz::string::printf_test::Value::Ptr (new compiz::string::printf_test::TValue<float> (6.532)));
+    formatStrings["%f"] = CompString ("6.532000");
+    formatValues["%x"] = boost::shared_static_cast <compiz::string::printf_test::Value> (compiz::string::printf_test::Value::Ptr (new compiz::string::printf_test::TValue<int> (0x34fe5aa)));
+    formatStrings["%x"] = CompString ("34fe5aa");
+    formatValues["%d"] = boost::shared_static_cast <compiz::string::printf_test::Value> (compiz::string::printf_test::Value::Ptr (new compiz::string::printf_test::TValue<int> (2)));
+    formatStrings["%d"] = CompString ("2");
+
+    for (std::map <CompString, CompString>::iterator it = formatStrings.begin ();
+	    it != formatStrings.end (); it++)
+    {
+	CompString str = compiz::string::printf_test::get_format (it->first, formatValues[it->first]);
+	ASSERT_EQ(str, it->second);
+    }
+}

=== added file 'src/string/tests/test-string.cpp'
--- src/string/tests/test-string.cpp	1970-01-01 00:00:00 +0000
+++ src/string/tests/test-string.cpp	2011-11-01 23:16:57 +0000
@@ -0,0 +1,47 @@
+/*
+ * Copyright © 2011 Canonical Ltd.
+ *
+ * Permission to use, copy, modify, distribute, and sell this software
+ * and its documentation for any purpose is hereby granted without
+ * fee, provided that the above copyright notice appear in all copies
+ * and that both that copyright notice and this permission notice
+ * appear in supporting documentation, and that the name of
+ * Canonical Ltd. not be used in advertising or publicity pertaining to
+ * distribution of the software without specific, written prior permission.
+ * Canonical Ltd. makes no representations about the suitability of this
+ * software for any purpose. It is provided "as is" without express or
+ * implied warranty.
+ *
+ * CANONICAL, LTD. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
+ * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
+ * NO EVENT SHALL CANONICAL, LTD. BE LIABLE FOR ANY SPECIAL, INDIRECT OR
+ * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
+ * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
+ * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
+ * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+ * Authored by: Sam Spilsbury <sam.spilsbury@canonical.com>
+ */
+
+#include "test-string.h"
+
+CompStringTest::CompStringTest ()
+{
+}
+
+CompStringTest::~CompStringTest ()
+{
+}
+
+int
+main (int argc, char **argv)
+{
+    CompStringTest  *cst = static_cast <CompStringTest *> (getTestObject ());
+    bool ret = false;
+
+    ret = cst->test ();
+
+    delete cst;
+
+    return ret ? 0 : 1;
+}

=== added file 'src/string/tests/test-string.h'
--- src/string/tests/test-string.h	1970-01-01 00:00:00 +0000
+++ src/string/tests/test-string.h	2011-11-01 23:16:57 +0000
@@ -0,0 +1,52 @@
+/*
+ * Copyright © 2011 Canonical Ltd.
+ *
+ * Permission to use, copy, modify, distribute, and sell this software
+ * and its documentation for any purpose is hereby granted without
+ * fee, provided that the above copyright notice appear in all copies
+ * and that both that copyright notice and this permission notice
+ * appear in supporting documentation, and that the name of
+ * Canonical Ltd. not be used in advertising or publicity pertaining to
+ * distribution of the software without specific, written prior permission.
+ * Canonical Ltd. makes no representations about the suitability of this
+ * software for any purpose. It is provided "as is" without express or
+ * implied warranty.
+ *
+ * CANONICAL, LTD. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
+ * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
+ * NO EVENT SHALL CANONICAL, LTD. BE LIABLE FOR ANY SPECIAL, INDIRECT OR
+ * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
+ * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
+ * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
+ * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+ * Authored by: Sam Spilsbury <sam.spilsbury@canonical.com>
+ */
+
+#ifndef _COMPIZ_TEST_STRING_H
+#define _COMPIZ_TEST_STRING_H
+
+#include <core/string.h>
+
+class CompStringTest
+{
+public:
+
+    CompStringTest ();
+    virtual ~CompStringTest ();
+
+    virtual bool test () = 0;
+};
+
+class CompStringTestPrintf :
+    public CompStringTest
+{
+public:
+
+    bool test ();
+};
+
+CompStringTest *
+getTestObject ();
+
+#endif

=== added directory 'src/timer'
=== added file 'src/timer/CMakeLists.txt'
--- src/timer/CMakeLists.txt	1970-01-01 00:00:00 +0000
+++ src/timer/CMakeLists.txt	2011-12-23 03:20:52 +0000
@@ -0,0 +1,70 @@
+pkg_check_modules(
+  GLIBMM 
+  REQUIRED
+  glibmm-2.4 
+)
+
+INCLUDE_DIRECTORIES(  
+  ${CMAKE_CURRENT_SOURCE_DIR}/include
+  ${CMAKE_CURRENT_SOURCE_DIR}/src
+  
+  ${compiz_SOURCE_DIR}/include
+    
+  ${Boost_INCLUDE_DIRS}
+  
+  ${GLIBMM_INCLUDE_DIRS}
+)
+
+LINK_DIRECTORIES (${GLIBMM_LIBRARY_DIRS}) 
+
+SET( 
+  PUBLIC_HEADERS 
+  ${CMAKE_CURRENT_SOURCE_DIR}/include/core/timer.h
+  ${CMAKE_CURRENT_SOURCE_DIR}/include/core/timeouthandler.h
+)
+
+SET( 
+  PRIVATE_HEADERS 
+  ${CMAKE_CURRENT_SOURCE_DIR}/src/privatetimeouthandler.h
+  ${CMAKE_CURRENT_SOURCE_DIR}/src/privatetimeoutsource.h
+  ${CMAKE_CURRENT_SOURCE_DIR}/src/privatetimer.h
+
+)
+
+SET( 
+  SRCS 
+  ${CMAKE_CURRENT_SOURCE_DIR}/src/timeouthandler.cpp 
+  ${CMAKE_CURRENT_SOURCE_DIR}/src/timer.cpp 
+)
+
+ADD_LIBRARY( 
+  compiz_timer STATIC
+  
+  ${SRCS}
+  
+  ${PUBLIC_HEADERS}
+  ${PRIVATE_HEADERS}
+)
+
+ADD_SUBDIRECTORY( ${CMAKE_CURRENT_SOURCE_DIR}/tests )
+
+SET_TARGET_PROPERTIES(
+  compiz_timer PROPERTIES
+  PUBLIC_HEADER "${PUBLIC_HEADERS}"
+)
+
+INSTALL(
+  TARGETS                                 compiz_timer
+  RUNTIME             DESTINATION         bin
+  LIBRARY             DESTINATION         lib
+  ARCHIVE             DESTINATION         lib
+  PUBLIC_HEADER       DESTINATION         include/compiz
+)
+
+
+
+TARGET_LINK_LIBRARIES( 
+  compiz_timer
+  
+  ${GLIBMM_LIBRARIES}
+)

=== added directory 'src/timer/include'
=== added directory 'src/timer/include/core'
=== renamed file 'include/core/timeouthandler.h' => 'src/timer/include/core/timeouthandler.h'
=== renamed file 'include/core/timer.h' => 'src/timer/include/core/timer.h'
--- include/core/timer.h	2011-07-04 18:17:44 +0000
+++ src/timer/include/core/timer.h	2012-01-12 06:48:58 +0000
@@ -32,6 +32,24 @@
 
 class CompTimeoutSource;
 class PrivateTimer;
+
+namespace compiz {
+namespace core {
+namespace timer {
+
+    inline int timeval_diff (struct timeval *tv1, struct timeval *tv2)
+    {
+	if (tv1->tv_sec == tv2->tv_sec || tv1->tv_usec >= tv2->tv_usec)
+	    return (((tv1->tv_sec - tv2->tv_sec) * 1000000) +			   
+	      (tv1->tv_usec - tv2->tv_usec)) / 1000;
+	else
+	    return (((tv1->tv_sec - 1 - tv2->tv_sec) * 1000000) +
+		   (1000000 + tv1->tv_usec - tv2->tv_usec)) / 1000;
+    }
+}
+}
+}
+
 /**
  * A simple timer for use with invoking a CallBack during a timed duration.
  */

=== added directory 'src/timer/src'
=== renamed file 'src/privatetimeouthandler.h' => 'src/timer/src/privatetimeouthandler.h'
=== renamed file 'src/privatetimeoutsource.h' => 'src/timer/src/privatetimeoutsource.h'
--- src/privatetimeoutsource.h	2011-07-04 16:25:20 +0000
+++ src/timer/src/privatetimeoutsource.h	2012-01-12 06:48:58 +0000
@@ -47,10 +47,6 @@
 	explicit CompTimeoutSource (Glib::RefPtr <Glib::MainContext> &ctx);
 	virtual ~CompTimeoutSource ();
 
-    private:
-
-	gint64 mLastTime;
-
     friend class CompTimer;
 };
 

=== renamed file 'src/privatetimer.h' => 'src/timer/src/privatetimer.h'
--- src/privatetimer.h	2011-07-05 19:15:16 +0000
+++ src/timer/src/privatetimer.h	2012-01-12 06:48:58 +0000
@@ -38,8 +38,8 @@
 	bool         		 mActive;
 	unsigned int 		 mMinTime;
 	unsigned int 		 mMaxTime;
-	int          		 mMinLeft;
-	int          		 mMaxLeft;
+	gint64                   mMinDeadline;
+	gint64                   mMaxDeadline;
 	CompTimer::CallBack      mCallBack;
 };
 

=== renamed file 'src/timeouthandler.cpp' => 'src/timer/src/timeouthandler.cpp'
=== renamed file 'src/timer.cpp' => 'src/timer/src/timer.cpp'
--- src/timer.cpp	2011-07-05 19:15:24 +0000
+++ src/timer/src/timer.cpp	2012-01-12 06:48:58 +0000
@@ -35,11 +35,27 @@
 CompTimeoutSource::CompTimeoutSource (Glib::RefPtr <Glib::MainContext> &ctx) :
     Glib::Source ()
 {
-    set_priority (G_PRIORITY_HIGH);
+    /*
+     * Compiz MUST use priority G_PRIORITY_DEFAULT because that's the same
+     * as GDK_PRIORITY_EVENTS.
+     *
+     * Any higher priority than that and you will starve the glib event loop
+     * of X events. That causes stuttering and even complete starvation
+     * can cause compiz to hang inside X11/GLX functions indefinitely.
+     * In the best case, GLX functions would be slowed down by their messages
+     * being prioritized lower than us.
+     *
+     * Any lower priority than that and screen redraws can lag behind
+     * important things like dragging windows around.
+     *
+     * We have an "unfair" scheduling algorithm in the glib main event loop
+     * to thank for all this. Ideally, compiz should be threaded so it never
+     * slows down the handling of X11/GLX messages in the glib main loop, or
+     * vice-versa.
+     */
+    set_priority (G_PRIORITY_DEFAULT);
     attach (ctx);
 
-    mLastTime = get_time ();
-    
     connect (sigc::mem_fun <bool, CompTimeoutSource> (this, &CompTimeoutSource::callback));
 }
 
@@ -95,51 +111,20 @@
 		timeout = t->maxLeft ();
 	    it++;
 	}
-
-	mLastTime = get_time ();
-
-	return false;
     }
     else
     {
 	timeout = 0;
-
-	mLastTime = get_time ();
-
-	return true;
     }
+
+    return timeout <= 0;
 }
 
 bool
 CompTimeoutSource::check ()
 {
-    gint64		    fixedTimeDiff;
-    gdouble		    timeDiff;
-    gint64		    currentTime = get_time ();
-    bool		    ready = false;
-
-    timeDiff = (currentTime - mLastTime) / 1000.0;
-
-    mLastTime = currentTime;
-
-    /* prefer over-estimating rather than waking up */
-    fixedTimeDiff = ceil (timeDiff);
-
-    /* Handle clock rollback */
-    if (fixedTimeDiff < 0)
-    {
-	fixedTimeDiff = 0;
-    }
-
-    foreach (CompTimer *t, TimeoutHandler::Default ()->timers ())
-    {
-	t->decrement (fixedTimeDiff);
-    }
-
-    ready = (!TimeoutHandler::Default ()->timers ().empty () &&
+    return (!TimeoutHandler::Default ()->timers ().empty () &&
 	     TimeoutHandler::Default ()->timers ().front ()->minLeft () <= 0);
-
-    return ready;
 }
 
 bool
@@ -152,29 +137,37 @@
 bool
 CompTimeoutSource::callback ()
 {
-    while (TimeoutHandler::Default ()->timers ().begin () != TimeoutHandler::Default ()->timers ().end () &&
-	   TimeoutHandler::Default ()->timers ().front ()->minLeft () <= 0)
+    TimeoutHandler *handler = TimeoutHandler::Default ();
+    std::list<CompTimer*> requeue, &timers = handler->timers ();
+
+    while (!timers.empty ())
     {
-	CompTimer *t = TimeoutHandler::Default ()->timers ().front ();
-	TimeoutHandler::Default ()->timers ().pop_front ();
-
+	CompTimer *t = timers.front ();
+	if (t->minLeft () > 0)
+	    break;
+	timers.pop_front ();
 	t->setActive (false);
 	if (t->triggerCallback ())
-	{
-	    TimeoutHandler::Default ()->addTimer (t);
-	    t->setActive (true);
-	}
-    }
-
-    return !TimeoutHandler::Default ()->timers ().empty ();
+	    requeue.push_back (t);
+    }
+
+    std::list<CompTimer*>::const_iterator i = requeue.begin ();
+    for (; i != requeue.end (); i++)
+    {
+	CompTimer *t = *i;
+	handler->addTimer (t);
+	t->setActive (true);
+    }
+
+    return !timers.empty ();
 }
 
 PrivateTimer::PrivateTimer () :
     mActive (false),
     mMinTime (0),
     mMaxTime (0),
-    mMinLeft (0),
-    mMaxLeft (0),
+    mMinDeadline (0),
+    mMaxDeadline (0),
     mCallBack (NULL)
 {
 }
@@ -211,15 +204,15 @@
 void
 CompTimer::setExpiryTimes (unsigned int min, unsigned int max)
 {
-    priv->mMinLeft = min;
-    priv->mMaxLeft = (min <= max) ? max : min;
+    gint64 now = g_get_monotonic_time ();
+    priv->mMinDeadline = now + (static_cast <gint64> (min) * 1000);
+    priv->mMaxDeadline = now + (static_cast <gint64> (max >= min ? max : min) * 1000);
 }
 
 void
 CompTimer::decrement (unsigned int diff)
 {
-    priv->mMinLeft -= diff;
-    priv->mMaxLeft -= diff;
+    /* deprecated */
 }
 
 void
@@ -305,13 +298,17 @@
 unsigned int
 CompTimer::minLeft ()
 {
-    return (priv->mMinLeft < 0)? 0 : priv->mMinLeft;
+    gint64 now = g_get_monotonic_time ();
+    return (priv->mMinDeadline > now) ?
+	(unsigned int)(priv->mMinDeadline - now + 500) / 1000 : 0;
 }
 
 unsigned int
 CompTimer::maxLeft ()
 {
-    return (priv->mMaxLeft < 0)? 0 : priv->mMaxLeft;
+    gint64 now = g_get_monotonic_time ();
+    return (priv->mMaxDeadline > now) ?
+	(unsigned int)(priv->mMaxDeadline - now + 500) / 1000 : 0;
 }
 
 bool

=== added directory 'src/timer/tests'
=== renamed file 'tests/timer/CMakeLists.txt' => 'src/timer/tests/CMakeLists.txt'
--- tests/timer/CMakeLists.txt	2011-07-06 18:58:47 +0000
+++ src/timer/tests/CMakeLists.txt	2012-01-12 06:48:58 +0000
@@ -1,15 +1,57 @@
-include (CompizCommon)
-include (FindPkgConfig)
-
-pkg_check_modules (COMPIZ_TEST_TIMER glibmm-2.4)
-
-if (COMPIZ_TEST_TIMER_FOUND)
-
-	set (COMPIZ_TIMER_TEST_BASE_DIR ${CMAKE_CURRENT_SOURCE_DIR})
-
-	add_subdirectory (callbacks)
-	add_subdirectory (diffs)
-	add_subdirectory (set-values)
-	add_subdirectory (while-calling)
-
-endif (COMPIZ_TEST_TIMER_FOUND)
+include_directories(${CMAKE_CURRENT_SOURCE_DIR})
+
+add_library (compiz_timer_test
+             ${CMAKE_CURRENT_SOURCE_DIR}/test-timer.cpp)
+
+add_executable (compiz_timer_callbacks
+                ${CMAKE_CURRENT_SOURCE_DIR}/callbacks/src/test-timer-callbacks.cpp)
+
+add_executable (compiz_timer_diffs
+                ${CMAKE_CURRENT_SOURCE_DIR}/diffs/src/test-timer-diffs.cpp)
+
+add_executable (compiz_timer_set-values
+                ${CMAKE_CURRENT_SOURCE_DIR}/set-values/src/test-timer-set-values.cpp)
+
+add_executable (compiz_timer_while-calling
+                ${CMAKE_CURRENT_SOURCE_DIR}/while-calling/src/test-timer-set-times-while-calling.cpp)
+
+target_link_libraries (compiz_timer_callbacks 
+                       compiz_timer_test
+                       compiz_timer 
+                       ${GTEST_BOTH_LIBRARIES}
+		       ${GMOCK_LIBRARY}
+		       ${GMOCK_MAIN_LIBRARY}
+		       ${CMAKE_THREAD_LIBS_INIT} # Link in pthread. 
+                       )
+
+target_link_libraries (compiz_timer_diffs
+                       compiz_timer_test
+                       compiz_timer 
+                       ${GTEST_BOTH_LIBRARIES}
+		       ${GMOCK_LIBRARY}
+		       ${GMOCK_MAIN_LIBRARY}
+		       ${CMAKE_THREAD_LIBS_INIT} # Link in pthread. 
+                       )
+
+target_link_libraries (compiz_timer_set-values
+                       compiz_timer_test
+                       compiz_timer 
+                       ${GTEST_BOTH_LIBRARIES}
+		       ${GMOCK_LIBRARY}
+		       ${GMOCK_MAIN_LIBRARY}
+		       ${CMAKE_THREAD_LIBS_INIT} # Link in pthread. 
+                       )
+
+target_link_libraries (compiz_timer_while-calling
+                       compiz_timer_test
+                       compiz_timer 
+                       ${GTEST_BOTH_LIBRARIES}
+		       ${GMOCK_LIBRARY}
+		       ${GMOCK_MAIN_LIBRARY}
+		       ${CMAKE_THREAD_LIBS_INIT} # Link in pthread. 
+                       )
+
+add_test (compiz_timer_callbacks compiz_timer_callbacks)
+add_test (compiz_timer_diffs compiz_timer_diffs)
+add_test (compiz_timer_set-values compiz_timer_set-values)
+add_test (compiz_timer_while-calling compiz_timer_while-calling)

=== renamed directory 'tests/timer/callbacks' => 'src/timer/tests/callbacks'
=== added directory 'src/timer/tests/callbacks/src'
=== renamed file 'tests/timer/callbacks/test-timer-callbacks.cpp' => 'src/timer/tests/callbacks/src/test-timer-callbacks.cpp'
--- tests/timer/callbacks/test-timer-callbacks.cpp	2011-07-06 18:58:47 +0000
+++ src/timer/tests/callbacks/src/test-timer-callbacks.cpp	2012-01-12 06:48:58 +0000
@@ -24,83 +24,244 @@
  */
 
 #include "test-timer.h"
-
-bool
-CompTimerTestCallbacks::cb (int timernum)
-{
-    if (lastTimerTriggered == 0 && timernum == 1)
-    {
-	std::cout << "FAIL: timer with a higher timeout value triggered before the timer with the lower timeout" << std::endl;
-	exit (1);
-	return false;
-    }
-    else if (lastTimerTriggered == 2 && timernum != 1)
-    {
-	std::cout << "FAIL: the second timer should have triggered" << std::endl;
-	exit (1);
-	return false;
-    }
-
-    std::cout << "INFO: triggering timer " << timernum << std::endl;
-
-    lastTimerTriggered = timernum;
-
-    if (timernum == 1)
-    {
-	std::cout << "PASS: basic timers" << std::endl;
-	ml->quit ();
-    }
-
-    return false;
-}
-
-void
-CompTimerTestCallbacks::precallback ()
-{
-    /* Test 2: Adding timers */
-    std::cout << "-= TEST: adding timers and callbacks" << std::endl;
-    timers.push_back (new CompTimer ());
-    timers.front ()->setTimes (100, 110);
-    timers.front ()->setCallback (boost::bind (&CompTimerTestCallbacks::cb, this, 1));
-
-    /* TimeoutHandler::timers should be empty */
-    if (!TimeoutHandler::Default ()->timers ().empty ())
-    {
-	std::cout << "FAIL: timers list is not empty" << std::endl;
-	exit (1);
-    }
-
-    timers.push_back (new CompTimer ());
-    timers.back ()->setTimes (50, 90);
-    timers.back ()->setCallback (boost::bind (&CompTimerTestCallbacks::cb, this, 2));
-
-    /* Start both timers */
-    timers.front ()->start ();
-    timers.back ()->start ();
-
-    /* TimeoutHandler::timers should have the timer that
-     * is going to trigger first at the front of the
-     * list and the last timer at the back */
-    if (TimeoutHandler::Default ()->timers ().front () != timers.back ())
-    {
-	std::cout << "FAIL: timer with the least time is not at the front" << std::endl;
-	std::cout << "INFO: TimeoutHandler::Default ().size " << TimeoutHandler::Default ()->timers ().size () << std::endl;
-
-	std::cout << "INFO: TimeoutHandler::Default ().front->minLeft " << TimeoutHandler::Default ()->timers ().front ()->minLeft () << std::endl << \
-	"INFO: TimeoutHandler::Default ().front->maxLeft " << TimeoutHandler::Default ()->timers ().front ()->maxLeft () << std::endl << \
-	"INFO: TimeoutHandler::Default ().front->minTime " << TimeoutHandler::Default ()->timers ().front ()->minTime () << std::endl << \
-	"INFO: TimeoutHandler::Default ().front->maxTime " << TimeoutHandler::Default ()->timers ().front ()->maxTime () << std::endl;
-
-	std::cout << "INFO: TimeoutHandler::Default ().back->minLeft " << TimeoutHandler::Default ()->timers ().back ()->minLeft () <<  std::endl << \
-	"INFO: TimeoutHandler::Default ().back->maxLeft " << TimeoutHandler::Default ()->timers ().back ()->maxLeft () << std::endl << \
-	"INFO: TimeoutHandler::Default ().back->minTime " << TimeoutHandler::Default ()->timers ().back ()->minTime () << std::endl << \
-	"INFO: TimeoutHandler::Default ().back->maxTime " << TimeoutHandler::Default ()->timers ().back ()->maxTime () << std::endl;
-	exit (1);
-    }
-
-    if (TimeoutHandler::Default ()->timers ().back () != timers.front ())
-    {
-	std::cout << "FAIL: timer with the most time is not at the back" << std::endl;
-	exit (1);
-    }
+#include <ctime>
+#include <pthread.h>
+#include <boost/noncopyable.hpp>
+
+using ::testing::InSequence;
+using ::testing::Invoke;
+using ::testing::_;
+using ::testing::AtLeast;
+
+namespace
+{
+
+class CompTimerTestCallbackDispatchTable
+{
+public:
+
+    CompTimerTestCallbackDispatchTable () {};
+    virtual ~CompTimerTestCallbackDispatchTable () {};
+
+    virtual bool callback1 (unsigned int num) = 0;
+    virtual bool callback2 (unsigned int num) = 0;
+    virtual bool callback3 (unsigned int num) = 0;
+protected:
+
+    
+};
+
+class MockCompTimerTestCallbackDispatchTable :
+    public CompTimerTestCallbackDispatchTable,
+    boost::noncopyable
+{
+public:
+
+    static const unsigned int MaxAllowedCalls = 10;
+
+    MOCK_METHOD1 (callback1, bool (unsigned int));
+    MOCK_METHOD1 (callback2, bool (unsigned int));
+    MOCK_METHOD1 (callback3, bool (unsigned int));
+
+    MockCompTimerTestCallbackDispatchTable (const Glib::RefPtr <Glib::MainLoop> &ml) :
+	CompTimerTestCallbackDispatchTable (),
+	mMainLoop (ml)
+    {
+	memset (&mCallsCounter, 0, sizeof (mCallsCounter));
+	ON_CALL (*this, callback1 (_)).WillByDefault (Invoke (this, &MockCompTimerTestCallbackDispatchTable::QuitIfLast));
+	ON_CALL (*this, callback2 (_)).WillByDefault (Invoke (this, &MockCompTimerTestCallbackDispatchTable::QuitIfLast));
+	ON_CALL (*this, callback3 (_)).WillByDefault (Invoke (this, &MockCompTimerTestCallbackDispatchTable::QuitIfLast));
+    };
+
+    void setMax (unsigned int timerId, int maxCalls)
+    {
+	mCallsCounter[timerId].maxCalls = maxCalls;
+    }
+
+private:
+    Glib::RefPtr <Glib::MainLoop> mMainLoop;
+
+    class _counter
+    {
+	public:
+	    unsigned int calls;
+	    int maxCalls;
+    } mCallsCounter[3];
+
+    bool QuitIfLast (unsigned int num)
+    {
+	mCallsCounter[num].calls++;
+
+	if (mCallsCounter[num].maxCalls < 0 ||
+	    static_cast <unsigned int> (mCallsCounter[num].maxCalls) == mCallsCounter[num].calls)
+	{
+	    /* We are the last timer, quit the main loop */
+	    if (TimeoutHandler::Default ()->timers ().size () == 0)
+	    {
+		mMainLoop->quit ();
+		return false;
+	    }
+	    else if (mCallsCounter[num].maxCalls)
+		return false;
+	}
+
+	return true;
+    };
+};
+
+class CompTimerTestCallback: public CompTimerTest
+{
+public:
+    CompTimerTestCallback () :
+	mLastAdded (0),
+	mDispatchTable (new MockCompTimerTestCallbackDispatchTable (ml))
+    {
+    }
+
+    ~CompTimerTestCallback ()
+    {
+	delete mDispatchTable;
+    }
+protected:
+
+    unsigned int    mLastAdded;
+    MockCompTimerTestCallbackDispatchTable *mDispatchTable;
+
+    static void * runThread (void * cb)
+    {
+	if (cb == NULL)
+	{
+	    return NULL;
+	}
+	static_cast <CompTimerTestCallback *> (cb)->ml->run ();
+	return NULL;
+    }
+
+    pthread_t mMainLoopThread;
+
+    void AddTimer (unsigned int min,
+		   unsigned int max,
+		   const boost::function <bool ()> &callback,
+		   int maxAllowedCalls)
+    {
+	timers.push_back (new CompTimer ());
+	timers.back ()->setTimes (min, max);
+	timers.back ()->setCallback (callback);
+
+	ASSERT_FALSE (callback.empty ());
+
+	/* TimeoutHandler::timers should be empty */
+	EXPECT_TRUE (TimeoutHandler::Default ()->timers ().empty ()) << "timers list is not empty";
+
+	mDispatchTable->setMax (mLastAdded, maxAllowedCalls);
+
+	mLastAdded++;
+    }
+
+    void Run ()
+    {
+	for (std::deque <CompTimer *>::iterator it = timers.begin ();
+	     it != timers.end (); it++)
+	    (*it)->start ();
+
+	/* TimeoutHandler::timers should have the timer that
+	 * is going to trigger first at the front of the
+	 * list and the last timer at the back */
+	if (TimeoutHandler::Default ()->timers ().front () != timers.back ())
+	{
+	    RecordProperty ("TimeoutHandler::Default ().size",
+		    TimeoutHandler::Default ()->timers ().size ());
+	    RecordProperty ("TimeoutHandler::Default ().front->minLeft",
+		    TimeoutHandler::Default ()->timers ().front ()->minLeft());
+	    RecordProperty ("TimeoutHandler::Default ().front->maxLeft",
+		    TimeoutHandler::Default ()->timers ().front ()->maxLeft());
+	    RecordProperty ("TimeoutHandler::Default ().front->minTime",
+		    TimeoutHandler::Default ()->timers ().front ()->minTime());
+	    RecordProperty ("TimeoutHandler::Default ().front->maxTime",
+		    TimeoutHandler::Default ()->timers ().front ()->maxTime());
+	    RecordProperty ("TimeoutHandler::Default ().back->minLeft",
+		    TimeoutHandler::Default ()->timers ().back ()->minLeft());
+	    RecordProperty ("TimeoutHandler::Default ().back->maxLeft",
+		    TimeoutHandler::Default ()->timers ().back ()->maxLeft());
+	    RecordProperty ("TimeoutHandler::Default ().back->minTime",
+		    TimeoutHandler::Default ()->timers ().back ()->minTime());
+	    RecordProperty ("TimeoutHandler::Default ().back->maxTime",
+		    TimeoutHandler::Default ()->timers ().back ()->maxTime());
+	    FAIL () << "timer with the least time is not at the front";
+	}
+
+	if (TimeoutHandler::Default ()->timers ().back () != timers.front ())
+	{
+	    FAIL () << "timer with the most time is not at the back";
+	}
+
+	ASSERT_EQ (0,
+		   pthread_create (&mMainLoopThread, NULL,
+				   CompTimerTestCallback::runThread, this));
+
+	pthread_join (mMainLoopThread, NULL);
+    }
+
+    void SetUp ()
+    {
+	CompTimerTest::SetUp ();
+
+	::sleep (1);
+    }
+
+    void TearDown ()
+    {
+	CompTimerTest::TearDown ();
+    }
+};
+
+TEST_F (CompTimerTestCallback, TimerOrder)
+{
+    AddTimer (100, 110, boost::bind (&MockCompTimerTestCallbackDispatchTable::callback1, mDispatchTable, 0), 10);
+    AddTimer (50, 90, boost::bind (&MockCompTimerTestCallbackDispatchTable::callback2, mDispatchTable, 1), 10);
+    AddTimer (0, 0, boost::bind (&MockCompTimerTestCallbackDispatchTable::callback3, mDispatchTable, 2), 10);
+
+    /* TimeoutHandler::timers should be empty since no timers have started */
+    ASSERT_TRUE (TimeoutHandler::Default ()->timers ().empty ()) << "timers list is not empty";
+
+    InSequence s;
+
+    EXPECT_CALL (*mDispatchTable, callback3 (2)).Times (10);
+    EXPECT_CALL (*mDispatchTable, callback2 (1)).Times (1);
+    EXPECT_CALL (*mDispatchTable, callback1 (0)).Times (1);
+    EXPECT_CALL (*mDispatchTable, callback2 (1)).Times (1);
+    EXPECT_CALL (*mDispatchTable, callback1 (0)).Times (1);
+    EXPECT_CALL (*mDispatchTable, callback2 (1)).Times (1);
+    EXPECT_CALL (*mDispatchTable, callback1 (0)).Times (1);
+    EXPECT_CALL (*mDispatchTable, callback2 (1)).Times (1);
+    EXPECT_CALL (*mDispatchTable, callback2 (1)).Times (1);
+    EXPECT_CALL (*mDispatchTable, callback1 (0)).Times (1);
+    EXPECT_CALL (*mDispatchTable, callback2 (1)).Times (1);
+    EXPECT_CALL (*mDispatchTable, callback1 (0)).Times (1);
+    EXPECT_CALL (*mDispatchTable, callback2 (1)).Times (1);
+    EXPECT_CALL (*mDispatchTable, callback1 (0)).Times (1);
+    EXPECT_CALL (*mDispatchTable, callback2 (1)).Times (1);
+    EXPECT_CALL (*mDispatchTable, callback2 (1)).Times (1);
+    EXPECT_CALL (*mDispatchTable, callback1 (0)).Times (1);
+    EXPECT_CALL (*mDispatchTable, callback2 (1)).Times (1);
+    EXPECT_CALL (*mDispatchTable, callback1 (0)).Times (1);
+    EXPECT_CALL (*mDispatchTable, callback1 (0)).Times (1);
+    EXPECT_CALL (*mDispatchTable, callback1 (0)).Times (1);
+
+    Run ();
+}
+
+TEST_F (CompTimerTestCallback, NoZeroStarvation)
+{
+    AddTimer (100, 110, boost::bind (&MockCompTimerTestCallbackDispatchTable::callback1, mDispatchTable, 0), 1);
+    AddTimer (50, 90, boost::bind (&MockCompTimerTestCallbackDispatchTable::callback2, mDispatchTable, 1), 1);
+    AddTimer (0, 0, boost::bind (&MockCompTimerTestCallbackDispatchTable::callback3, mDispatchTable, 2), -1);
+
+    EXPECT_CALL (*mDispatchTable, callback3 (2)).Times (AtLeast (1));
+    EXPECT_CALL (*mDispatchTable, callback2 (1)).Times (1);
+    EXPECT_CALL (*mDispatchTable, callback1 (0)).Times (1);
+
+    Run ();
+}
+
 }

=== renamed directory 'tests/timer/diffs' => 'src/timer/tests/diffs'
=== added directory 'src/timer/tests/diffs/src'
=== added file 'src/timer/tests/diffs/src/test-timer-diffs.cpp'
--- src/timer/tests/diffs/src/test-timer-diffs.cpp	1970-01-01 00:00:00 +0000
+++ src/timer/tests/diffs/src/test-timer-diffs.cpp	2011-12-19 07:06:22 +0000
@@ -0,0 +1,117 @@
+/*
+ * Copyright © 2011 Canonical Ltd.
+ *
+ * Permission to use, copy, modify, distribute, and sell this software
+ * and its documentation for any purpose is hereby granted without
+ * fee, provided that the above copyright notice appear in all copies
+ * and that both that copyright notice and this permission notice
+ * appear in supporting documentation, and that the name of
+ * Canonical Ltd. not be used in advertising or publicity pertaining to
+ * distribution of the software without specific, written prior permission.
+ * Canonical Ltd. makes no representations about the suitability of this
+ * software for any purpose. It is provided "as is" without express or
+ * implied warranty.
+ *
+ * CANONICAL, LTD. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
+ * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
+ * NO EVENT SHALL CANONICAL, LTD. BE LIABLE FOR ANY SPECIAL, INDIRECT OR
+ * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
+ * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
+ * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
+ * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+ * Authored by: Sam Spilsbury <sam.spilsbury@canonical.com>
+ */
+
+#include "test-timer.h"
+
+#include <pthread.h>
+
+class CompTimerTestDiffs: public CompTimerTest
+{
+protected:
+
+    static void* run (void* cb)
+    {
+	if (cb == NULL)
+	{
+	    return NULL;
+	}
+	static_cast<CompTimerTestDiffs*>(cb)->ml->run();
+	return NULL;
+    }
+
+    pthread_t mmainLoopThread;
+    std::list<int> mtriggeredTimers;
+
+    bool cb (int timernum, CompTimer* t1, CompTimer* t2, CompTimer* t3)
+    {
+	if (timernum == 1 || timernum == 2 || timernum == 3)
+	{
+	    RecordProperty("executing timer", timernum);
+	    RecordProperty("t1->minLeft", t1->minLeft());
+	    RecordProperty("t1->maxLeft", t1->maxLeft());
+	    RecordProperty("t1->minTime", t1->minTime());
+	    RecordProperty("t1->maxTime", t1->maxTime());
+
+	    RecordProperty("t3->minLeft", t3->minLeft());
+	    RecordProperty("t3->maxLeft", t3->maxLeft());
+	    RecordProperty("t3->minTime", t3->minTime());
+	    RecordProperty("t3->maxTime", t3->maxTime());
+
+	    RecordProperty("t3->minLeft", t3->minLeft());
+	    RecordProperty("t3->maxLeft", t3->maxLeft());
+	    RecordProperty("t3->minTime", t3->minTime());
+	    RecordProperty("t3->maxTime", t3->maxTime());
+
+	}
+
+	return false;
+    }
+
+    void SetUp ()
+    {
+	CompTimerTest::SetUp();
+	mtriggeredTimers.clear();
+
+	CompTimer *t1, *t2, *t3;
+
+	t1 = new CompTimer();
+	t2 = new CompTimer();
+	t3 = new CompTimer();
+
+	timers.push_back(t1);
+	timers.push_back(t2);
+	timers.push_back(t3);
+
+	t1->setCallback(
+		boost::bind(&CompTimerTestDiffs::cb, this, 1, t1, t2, t3));
+	t1->setTimes(1000, 1100);
+	t1->start();
+	t2->setCallback(
+		boost::bind(&CompTimerTestDiffs::cb, this, 2, t1, t2, t3));
+	t2->setTimes(2000, 2100);
+	t2->start();
+	t3->setCallback(
+		boost::bind(&CompTimerTestDiffs::cb, this, 3, t1, t2, t3));
+	t3->setTimes(3000, 3100);
+	t3->start();
+
+	ASSERT_EQ(
+		0,
+		pthread_create(&mmainLoopThread, NULL, CompTimerTestDiffs::run, this));
+
+	::sleep(4);
+    }
+
+    void TearDown ()
+    {
+	ml->quit();
+	pthread_join(mmainLoopThread, NULL);
+
+	CompTimerTest::TearDown();
+    }
+};
+
+TEST_F(CompTimerTestDiffs,TimerDiffs) {}
+

=== renamed directory 'tests/timer/set-values' => 'src/timer/tests/set-values'
=== added directory 'src/timer/tests/set-values/src'
=== renamed file 'tests/timer/set-values/test-timer-set-values.cpp' => 'src/timer/tests/set-values/src/test-timer-set-values.cpp'
--- tests/timer/set-values/test-timer-set-values.cpp	2011-07-06 18:58:47 +0000
+++ src/timer/tests/set-values/src/test-timer-set-values.cpp	2012-01-12 06:48:58 +0000
@@ -24,116 +24,26 @@
  */
 
 #include "test-timer.h"
-
-bool
-CompTimerTestSetValues::cb (int timernum)
-{
-    if (timernum == 3)
-    {
-	ml->quit ();
-	std::cout << "PASS: testing values" << std::endl;
-    }
-    return false;
-}
-
-void
-CompTimerTestSetValues::precallback ()
-{
-    CompTimer      *t1, *t2, *t3;
-
-    std::cout << "-= TEST: testing values" << std::endl;
-
-    t1 = new CompTimer ();
+#include <unistd.h>
+
+TEST_F(CompTimerTest, TimerSetValues)
+{
+    CompTimer* t1 = new CompTimer ();
 
     t1->setTimes (100, 90);
-    t1->setCallback (boost::bind (&CompTimerTestSetValues::cb, this, 1));
     t1->start ();
 
-    if (t1->minTime () != 100)
-    {
-	std::cout << "FAIL: min time was not the min value passed" << std::endl;
-	exit (1);
-    }
-
-    if (t1->maxTime () != 100)
-    {
-	std::cout << "FAIL: max time was not the min value passed" << std::endl;
-	exit (1);
-    }
-
-    if (t1->minLeft () != 100)
-    {
-	std::cout << "FAIL: min left was not the min value passed " << std::endl;
-	exit (1);
-    }
-
-    if (t1->maxLeft () != 100)
-    {
-	std::cout << "FAIL: max left was not the min value passed" << std::endl;
-	exit (1);
-    }
-
-    t2 = new CompTimer ();
-
-    t2->setTimes (100, 110);
-    t2->setCallback (boost::bind (&CompTimerTestSetValues::cb, this, 2));
-    t2->start ();
-
-    if (t2->minTime () != 100)
-    {
-	std::cout << "FAIL: min time was not the min value passed" << std::endl;
-	exit (1);
-    }
-
-    if (t2->maxTime () != 110)
-    {
-	std::cout << "FAIL: max time was not the max value passed" << std::endl;
-	exit (1);
-    }
-
-    if (t2->minLeft () != 100)
-    {
-	std::cout << "FAIL: min left was not the min value passed " << std::endl;
-	exit (1);
-    }
-
-    if (t2->maxLeft () != 110)
-    {
-	std::cout << "FAIL: max left was not the max value passed" << std::endl;
-	exit (1);
-    }
-
-    t3 = new CompTimer ();
-
-    t3->setTimes (100);
-    t3->setCallback (boost::bind (&CompTimerTestSetValues::cb, this, 3));
-    t3->start ();
-
-    if (t3->minTime () != 100)
-    {
-	std::cout << "FAIL: min time was not the value passed" << std::endl;
-	exit (1);
-    }
-
-    if (t3->maxTime () != 100)
-    {
-	std::cout << "FAIL: max time was not the value passed" << std::endl;
-	exit (1);
-    }
-
-    if (t3->minLeft () != 100)
-    {
-	std::cout << "FAIL: min left was not the value passed" << t3->minLeft () << std::endl;
-	exit (1);
-    }
-
-    if (t3->maxLeft () != 100)
-    {
-	std::cout << "FAIL: max left was not the value passed" << std::endl;
-	exit (1);
-    }
-
-    timers.push_back (t1);
-    timers.push_back (t2);
-    timers.push_back (t3);
+    usleep (100000);
+
+    /* minLeft and maxLeft are now
+     * real-time, so wait the guarunteed
+     * expiry time in order to check them
+     * for an accurate value of zero */
+
+    ASSERT_EQ (t1->minTime(), 100);
+    ASSERT_EQ (t1->maxTime(), 100);
+    ASSERT_EQ (t1->minLeft(), 0);
+    ASSERT_EQ (t1->maxLeft(), 0);
+
+    delete t1;
 }

=== renamed file 'tests/timer/test-timer.cpp' => 'src/timer/tests/test-timer.cpp'
--- tests/timer/test-timer.cpp	2011-07-06 18:58:47 +0000
+++ src/timer/tests/test-timer.cpp	2012-01-12 06:48:58 +0000
@@ -26,36 +26,26 @@
 #include "test-timer.h"
 
 CompTimerTest::CompTimerTest () :
-    mc (Glib::MainContext::get_default ()),
-    ml (Glib::MainLoop::create (mc, false)),
-    ts (CompTimeoutSource::create (mc)),
-    lastTimerTriggered (0)
+	mc(Glib::MainContext::get_default()),
+		ml(Glib::MainLoop::create(mc, false)),
+		ts(CompTimeoutSource::create(mc)),
+		lastTimerTriggered(0)
 {
 }
 
 CompTimerTest::~CompTimerTest ()
 {
-    while (timers.size ())
+    while (timers.size())
     {
-	CompTimer *t = timers.front ();
+	CompTimer *t = timers.front();
 
-	timers.pop_front ();
+	timers.pop_front();
 	delete t;
     }
 }
 
-int
-main (int argc, char **argv)
+void CompTimerTest::SetUp ()
 {
-    CompTimerTest  *ctt = static_cast <CompTimerTest *> (new OBJECT ());
-    TimeoutHandler *th = new TimeoutHandler ();
-    TimeoutHandler::SetDefault (th);
-
-    ctt->precallback ();
-    ctt->ml->run ();
-
-    delete ctt;
-    delete th;
-
-    return 0;
+    TimeoutHandler *th = new TimeoutHandler();
+    TimeoutHandler::SetDefault(th);
 }

=== renamed file 'tests/timer/test-timer.h' => 'src/timer/tests/test-timer.h'
--- tests/timer/test-timer.h	2011-07-06 18:58:47 +0000
+++ src/timer/tests/test-timer.h	2012-01-12 06:48:58 +0000
@@ -26,64 +26,33 @@
 #ifndef _COMPIZ_TEST_TIMER_H
 #define _COMPIZ_TEST_TIMER_H
 
+#include <gtest/gtest.h>
+#include <gmock/gmock.h>
+
 #include <glibmm/main.h>
 #include <core/timer.h>
 #include <privatetimeouthandler.h>
 #include <privatetimeoutsource.h>
 #include <iostream>
 #include <boost/bind.hpp>
+#include <boost/shared_ptr.hpp>
+#include <deque>
 
-class CompTimerTest
+class CompTimerTest : public ::testing::Test
 {
 public:
 
     CompTimerTest ();
     virtual ~CompTimerTest ();
 
+    virtual void SetUp();
+
     Glib::RefPtr <Glib::MainContext> mc;
     Glib::RefPtr <Glib::MainLoop> ml;
     Glib::RefPtr <CompTimeoutSource> ts;
-    std::list <CompTimer *> timers;
-
-    virtual void precallback () = 0;
+    std::deque <CompTimer *> timers;
 
     int lastTimerTriggered;
 };
 
-class CompTimerTestCallbacks :
-    public CompTimerTest
-{
-public:
-
-    void precallback ();
-    bool cb (int timernum);
-};
-
-class CompTimerTestDiffs :
-    public CompTimerTest
-{
-public:
-
-    void precallback ();
-    bool cb (int timernum, CompTimer *t1, CompTimer *t2, CompTimer *t3);
-};
-
-class CompTimerTestSetValues :
-    public CompTimerTest
-{
-public:
-
-    void precallback ();
-    bool cb (int timernum);
-};
-
-class CompTimerTestSetCalling :
-    public CompTimerTest
-{
-public:
-
-    void precallback ();
-    bool cb (int timernum, CompTimer *t1, CompTimer *t2, CompTimer *t3);
-};
-
 #endif

=== renamed directory 'tests/timer/while-calling' => 'src/timer/tests/while-calling'
=== added directory 'src/timer/tests/while-calling/src'
=== renamed file 'tests/timer/while-calling/test-timer-set-times-while-calling.cpp' => 'src/timer/tests/while-calling/src/test-timer-set-times-while-calling.cpp'
--- tests/timer/while-calling/test-timer-set-times-while-calling.cpp	2011-07-06 18:58:47 +0000
+++ src/timer/tests/while-calling/src/test-timer-set-times-while-calling.cpp	2012-01-12 06:48:58 +0000
@@ -25,100 +25,118 @@
 
 #include "test-timer.h"
 
-bool
-CompTimerTestSetCalling::cb (int timernum, CompTimer *t1, CompTimer *t2, CompTimer *t3)
-{
-    std::cout << "INFO: triggering timer " << timernum << std::endl;
-
-    if (lastTimerTriggered == 0 && timernum == 1)
-    {
-	/* Change the timeout time of the second timer to be after the third */
-	t2->setTimes (4000, 4100);
-
-	/* Check if it is now at the back of the timeout list */
-	if (TimeoutHandler::Default ()->timers ().back () != t2)
-	{
-	    std::cout << "FAIL: timer with higher timeout time is not at the back of the list" << std::endl;
-
-	    for (std::list <CompTimer *>::iterator it = TimeoutHandler::Default ()->timers ().begin ();
-		 it != TimeoutHandler::Default ()->timers ().end (); it++)
-	    {
-		CompTimer *t = (*it);
-		std::cout << "INFO: t->minLeft " << t->minLeft () << std::endl << \
-			     "INFO: t->maxLeft " << t->maxLeft () << std::endl << \
-			     "INFO: t->minTime " << t->minTime () << std::endl << \
-			     "INFO: t->maxTime " << t->maxTime () << std::endl;
-	    }
-
-	    exit (1);
-	}
-    }
-    else if (lastTimerTriggered == 1 && timernum == 2)
-    {
-	std::cout << "FAIL: timer with a higher timeout time got triggered before a timer with a lower timeout time" << std::endl;
-
-	for (std::list <CompTimer *>::iterator it = TimeoutHandler::Default ()->timers ().begin ();
-	     it != TimeoutHandler::Default ()->timers ().end (); it++)
-	{
-	    CompTimer *t = (*it);
-	    std::cout << "INFO: t->minLeft " << t->minLeft () << std::endl << \
-			 "INFO: t->maxLeft " << t->maxLeft () << std::endl << \
-			 "INFO: t->minTime " << t->minTime () << std::endl << \
-			 "INFO: t->maxTime " << t->maxTime () << std::endl;
-	}
-
-	exit (1);
-    }
-    else if (lastTimerTriggered == 2 && timernum != 1)
-    {
-	std::cout << "FAIL: timer with higher timeout time didn't get triggered after a lower timeout time" << std::endl;
-
-	for (std::list <CompTimer *>::iterator it = TimeoutHandler::Default ()->timers ().begin ();
-	     it != TimeoutHandler::Default ()->timers ().end (); it++)
-	{
-	    CompTimer *t = (*it);
-	    std::cout << "INFO: t->minLeft " << t->minLeft () << std::endl << \
-			 "INFO: t->maxLeft " << t->maxLeft () << std::endl << \
-			 "INFO: t->minTime " << t->minTime () << std::endl << \
-			 "INFO: t->maxTime " << t->maxTime () << std::endl;
-	}
-
-	exit (1);
-    }
-
-    lastTimerTriggered = timernum;
-
-    if (timernum == 2)
-    {
-	std::cout << "PASS: retiming" << std::endl;
-	ml->quit ();
-    }
-
-    return false;
-}
-
-void
-CompTimerTestSetCalling::precallback ()
-{
-    CompTimer      *t1, *t2, *t3;
-
-    std::cout << "-= TEST: changing timeout time" << std::endl;
-
-    t1 = new CompTimer ();
-    t2 = new CompTimer ();
-    t3 = new CompTimer ();
-
-    timers.push_back (t1);
-    timers.push_back (t2);
-    timers.push_back (t3);
-
-    t1->setCallback (boost::bind (&CompTimerTestSetCalling::cb, this, 1, t1, t2, t3));
-    t1->setTimes (1000, 1100);
-    t1->start ();
-    t2->setCallback (boost::bind (&CompTimerTestSetCalling::cb, this, 2, t1, t2, t3));
-    t2->setTimes (2000, 2100);
-    t2->start ();
-    t3->setCallback (boost::bind (&CompTimerTestSetCalling::cb, this, 3, t1, t2, t3));
-    t3->setTimes (3000, 3100);
-    t3->start ();
+#include <pthread.h>
+
+class CompTimerTestSetTimes: public CompTimerTest
+{
+protected:
+
+    int mlastTimerTriggered;
+
+    static void* run (void* cb)
+    {
+	if (cb == NULL)
+	{
+	    return NULL;
+	}
+	static_cast<CompTimerTestSetTimes*>(cb)->ml->run();
+	return NULL;
+    }
+
+    pthread_t mmainLoopThread;
+    std::list<int> mtriggeredTimers;
+
+    void recordTimers ()
+    {
+	for (std::list<CompTimer *>::iterator it =
+		TimeoutHandler::Default()->timers().begin();
+		it != TimeoutHandler::Default()->timers().end(); it++)
+	{
+	    CompTimer *t = (*it);
+	    RecordProperty("t->minLeft", t->minLeft());
+	    RecordProperty("t->maxLeft", t->maxLeft());
+	    RecordProperty("t->minTime", t->minTime());
+	    RecordProperty("t->maxTime", t->maxTime());
+	}
+    }
+
+    bool cb (int timernum, CompTimer* t1, CompTimer* t2, CompTimer* t3) {
+	cb_(timernum,t1,t2,t3);
+	return(true);
+    }
+    void cb_ (int timernum, CompTimer* t1, CompTimer* t2, CompTimer* t3)
+    {
+	recordTimers();
+	if (mlastTimerTriggered == 0 && timernum == 1)
+	{
+	    /* Change the timeout time of the second timer to be after the third */
+	    t2->setTimes(4000, 4100);
+
+	    recordTimers();
+
+	    /* Check if it is now at the back of the timeout list */
+	    ASSERT_EQ( TimeoutHandler::Default()->timers().back(), t2 );
+	}
+	else if (mlastTimerTriggered == 1 && timernum == 2)
+	{
+	    recordTimers();
+	    FAIL() << "timer with a higher timeout time got triggered "
+		    "before a timer with a lower timeout time";
+	}
+	else if (mlastTimerTriggered == 2 && timernum != 1)
+	{
+
+	    recordTimers();
+	    FAIL() << "timer with higher timeout time didn't get "
+	       	    "triggered after a lower timeout time";
+	}
+
+	mlastTimerTriggered = timernum;
+    }
+
+    void SetUp ()
+    {
+	CompTimerTest::SetUp();
+	mlastTimerTriggered = 0;
+	CompTimer *t1, *t2, *t3;
+
+	t1 = new CompTimer();
+	t2 = new CompTimer();
+	t3 = new CompTimer();
+
+	timers.push_back(t1);
+	timers.push_back(t2);
+	timers.push_back(t3);
+
+	t1->setCallback(
+		boost::bind(&CompTimerTestSetTimes::cb, this, 1, t1, t2, t3));
+	t1->setTimes(1000, 1100);
+	t1->start();
+	t2->setCallback(
+		boost::bind(&CompTimerTestSetTimes::cb, this, 2, t1, t2, t3));
+	t2->setTimes(2000, 2100);
+	t2->start();
+	t3->setCallback(
+		boost::bind(&CompTimerTestSetTimes::cb, this, 3, t1, t2, t3));
+	t3->setTimes(3000, 3100);
+	t3->start();
+
+	ASSERT_EQ(
+		0,
+		pthread_create(&mmainLoopThread, NULL, CompTimerTestSetTimes::run, this));
+    }
+
+    void TearDown ()
+    {
+	ml->quit();
+	pthread_join(mmainLoopThread, NULL);
+
+	CompTimerTest::TearDown();
+    }
+};
+
+TEST_F(CompTimerTestSetTimes, SetTimesWhileCalling)
+{
+    ::sleep(4);
+    // Just a dummy forcing instantiation of fixture.
 }

=== modified file 'src/window.cpp'
--- src/window.cpp	2011-11-07 02:27:18 +0000
+++ src/window.cpp	2012-01-12 06:48:58 +0000
@@ -23,8 +23,6 @@
  * Author: David Reveman <davidr@novell.com>
  */
 
-#include <compiz.h>
-
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #include <X11/Xproto.h>
@@ -1532,7 +1530,7 @@
     XConfigureEvent xev;
     XWindowAttributes attrib;
     unsigned int      nchildren;
-    Window            rootRet, parentRet;
+    Window            rootRet, parentRet = 0;
     Window            *children;
 
     xev.type   = ConfigureNotify;
@@ -1554,25 +1552,9 @@
 	xev.border_width = attrib.border_width;
 	xev.above = None;
 
-	if (priv->frame)
-	{
-	    XWindowAttributes fAttrib;
-	    XWindowAttributes wAttrib;
-
-	    /* Add offset between wrapper and client */
-	    if (XGetWindowAttributes (screen->dpy (), priv->wrapper, &wAttrib))
-	    {
-		xev.x += wAttrib.x;
-		xev.y += wAttrib.y;
-	    }
-
-	    /* Add offset between frame and client */
-	    if (XGetWindowAttributes (screen->dpy (), priv->frame, &fAttrib))
-	    {
-		xev.x += fAttrib.x;
-		xev.y += fAttrib.y;
-	    }
-	}
+	/* Translate co-ordinates to root space */
+	XTranslateCoordinates (screen->dpy (), priv->id, screen->root (), 0, 0,
+			       &xev.x, &xev.y, &parentRet);
 
 	/* We need to ensure that the stacking order is
 	 * based on the current server stacking order so
@@ -6149,14 +6131,14 @@
     if (screen->vpSize ().width () != 1)
     {
 	x += screen->vp ().x () * screen->width ();
-	x = MOD (x, vWidth);
+	x = compiz::core::screen::wraparound_mod (x, vWidth);
 	x -= screen->vp ().x () * screen->width ();
     }
 
     if (screen->vpSize ().height () != 1)
     {
 	y += screen->vp ().y () * screen->height ();
-	y = MOD (y, vHeight);
+	y = compiz::core::screen::wraparound_mod (y, vHeight);
 	y -= screen->vp ().y () * screen->height ();
     }
 
@@ -6966,31 +6948,42 @@
 	priv->border.top    != b->top ||
 	priv->border.bottom != b->bottom)
     {
-	unsigned long data[4];
-
 	priv->serverInput = *i;
 	priv->border = *b;
 
 	recalcActions ();
 
-	/* Use b for _NET_WM_FRAME_EXTENTS here because
-	 * that is the representation of the actual decoration
-	 * around the window that the user sees and should
-	 * be used for placement and such */
-
-	data[0] = b->left;
-	data[1] = b->right;
-	data[2] = b->top;
-	data[3] = b->bottom;
-
-	XChangeProperty (screen->dpy (), priv->id,
-			 Atoms::frameExtents,
-			 XA_CARDINAL, 32, PropModeReplace,
-			 (unsigned char *) data, 4);
-
 	priv->updateSize ();
 	priv->updateFrameWindow ();
     }
+
+    /* Use b for _NET_WM_FRAME_EXTENTS here because
+     * that is the representation of the actual decoration
+     * around the window that the user sees and should
+     * be used for placement and such */
+
+    /* Also update frame extents regardless of whether or not
+     * the frame extents actually changed, eg, a plugin could
+     * suggest that a window has no frame extents and that it
+     * might later get frame extents - this is mandatory if we
+     * say that we support it, so set them
+     * additionaly some applications might request frame extents
+     * and we must respond by setting the property - ideally
+     * this should only ever be done when some plugin actually
+     * need to change the frame extents or the applications
+     * requested it */
+
+    unsigned long data[4];
+
+    data[0] = b->left;
+    data[1] = b->right;
+    data[2] = b->top;
+    data[3] = b->bottom;
+
+    XChangeProperty (screen->dpy (), priv->id,
+		     Atoms::frameExtents,
+		     XA_CARDINAL, 32, PropModeReplace,
+		     (unsigned char *) data, 4);
 }
 
 bool

=== removed directory 'tests'
=== removed file 'tests/CMakeLists.txt'
--- tests/CMakeLists.txt	2011-07-06 19:52:01 +0000
+++ tests/CMakeLists.txt	1970-01-01 00:00:00 +0000
@@ -1,2 +0,0 @@
-add_subdirectory (pluginclasshandler)
-add_subdirectory (timer)

=== removed file 'tests/pluginclasshandler/construct/CMakeLists.txt'
--- tests/pluginclasshandler/construct/CMakeLists.txt	2011-07-06 19:52:01 +0000
+++ tests/pluginclasshandler/construct/CMakeLists.txt	1970-01-01 00:00:00 +0000
@@ -1,25 +0,0 @@
-link_directories (${COMPIZ_TEST_PCH_LINK_DIRS})
-
-include_directories (${COMPIZ_TEST_PCH_INCLUDE_DIRS}
-		     ${compiz_SOURCE_DIR}/include
-		     ${compiz_BINARY_DIR}
-		     ${compiz_BINARY_DIR}/generated
-		     ${compiz_SOURCE_DIR}/src
-		     ${COMPIZ_TEST_PCH_BASE_DIR})
-
-set (OBJECT CompizPCHTestConstruct)
-add_definitions (-DOBJECT=${OBJECT})
-
-add_executable (test-pch-construct
-		test-pch-construct.cpp
-		../test-pluginclasshandler.cpp 
-	        ${compiz_SOURCE_DIR}/src/valueholder.cpp
-	        ${compiz_SOURCE_DIR}/src/pluginclasses.cpp
-		${compiz_SOURCE_DIR}/src/string.cpp
-		${compiz_SOURCE_DIR}/src/logmessage.cpp)
-
-target_link_libraries (test-pch-construct
-		       ${COMPIZ_TEST_PCH_LIBRARIES})
-
-add_test (test-pch-construct
-	  ${CMAKE_CURRENT_BINARY_DIR}/test-pch-construct)

=== removed file 'tests/pluginclasshandler/get/CMakeLists.txt'
--- tests/pluginclasshandler/get/CMakeLists.txt	2011-07-06 19:52:01 +0000
+++ tests/pluginclasshandler/get/CMakeLists.txt	1970-01-01 00:00:00 +0000
@@ -1,25 +0,0 @@
-link_directories (${COMPIZ_TEST_PCH_LINK_DIRS})
-
-include_directories (${COMPIZ_TEST_PCH_INCLUDE_DIRS}
-		     ${compiz_SOURCE_DIR}/include
-		     ${compiz_BINARY_DIR}
-		     ${compiz_BINARY_DIR}/generated
-		     ${compiz_SOURCE_DIR}/src
-		     ${COMPIZ_TEST_PCH_BASE_DIR})
-
-set (OBJECT CompizPCHTestGet)
-add_definitions (-DOBJECT=${OBJECT})
-
-add_executable (test-pch-get
-		test-pch-get.cpp
-		../test-pluginclasshandler.cpp 
-	        ${compiz_SOURCE_DIR}/src/valueholder.cpp
-	        ${compiz_SOURCE_DIR}/src/pluginclasses.cpp
-		${compiz_SOURCE_DIR}/src/string.cpp
-		${compiz_SOURCE_DIR}/src/logmessage.cpp)
-
-target_link_libraries (test-pch-get
-		       ${COMPIZ_TEST_PCH_LIBRARIES})
-
-add_test (test-pch-get
-	  ${CMAKE_CURRENT_BINARY_DIR}/test-pch-get)

=== removed file 'tests/pluginclasshandler/indexes/CMakeLists.txt'
--- tests/pluginclasshandler/indexes/CMakeLists.txt	2011-07-06 19:52:01 +0000
+++ tests/pluginclasshandler/indexes/CMakeLists.txt	1970-01-01 00:00:00 +0000
@@ -1,28 +0,0 @@
-link_directories (${COMPIZ_TEST_PCH_LINK_DIRS})
-
-include_directories (${COMPIZ_TEST_PCH_INCLUDE_DIRS}
-		     ${compiz_SOURCE_DIR}/include
-		     ${compiz_BINARY_DIR}
-		     ${compiz_BINARY_DIR}/generated
-		     ${compiz_SOURCE_DIR}/src
-		     ${COMPIZ_TEST_PCH_BASE_DIR})
-
-set (OBJECT CompizPCHTestIndexes)
-add_definitions (-DOBJECT=${OBJECT})
-# hack
-add_definitions (-Dprivate=public)
-add_definitions (-Dprotected=public)
-
-add_executable (test-pch-indexes
-		test-pch-indexes.cpp
-		../test-pluginclasshandler.cpp 
-	        ${compiz_SOURCE_DIR}/src/valueholder.cpp
-	        ${compiz_SOURCE_DIR}/src/pluginclasses.cpp
-		${compiz_SOURCE_DIR}/src/string.cpp
-		${compiz_SOURCE_DIR}/src/logmessage.cpp)
-
-target_link_libraries (test-pch-indexes
-		       ${COMPIZ_TEST_PCH_LIBRARIES})
-
-add_test (test-pch-indexes
-	  ${CMAKE_CURRENT_BINARY_DIR}/test-pch-indexes)

=== removed file 'tests/pluginclasshandler/typenames/CMakeLists.txt'
--- tests/pluginclasshandler/typenames/CMakeLists.txt	2011-07-06 19:52:01 +0000
+++ tests/pluginclasshandler/typenames/CMakeLists.txt	1970-01-01 00:00:00 +0000
@@ -1,25 +0,0 @@
-link_directories (${COMPIZ_TEST_PCH_LINK_DIRS})
-
-include_directories (${COMPIZ_TEST_PCH_INCLUDE_DIRS}
-		     ${compiz_SOURCE_DIR}/include
-		     ${compiz_BINARY_DIR}
-		     ${compiz_BINARY_DIR}/generated
-		     ${compiz_SOURCE_DIR}/src
-		     ${COMPIZ_TEST_PCH_BASE_DIR})
-
-set (OBJECT CompizPCHTestTypenames)
-add_definitions (-DOBJECT=${OBJECT})
-
-add_executable (test-pch-typenames
-		test-pch-typenames.cpp
-		../test-pluginclasshandler.cpp 
-	        ${compiz_SOURCE_DIR}/src/valueholder.cpp
-	        ${compiz_SOURCE_DIR}/src/pluginclasses.cpp
-		${compiz_SOURCE_DIR}/src/string.cpp
-		${compiz_SOURCE_DIR}/src/logmessage.cpp)
-
-target_link_libraries (test-pch-typenames
-		       ${COMPIZ_TEST_PCH_LIBRARIES})
-
-add_test (test-pch-typenames
-	  ${CMAKE_CURRENT_BINARY_DIR}/test-pch-typenames)

=== removed directory 'tests/timer'
=== removed file 'tests/timer/callbacks/CMakeLists.txt'
--- tests/timer/callbacks/CMakeLists.txt	2011-07-06 18:58:47 +0000
+++ tests/timer/callbacks/CMakeLists.txt	1970-01-01 00:00:00 +0000
@@ -1,23 +0,0 @@
-link_directories (${COMPIZ_TEST_TIMER_LINK_DIRS})
-
-include_directories (${COMPIZ_TEST_TIMER_INCLUDE_DIRS}
-		     ${compiz_SOURCE_DIR}/include
-		     ${compiz_BINARY_DIR}
-		     ${compiz_BINARY_DIR}/generated
-		     ${compiz_SOURCE_DIR}/src
-		     ${COMPIZ_TIMER_TEST_BASE_DIR})
-
-set (OBJECT CompTimerTestCallbacks)
-add_definitions (-DOBJECT=${OBJECT})
-
-add_executable (test-timer-callbacks
-		test-timer-callbacks.cpp
-		../test-timer.cpp 
-		${compiz_SOURCE_DIR}/src/timer.cpp
-		${compiz_SOURCE_DIR}/src/timeouthandler.cpp)
-
-target_link_libraries (test-timer-callbacks
-		       ${COMPIZ_TEST_TIMER_LIBRARIES})
-
-add_test (test-timer-callbacks
-	  ${CMAKE_CURRENT_BINARY_DIR}/test-timer-callbacks)

=== removed file 'tests/timer/diffs/CMakeLists.txt'
--- tests/timer/diffs/CMakeLists.txt	2011-07-06 18:58:47 +0000
+++ tests/timer/diffs/CMakeLists.txt	1970-01-01 00:00:00 +0000
@@ -1,23 +0,0 @@
-link_directories (${COMPIZ_TEST_TIMER_LINK_DIRS})
-
-include_directories (${COMPIZ_TEST_TIMER_INCLUDE_DIRS}
-		     ${compiz_SOURCE_DIR}/include
-		     ${compiz_BINARY_DIR}
-		     ${compiz_BINARY_DIR}/generated
-		     ${compiz_SOURCE_DIR}/src
-		     ${COMPIZ_TIMER_TEST_BASE_DIR})
-
-set (OBJECT CompTimerTestDiffs)
-add_definitions (-DOBJECT=${OBJECT})
-
-add_executable (test-timer-diffs
-		test-timer-diffs.cpp
-		../test-timer.cpp 
-		${compiz_SOURCE_DIR}/src/timer.cpp
-		${compiz_SOURCE_DIR}/src/timeouthandler.cpp)
-
-target_link_libraries (test-timer-diffs
-		       ${COMPIZ_TEST_TIMER_LIBRARIES})
-
-add_test (test-timer-diffs
-	  ${CMAKE_CURRENT_BINARY_DIR}/test-timer-diffs)

=== removed file 'tests/timer/diffs/test-timer-diffs.cpp'
--- tests/timer/diffs/test-timer-diffs.cpp	2011-07-06 18:58:47 +0000
+++ tests/timer/diffs/test-timer-diffs.cpp	1970-01-01 00:00:00 +0000
@@ -1,82 +0,0 @@
-/*
- * Copyright © 2011 Canonical Ltd.
- *
- * Permission to use, copy, modify, distribute, and sell this software
- * and its documentation for any purpose is hereby granted without
- * fee, provided that the above copyright notice appear in all copies
- * and that both that copyright notice and this permission notice
- * appear in supporting documentation, and that the name of
- * Canonical Ltd. not be used in advertising or publicity pertaining to
- * distribution of the software without specific, written prior permission.
- * Canonical Ltd. makes no representations about the suitability of this
- * software for any purpose. It is provided "as is" without express or
- * implied warranty.
- *
- * CANONICAL, LTD. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
- * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
- * NO EVENT SHALL CANONICAL, LTD. BE LIABLE FOR ANY SPECIAL, INDIRECT OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
- * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
- * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
- * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * Authored by: Sam Spilsbury <sam.spilsbury@canonical.com>
- */
-
-#include "test-timer.h"
-
-bool
-CompTimerTestDiffs::cb (int timernum, CompTimer *t1, CompTimer *t2, CompTimer *t3)
-{
-    if (timernum == 1 || timernum == 2 || timernum == 3)
-    {
-	std::cout << "INFO: executing timer " << timernum << std::endl;
-
-	std::cout << "INFO: t1->minLeft " << t1->minLeft () << std::endl << \
-		     "INFO: t1->maxLeft " << t1->maxLeft () << std::endl << \
-		     "INFO: t1->minTime " << t1->minTime () << std::endl << \
-		     "INFO: t1->maxTime " << t1->maxTime () << std::endl;
-
-	std::cout << "INFO: t2->minLeft " << t2->minLeft () << std::endl << \
-		     "INFO: t2->maxLeft " << t2->maxLeft () << std::endl << \
-		     "INFO: t2->minTime " << t2->minTime () << std::endl << \
-		     "INFO: t2->maxTime " << t2->maxTime () << std::endl;
-
-	std::cout << "INFO: t3->minLeft " << t3->minLeft () << std::endl << \
-		     "INFO: t3->maxLeft " << t3->maxLeft () << std::endl << \
-		     "INFO: t3->minTime " << t3->minTime () << std::endl << \
-		     "INFO: t3->maxTime " << t3->maxTime () << std::endl;
-    }
-
-    if (timernum == 3)
-	ml->quit ();
-
-    std::cout << "return false" << std::endl;
-    return false;
-}
-
-void
-CompTimerTestDiffs::precallback ()
-{
-    CompTimer      *t1, *t2, *t3;
-
-    std::cout << "-= TEST: checking timeout diff" << std::endl;
-
-    t1 = new CompTimer ();
-    t2 = new CompTimer ();
-    t3 = new CompTimer ();
-
-    timers.push_back (t1);
-    timers.push_back (t2);
-    timers.push_back (t3);
-
-    t1->setCallback (boost::bind (&CompTimerTestDiffs::cb, this, 1, t1, t2, t3));
-    t1->setTimes (1000, 1100);
-    t1->start ();
-    t2->setCallback (boost::bind (&CompTimerTestDiffs::cb, this, 2, t1, t2, t3));
-    t2->setTimes (2000, 2100);
-    t2->start ();
-    t3->setCallback (boost::bind (&CompTimerTestDiffs::cb, this, 3, t1, t2, t3));
-    t3->setTimes (3000, 3100);
-    t3->start ();
-}

=== removed file 'tests/timer/set-values/CMakeLists.txt'
--- tests/timer/set-values/CMakeLists.txt	2011-07-06 18:58:47 +0000
+++ tests/timer/set-values/CMakeLists.txt	1970-01-01 00:00:00 +0000
@@ -1,23 +0,0 @@
-link_directories (${COMPIZ_TEST_TIMER_LINK_DIRS})
-
-include_directories (${COMPIZ_TEST_TIMER_INCLUDE_DIRS}
-		     ${compiz_SOURCE_DIR}/include
-		     ${compiz_BINARY_DIR}
-		     ${compiz_BINARY_DIR}/generated
-		     ${compiz_SOURCE_DIR}/src
-		     ${COMPIZ_TIMER_TEST_BASE_DIR})
-
-set (OBJECT CompTimerTestSetValues)
-add_definitions (-DOBJECT=${OBJECT})
-
-add_executable (test-timer-set-values
-		test-timer-set-values.cpp
-		../test-timer.cpp 
-		${compiz_SOURCE_DIR}/src/timer.cpp
-		${compiz_SOURCE_DIR}/src/timeouthandler.cpp)
-
-target_link_libraries (test-timer-set-values
-		       ${COMPIZ_TEST_TIMER_LIBRARIES})
-
-add_test (test-timer-set-values
-	  ${CMAKE_CURRENT_BINARY_DIR}/test-timer-set-values)

=== removed file 'tests/timer/while-calling/CMakeLists.txt'
--- tests/timer/while-calling/CMakeLists.txt	2011-07-06 18:58:47 +0000
+++ tests/timer/while-calling/CMakeLists.txt	1970-01-01 00:00:00 +0000
@@ -1,23 +0,0 @@
-link_directories (${COMPIZ_TEST_TIMER_LINK_DIRS})
-
-include_directories (${COMPIZ_TEST_TIMER_INCLUDE_DIRS}
-		     ${compiz_SOURCE_DIR}/include
-		     ${compiz_BINARY_DIR}
-		     ${compiz_BINARY_DIR}/generated
-		     ${compiz_SOURCE_DIR}/src
-		     ${COMPIZ_TIMER_TEST_BASE_DIR})
-
-set (OBJECT CompTimerTestSetCalling)
-add_definitions (-DOBJECT=${OBJECT})
-
-add_executable (test-timer-set-times-while-calling
-		test-timer-set-times-while-calling.cpp
-		../test-timer.cpp 
-		${compiz_SOURCE_DIR}/src/timer.cpp
-		${compiz_SOURCE_DIR}/src/timeouthandler.cpp)
-
-target_link_libraries (test-timer-set-times-while-calling
-		       ${COMPIZ_TEST_TIMER_LIBRARIES})
-
-add_test (test-timer-set-times-while-calling
-	  ${CMAKE_CURRENT_BINARY_DIR}/test-timer-set-times-while-calling)

